@@ -1,5 +1,4 @@
 /************************** BEGIN dsp-optimizer.h **************************/
-
 /************************************************************************
     FAUST Architecture File
     Copyright (C) 2016-2020 GRAME, Centre National de Creation Musicale
@@ -27,8 +26,7 @@
 #ifndef __dsp_optimizer__
 #define __dsp_optimizer__
 
-#include <iostream>
-#include <sstream>
+#include <stdio.h>
 #include <sys/types.h>
 #include <pwd.h>
 #include <unistd.h>
@@ -41,7 +39,7 @@
     A class to find optimal Faust compiler parameters for a given DSP.
 */
 template <typename REAL>
-class dsp_optimizer {
+class dsp_optimizer_real {
 
     private:
     
@@ -59,6 +57,9 @@ class dsp_optimizer {
         bool fTrace;
         bool fControl;
         bool fNeedExp10;
+        int fDownSampling;
+        int fUpSampling;
+        int fFilter;
     
         std::string fFilename;
         std::string fInput;
@@ -71,16 +72,18 @@ class dsp_optimizer {
         {
             // First call with fCount = -1 will be used to estimate fCount by giving the wanted measure duration
             if (fCount == -1) {
-                measure_dsp_aux<REAL> mes(fDSP, fBufferSize, 5., fTrace, fControl);
+                measure_dsp_real<REAL> mes(fDSP, fBufferSize, 5., fTrace, fControl, fDownSampling, fUpSampling, fFilter);
                 mes.measure();
                 // fCount is kept from the first duration measure
                 fCount = mes.getCount();
                 return mes.getStats();
             } else {
-                measure_dsp_aux<REAL> mes(fDSP, fBufferSize, fCount, fTrace, fControl);
+                measure_dsp_real<REAL> mes(fDSP, fBufferSize, fCount, fTrace, fControl, fDownSampling, fUpSampling, fFilter);
                 for (int i = 0; i < run; i++) {
                     mes.measure();
-                    if (fTrace) std::cout << mes.getStats() << " " << "(DSP CPU % : " << (mes.getCPULoad() * 100) << ")" << std::endl;
+                    if (fTrace) {
+                        fprintf(stdout, "%f MBytes/sec (DSP CPU %% : %f at %d Hz)\n", mes.getStats(), (mes.getCPULoad() * 100), int(BENCH_SAMPLE_RATE));
+                    }
                     FAUSTBENCH_LOG<double>(mes.getStats());
                 }
                 return mes.getStats();
@@ -197,9 +200,9 @@ class dsp_optimizer {
         void printItem(const std::vector <std::string>& item)
         {
             for (int i = 0; i < item.size(); i++) {
-                std::cout << " " << item[i];
+                fprintf(stdout, " %s", item[i].c_str());
             }
-            std::cout << " : ";
+            fprintf(stdout, " : ");
         }
     
         std::vector <std::string> addArgvItems(const std::vector <std::string>& item, int argc, const char* argv[])
@@ -227,13 +230,13 @@ class dsp_optimizer {
             }
             
             if (!fFactory) {
-                std::cerr << "Cannot create factory : " << fError;
+                fprintf(stderr, "Cannot create factory : %s\n", fError.c_str());
                 return false;
             }
             
             fDSP = fFactory->createDSPInstance();
             if (!fDSP) {
-                std::cerr << "Cannot create instance..." << std::endl;
+                fprintf(stderr, "Cannot create instance...\n");
                 return false;
             }
             
@@ -258,7 +261,7 @@ class dsp_optimizer {
                 if (computeOne(addArgvItems(options[i], fArgc, fArgv), fRun, res)) {
                     table_res.push_back(std::make_pair(i, res));
                 } else {
-                    std::cerr << "computeOne error..." << std::endl;
+                    fprintf(stderr, "computeOne error...\n");
                 }
             }
             
@@ -268,13 +271,19 @@ class dsp_optimizer {
 
         static bool compareFun(std::pair<int, double> i, std::pair<int, double> j) { return (i.second > j.second); }
     
-        bool init(const std::string& filename, const std::string input,
-                  int argc, const char* argv[],
+        bool init(const std::string& filename,
+                  const std::string& input,
+                  int argc,
+                  const char* argv[],
                   const std::string& target,
-                  int buffer_size, int run,
+                  int buffer_size,
+                  int run,
                   int opt_level_max,
                   bool trace,
-                  bool control)
+                  bool control,
+                  int ds,
+                  int us,
+                  int filter)
         {
             fFilename = filename;
             fInput = input;
@@ -288,19 +297,22 @@ class dsp_optimizer {
             fTrace = trace;
             fControl = control;
             fNeedExp10 = false;
+            fDownSampling = ds;
+            fUpSampling = us;
+            fFilter = filter;
             
             init();
             
-            if (fTrace) std::cout << "Estimate timing parameters" << std::endl;
+            if (fTrace) fprintf(stdout, "Estimate timing parameters\n");
             double res1 = 0.;
             if (!computeOne(addArgvItems(fOptionsTable[0], fArgc, fArgv), 1, res1)) {
-                std::cerr << "computeOne error..." << std::endl;
+                fprintf(stderr, "computeOne error...\n");
                 return false;
             }
-            if (fTrace) std::cout << "Testing -exp10 need" << std::endl;
+            if (fTrace) fprintf(stdout, "Testing -exp10 need\n");
             double res2 = 0.;
             if (!computeOne(addArgvItems(fOptionsTable[1], fArgc, fArgv), 1, res2)) {
-                std::cerr << "computeOne error..." << std::endl;
+                fprintf(stderr, "computeOne error...\n");
                 return false;
             }
             fNeedExp10 = (res2 > (res1 * 1.05)); // If more than 5% faster
@@ -321,47 +333,30 @@ class dsp_optimizer {
          * @param opt_level - LLVM IR to IR optimization level (from -1 to 4, -1 means 'maximum possible value'
          * @param trace - whether to log the trace
          * @param control - whether to activate random changes of all control values at each cycle
+         * @param ds - downsampling factor
+         * @param us - upsampling factor
+         * @param filter - filter type
          * since the maximum value may change with new LLVM versions)
          */
-        dsp_optimizer(const char* filename,
-                      int argc,
-                      const char* argv[],
-                      const std::string& target,
-                      int buffer_size,
-                      int run = 1,
-                      int opt_level = -1,
-                      bool trace = true,
-                      bool control = false)
+        dsp_optimizer_real(const char* filename,
+                           int argc,
+                           const char* argv[],
+                           const std::string& target,
+                           int buffer_size,
+                           int run = 1,
+                           int opt_level = -1,
+                           bool trace = true,
+                           bool control = false,
+                           int ds = 0,
+                           int us = 0,
+                           int filter = 0)
         {
-            if (!init(filename, "", argc, argv, target, buffer_size, run, opt_level, trace, control)) {
+            if (!init(filename, "", argc, argv, target, buffer_size, run, opt_level, trace, control, ds, us, filter)) {
                 throw std::bad_alloc();
             }
         }
     
-        /**
-         * Constructor.
-         *
-         * @param input - the Faust program as a string
-         * @param argc - the number of parameters in argv array
-         * @param argv - the array of parameters
-         * @param target - the LLVM machine target (using empty string will take current machine settings)
-         * @param buffer_size - the buffer size in sampels
-         * @param opt_level - LLVM IR to IR optimization level (from -1 to 4, -1 means 'maximum possible value'
-         * since the maximum value may change with new LLVM versions)
-         */
-        dsp_optimizer(const std::string& input,
-                      int argc,
-                      const char* argv[],
-                      const std::string& target,
-                      int buffer_size,
-                      int opt_level = -1)
-        {
-            if (!init("", input, argc, argv, target, buffer_size, opt_level)) {
-                throw std::bad_alloc();
-            }
-        }
-    
-        virtual ~dsp_optimizer()
+        virtual ~dsp_optimizer_real()
         {}
     
         /**
@@ -371,10 +366,10 @@ class dsp_optimizer {
          */
         std::pair<double, std::vector<std::string> > findOptimizedParameters()
         {
-            if (fTrace) std::cout << "Discover best parameters option" << std::endl;
+            if (fTrace) fprintf(stdout, "Discover best parameters option\n");
             std::pair<double, std::vector<std::string> > best1 = findOptimizedParametersAux(fOptionsTable);
             
-            if (fTrace) std::cout << "Refined with -mcd" << std::endl;
+            if (fTrace) fprintf(stdout, "Refined with -mcd\n");
             std::vector<std::vector <std::string> > options_table;
             for (int size = 2; size <= 256; size *= 2) {
                 std::vector<std::string> best2 = best1.second;
@@ -384,7 +379,7 @@ class dsp_optimizer {
             }
             
             if (fNeedExp10) {
-                if (fTrace) std::cout << "Use -exp10" << std::endl;
+                if (fTrace) fprintf(stdout, "Use -exp10\n");
                 std::vector<std::string> t0_exp10;
                 t0_exp10.push_back("-exp10");
                 options_table.push_back(t0_exp10);
@@ -402,5 +397,31 @@ class dsp_optimizer {
     
 };
 
+class dsp_optimizer : public dsp_optimizer_real<FAUSTFLOAT> {
+
+    public:
+    
+        dsp_optimizer(const char* filename,
+                       int argc,
+                       const char* argv[],
+                       const std::string& target,
+                       int buffer_size,
+                       int run = 1,
+                       int opt_level = -1,
+                       bool trace = true,
+                       bool control = false,
+                       int ds = 0,
+                       int us = 0,
+                       int filter = 0)
+        :dsp_optimizer_real<FAUSTFLOAT>(filename, argc, argv,
+                                        target, buffer_size,
+                                        run, opt_level,
+                                        trace, control,
+                                        ds, us, filter)
+        {}
+    
+};
+    
 #endif
-/**************************  END  dsp-optimizer.h **************************/
+
+/************************** END dsp-optimizer.h **************************/
