@@ -39,15 +39,19 @@
 
 #include "faust/midi/midi.h"
 #include "faust/dsp/dsp-combiner.h"
+#include "faust/dsp/dsp-adapter.h"
+#include "faust/dsp/proxy-dsp.h"
+
+#include "faust/gui/DecoratorUI.h"
 #include "faust/gui/GUI.h"
 #include "faust/gui/MapUI.h"
-#include "faust/dsp/proxy-dsp.h"
 #include "faust/gui/JSONControl.h"
 
-#define kActiveVoice      0
-#define kFreeVoice        -1
-#define kReleaseVoice     -2
-#define kNoVoice          -3
+#define kActiveVoice    0
+#define kFreeVoice     -1
+#define kReleaseVoice  -2
+#define kLegatoVoice   -3
+#define kNoVoice       -4
 
 #define VOICE_STOP_LEVEL  0.0005    // -70 db
 #define MIX_BUFFER_SIZE   4096
@@ -55,9 +59,7 @@
 /**
  * Allows to control zones in a grouped manner.
  */
-
-class GroupUI : public GUI, public PathBuilder
-{
+class GroupUI : public GUI, public PathBuilder {
 
     private:
 
@@ -150,7 +152,6 @@ class GroupUI : public GUI, public PathBuilder
 /**
  * One voice of polyphony.
  */
-
 struct dsp_voice : public MapUI, public decorator_dsp {
     
     typedef std::function<double(int)> TransformFunction;
@@ -159,17 +160,21 @@ struct dsp_voice : public MapUI, public decorator_dsp {
     {
         return 440.0 * std::pow(2.0, (note-69.0)/12.0);
     }
-
-    int fNote;                          // Playing note actual pitch
+    
+    int fCurNote;                       // Playing note pitch
+    int fNextNote;                      // In kLegatoVoice state, next note to play
+    int fNextVel;                       // In kLegatoVoice state, next velocity to play
     int fDate;                          // KeyOn date
     int fRelease;                       // Current number of samples used in release mode to detect end of note
-    int fMaxRelease;                    // Max of samples used in release mode to detect end of note
     FAUSTFLOAT fLevel;                  // Last audio block level
     std::vector<std::string> fGatePath; // Paths of 'gate' control
     std::vector<std::string> fGainPath; // Paths of 'gain/vel|velocity' control
     std::vector<std::string> fFreqPath; // Paths of 'freq/key' control
     TransformFunction        fKeyFun;   // MIDI key to freq conversion function
     TransformFunction        fVelFun;   // MIDI velocity to gain conversion function
+    
+    FAUSTFLOAT** fInputsSlice;
+    FAUSTFLOAT** fOutputsSlice;
  
     dsp_voice(dsp* dsp):decorator_dsp(dsp)
     {
@@ -177,20 +182,51 @@ struct dsp_voice : public MapUI, public decorator_dsp {
         fVelFun = [](int velocity) { return double(velocity)/127.0; };
         fKeyFun = [](int pitch) { return midiToFreq(pitch); };
         dsp->buildUserInterface(this);
-        fNote = kFreeVoice;
+        fCurNote = kFreeVoice;
+        fNextNote = fNextVel = -1;
         fLevel = FAUSTFLOAT(0);
-        fDate = 0;
-        fRelease = 0;
-        fMaxRelease = dsp->getSampleRate()/2; // One 1/2 sec used in release mode to detect end of note
+        fDate = fRelease = 0;
         extractPaths(fGatePath, fFreqPath, fGainPath);
     }
     virtual ~dsp_voice()
     {}
+    
+    void computeSlice(int offset, int slice, FAUSTFLOAT** inputs, FAUSTFLOAT** outputs)
+    {
+        FAUSTFLOAT** inputsSlice = static_cast<FAUSTFLOAT**>(alloca(sizeof(FAUSTFLOAT*) * getNumInputs()));
+        for (int chan = 0; chan < getNumInputs(); chan++) {
+            inputsSlice[chan] = &(inputs[chan][offset]);
+        }
+        FAUSTFLOAT** outputsSlice = static_cast<FAUSTFLOAT**>(alloca(sizeof(FAUSTFLOAT*) * getNumOutputs()));
+        for (int chan = 0; chan < getNumOutputs(); chan++) {
+            outputsSlice[chan] = &(outputs[chan][offset]);
+        }
+        compute(slice, inputsSlice, outputsSlice);
+    }
+    
+    void computeLegato(int count, FAUSTFLOAT** inputs, FAUSTFLOAT** outputs)
+    {
+        int slice = count/2;
+        
+        // Reset envelops
+        for (size_t i = 0; i < fGatePath.size(); i++) {
+            setParamValue(fGatePath[i], FAUSTFLOAT(0));
+        }
+        
+        // Compute current voice on half buffer
+        computeSlice(0, slice, inputs, outputs);
+         
+        // Start next keyOn
+        keyOn(fNextNote, fNextVel);
+        
+        // Compute on second half buffer
+        computeSlice(slice, slice, inputs, outputs);
+    }
 
     void extractPaths(std::vector<std::string>& gate, std::vector<std::string>& freq, std::vector<std::string>& gain)
     {
         // Keep gain/vel|velocity, freq/key and gate labels
-        for (auto& it : getMap()) {
+        for (const auto& it : getMap()) {
             std::string path = it.first;
             if (endsWith(path, "/gate")) {
                 gate.push_back(path);
@@ -209,19 +245,30 @@ struct dsp_voice : public MapUI, public decorator_dsp {
             }
         }
     }
-   
-    // MIDI velocity [0..127]
-    void keyOn(int pitch, int velocity)
+ 
+    void instanceClear()
     {
-        keyOn(pitch, fVelFun(velocity));
+        decorator_dsp::instanceClear();
+        fCurNote = kFreeVoice;
+        fNextNote = fNextVel = -1;
+        fLevel = FAUSTFLOAT(0);
+        fDate = fRelease = 0;
+    }
+    
+    // Keep 'pitch' and 'velocity' to fadeOut the current voice and start next one in the next buffer
+    void keyOn(int pitch, int velocity, bool legato = false)
+    {
+        if (legato) {
+            fNextNote = pitch;
+            fNextVel = velocity;
+        } else {
+            keyOn(pitch, fVelFun(velocity));
+        }
     }
 
     // Normalized MIDI velocity [0..1]
     void keyOn(int pitch, double velocity)
     {
-        // So that DSP state is always re-initialized
-        fDSP->instanceClear();
-        
         for (size_t i = 0; i < fFreqPath.size(); i++) {
             setParamValue(fFreqPath[i], fKeyFun(pitch));
         }
@@ -232,7 +279,7 @@ struct dsp_voice : public MapUI, public decorator_dsp {
             setParamValue(fGainPath[i], velocity);
         }
         
-        fNote = pitch;
+        fCurNote = pitch;
     }
 
     void keyOff(bool hard = false)
@@ -244,11 +291,11 @@ struct dsp_voice : public MapUI, public decorator_dsp {
         
         if (hard) {
             // Immediately stop voice
-            fNote = kFreeVoice;
+            fCurNote = kFreeVoice;
         } else {
             // Release voice
-            fRelease = fMaxRelease;
-            fNote = kReleaseVoice;
+            fRelease = fDSP->getSampleRate()/2; // Half sec used in release mode to detect end of note
+            fCurNote = kReleaseVoice;
         }
     }
 
@@ -257,7 +304,6 @@ struct dsp_voice : public MapUI, public decorator_dsp {
 /**
  * A group of voices.
  */
-
 struct dsp_voice_group {
 
     GroupUI fGroups;
@@ -323,7 +369,7 @@ struct dsp_voice_group {
             ui_interface->closeBox();
 
             // If not grouped, also add individual voices UI
-            if (!fGroupControl) {
+            if (!fGroupControl || dynamic_cast<SoundUIInterface*>(ui_interface)) {
                 for (size_t i = 0; i < fVoiceTable.size(); i++) {
                     char buffer[32];
                     snprintf(buffer, 32, ((fVoiceTable.size() < 8) ? "Voice%ld" : "V%ld"), long(i+1));
@@ -342,9 +388,8 @@ struct dsp_voice_group {
 };
 
 /**
- * Base class for MIDI controllable DSP.
+ * Base class for MIDI controllable polyphonic DSP.
  */
-
 #ifdef EMCC
 #include "faust/gui/JSONUI.h"
 #include "faust/gui/MidiUI.h"
@@ -357,14 +402,14 @@ class dsp_poly : public decorator_dsp, public midi, public JSONControl {
     #ifdef EMCC
         MapUI fMapUI;
         std::string fJSON;
-        midi_handler fMIDIHandler;
+        midi_handler fMidiHandler;
         MidiUI fMIDIUI;
     #endif
     
     public:
     
     #ifdef EMCC
-        dsp_poly(dsp* dsp):decorator_dsp(dsp), fMIDIUI(&fMIDIHandler)
+        dsp_poly(dsp* dsp):decorator_dsp(dsp), fMIDIUI(&fMidiHandler)
         {
             JSONUI jsonui(getNumInputs(), getNumOutputs());
             buildUserInterface(&jsonui);
@@ -451,10 +496,6 @@ class dsp_poly : public decorator_dsp, public midi, public JSONControl {
             midi::progChange(channel, pgm);
         }
     
-        // Group API
-        virtual void setGroup(bool group) {}
-        virtual bool getGroup() { return false; }
-
 };
 
 /**
@@ -463,15 +504,27 @@ class dsp_poly : public decorator_dsp, public midi, public JSONControl {
  * All voices are preallocated by cloning the single DSP voice given at creation time.
  * Dynamic voice allocation is done in 'getFreeVoice'
  */
-
 class mydsp_poly : public dsp_voice_group, public dsp_poly {
 
     private:
 
         FAUSTFLOAT** fMixBuffer;
         FAUSTFLOAT** fOutBuffer;
+        midi_interface* fMidiHandler; // The midi_interface the DSP is connected to
         int fDate;
-  
+    
+        void fadeOut(int count, FAUSTFLOAT** outBuffer)
+        {
+            // FadeOut on half buffer
+            for (int chan = 0; chan < getNumOutputs(); chan++) {
+                double factor = 1., step = 1./double(count);
+                for (int frame = 0; frame < count; frame++) {
+                    outBuffer[chan][frame] *= factor;
+                    factor -= step;
+                }
+            }
+        }
+    
         FAUSTFLOAT mixCheckVoice(int count, FAUSTFLOAT** mixBuffer, FAUSTFLOAT** outBuffer)
         {
             FAUSTFLOAT level = 0;
@@ -517,7 +570,7 @@ class mydsp_poly : public dsp_voice_group, public dsp_poly {
             int oldest_date_playing = INT_MAX;
             
             for (size_t i = 0; i < fVoiceTable.size(); i++) {
-                if (fVoiceTable[i]->fNote == pitch) {
+                if (fVoiceTable[i]->fCurNote == pitch) {
                     // Keeps oldest playing voice
                     if (fVoiceTable[i]->fDate < oldest_date_playing) {
                         oldest_date_playing = fVoiceTable[i]->fDate;
@@ -529,65 +582,63 @@ class mydsp_poly : public dsp_voice_group, public dsp_poly {
             return voice_playing;
         }
     
+        int allocVoice(int voice, int type)
+        {
+            fVoiceTable[voice]->fDate++;
+            fVoiceTable[voice]->fCurNote = type;
+            return voice;
+        }
+    
         // Always returns a voice
         int getFreeVoice()
         {
-            int voice = kNoVoice;
-            
             // Looks for the first available voice
             for (size_t i = 0; i < fVoiceTable.size(); i++) {
-                if (fVoiceTable[i]->fNote == kFreeVoice) {
-                    voice = int(i);
-                    goto result;
+                if (fVoiceTable[i]->fCurNote == kFreeVoice) {
+                    return allocVoice(i, kActiveVoice);
                 }
             }
 
-            {
-                // Otherwise steal one
-                int voice_release = kNoVoice;
-                int voice_playing = kNoVoice;
-                
-                int oldest_date_release = INT_MAX;
-                int oldest_date_playing = INT_MAX;
+            // Otherwise steal one
+            int voice_release = kNoVoice;
+            int voice_playing = kNoVoice;
+            int oldest_date_release = INT_MAX;
+            int oldest_date_playing = INT_MAX;
 
-                // Scan all voices
-                for (size_t i = 0; i < fVoiceTable.size(); i++) {
-                    if (fVoiceTable[i]->fNote == kReleaseVoice) {
-                        // Keeps oldest release voice
-                        if (fVoiceTable[i]->fDate < oldest_date_release) {
-                            oldest_date_release = fVoiceTable[i]->fDate;
-                            voice_release = int(i);
-                        }
-                    } else {
-                        // Otherwise keeps oldest playing voice
-                        if (fVoiceTable[i]->fDate < oldest_date_playing) {
-                            oldest_date_playing = fVoiceTable[i]->fDate;
-                            voice_playing = int(i);
-                        }
+            // Scan all voices
+            for (size_t i = 0; i < fVoiceTable.size(); i++) {
+                if (fVoiceTable[i]->fCurNote == kReleaseVoice) {
+                    // Keeps oldest release voice
+                    if (fVoiceTable[i]->fDate < oldest_date_release) {
+                        oldest_date_release = fVoiceTable[i]->fDate;
+                        voice_release = int(i);
+                    }
+                } else {
+                    // Otherwise keeps oldest playing voice
+                    if (fVoiceTable[i]->fDate < oldest_date_playing) {
+                        oldest_date_playing = fVoiceTable[i]->fDate;
+                        voice_playing = int(i);
                     }
                 }
-            
-                // Then decide which one to steal
-                if (oldest_date_release != INT_MAX) {
-                    std::cout << "Steal release voice : voice_date " << fVoiceTable[voice_release]->fDate;
-                    std::cout << " cur_date = " << fDate << " voice = " << voice_release << std::endl;
-                    voice = voice_release;
-                    goto result;
-                } else if (oldest_date_playing != INT_MAX) {
-                    std::cout << "Steal playing voice : voice_date " << fVoiceTable[voice_playing]->fDate;
-                    std::cout << " cur_date = " << fDate << " voice = " << voice_playing << std::endl;
-                    voice = voice_playing;
-                    goto result;
-                } else {
-                    assert(false);
-                    return kNoVoice;
-                }
             }
-            
-        result:
-            fVoiceTable[voice]->fDate = fDate++;
-            fVoiceTable[voice]->fNote = kActiveVoice;
-            return voice;
+        
+            // Then decide which one to steal
+            if (oldest_date_release != INT_MAX) {
+                fprintf(stderr, "Steal release voice : voice_date = %d cur_date = %d voice = %d \n",
+                        fVoiceTable[voice_release]->fDate,
+                        fDate,
+                        voice_release);
+                return allocVoice(voice_release, kLegatoVoice);
+            } else if (oldest_date_playing != INT_MAX) {
+                fprintf(stderr, "Steal playing voice : voice_date = %d cur_date = %d voice = %d \n",
+                        fVoiceTable[voice_playing]->fDate,
+                        fDate,
+                        voice_release);
+                return allocVoice(voice_playing, kLegatoVoice);
+            } else {
+                assert(false);
+                return kNoVoice;
+            }
         }
 
         static void panic(FAUSTFLOAT val, void* arg)
@@ -602,7 +653,7 @@ class mydsp_poly : public dsp_voice_group, public dsp_poly {
             if (fVoiceTable.size() > 0) {
                 return true;
             } else {
-                std::cout << "DSP is not polyphonic...\n";
+                fprintf(stderr, "DSP is not polyphonic...\n");
                 return false;
             }
         }
@@ -619,7 +670,6 @@ class mydsp_poly : public dsp_voice_group, public dsp_poly {
          * @param group - if true, voices are not individually accessible, a global "Voices" tab will automatically dispatch
          *                a given control on all voices, assuming GUI::updateAllGuis() is called.
          *                If false, all voices can be individually controlled.
-         *                setGroup/getGroup methods can be used to set/get the group state.
          *
          */
         mydsp_poly(dsp* dsp,
@@ -629,6 +679,7 @@ class mydsp_poly : public dsp_voice_group, public dsp_poly {
         : dsp_voice_group(panic, this, control, group), dsp_poly(dsp) // dsp parameter is deallocated by ~dsp_poly
         {
             fDate = 0;
+            fMidiHandler = nullptr;
 
             // Create voices
             assert(nvoices > 0);
@@ -649,18 +700,25 @@ class mydsp_poly : public dsp_voice_group, public dsp_poly {
 
         virtual ~mydsp_poly()
         {
+            // Remove from fMidiHandler
+            if (fMidiHandler) fMidiHandler->removeMidiIn(this);
             for (int chan = 0; chan < getNumOutputs(); chan++) {
                 delete[] fMixBuffer[chan];
                 delete[] fOutBuffer[chan];
             }
             delete[] fMixBuffer;
             delete[] fOutBuffer;
+            
         }
 
         // DSP API
-    
         void buildUserInterface(UI* ui_interface)
         {
+            // MidiUI ui_interface contains the midi_handler connected to the MIDI driver
+            if (dynamic_cast<midi_interface*>(ui_interface)) {
+                fMidiHandler = dynamic_cast<midi_interface*>(ui_interface);
+                fMidiHandler->addMidiIn(this);
+            }
             dsp_voice_group::buildUserInterface(ui_interface);
         }
 
@@ -731,16 +789,24 @@ class mydsp_poly : public dsp_voice_group, public dsp_poly {
                 // Mix all playing voices
                 for (size_t i = 0; i < fVoiceTable.size(); i++) {
                     dsp_voice* voice = fVoiceTable[i];
-                    if (voice->fNote != kFreeVoice) {
+                    if (voice->fCurNote == kLegatoVoice) {
+                        // Play from current note and next note
+                        voice->computeLegato(count, inputs, fMixBuffer);
+                        // FadeOut on first half buffer
+                        fadeOut(count/2, fMixBuffer);
+                        // Mix it in result
+                        voice->fLevel = mixCheckVoice(count, fMixBuffer, fOutBuffer);
+                    } else if (voice->fCurNote != kFreeVoice) {
+                        // Compute current note
                         voice->compute(count, inputs, fMixBuffer);
                         // Mix it in result
                         voice->fLevel = mixCheckVoice(count, fMixBuffer, fOutBuffer);
                         // Check the level to possibly set the voice in kFreeVoice again
                         voice->fRelease -= count;
-                        if ((voice->fNote == kReleaseVoice)
+                        if ((voice->fCurNote == kReleaseVoice)
                             && (voice->fRelease < 0)
                             && (voice->fLevel < VOICE_STOP_LEVEL)) {
-                            voice->fNote = kFreeVoice;
+                            voice->fCurNote = kFreeVoice;
                         }
                     }
                 }
@@ -772,32 +838,25 @@ class mydsp_poly : public dsp_voice_group, public dsp_poly {
         // Additional polyphonic API
         MapUI* newVoice()
         {
-            int voice = getFreeVoice();
-            // So that DSP state is always re-initialized
-            fVoiceTable[voice]->instanceClear();
-            return fVoiceTable[voice];
+            return fVoiceTable[getFreeVoice()];
         }
 
         void deleteVoice(MapUI* voice)
         {
-            std::vector<dsp_voice*>::iterator it = find(fVoiceTable.begin(), fVoiceTable.end(), reinterpret_cast<dsp_voice*>(voice));
+            auto it = find(fVoiceTable.begin(), fVoiceTable.end(), reinterpret_cast<dsp_voice*>(voice));
             if (it != fVoiceTable.end()) {
                 (*it)->keyOff();
             } else {
-                std::cout << "Voice not found\n";
+                fprintf(stderr, "Voice not found\n");
             }
         }
-    
-        // Group API
-        void setGroup(bool group) { fGroupControl = group; }
-        bool getGroup() { return fGroupControl; }
 
         // MIDI API
         MapUI* keyOn(int channel, int pitch, int velocity)
         {
             if (checkPolyphony()) {
                 int voice = getFreeVoice();
-                fVoiceTable[voice]->keyOn(pitch, velocity);
+                fVoiceTable[voice]->keyOn(pitch, velocity, fVoiceTable[voice]->fCurNote == kLegatoVoice);
                 return fVoiceTable[voice];
             } else {
                 return 0;
@@ -811,7 +870,7 @@ class mydsp_poly : public dsp_voice_group, public dsp_poly {
                 if (voice != kNoVoice) {
                     fVoiceTable[voice]->keyOff();
                 } else {
-                    std::cout << "Playing pitch = " << pitch << " not found\n";
+                    fprintf(stderr, "Playing pitch = %d not found\n", pitch);
                 }
             }
         }
@@ -826,18 +885,19 @@ class mydsp_poly : public dsp_voice_group, public dsp_poly {
 };
 
 /**
- * Polyphonic DSP with an integrated effect. fPolyDSP will respond to MIDI messages.
+ * Polyphonic DSP with an integrated effect.
  */
 class dsp_poly_effect : public dsp_poly {
     
     private:
-        
+    
+        // fPolyDSP will respond to MIDI messages.
         dsp_poly* fPolyDSP;
         
     public:
         
-        dsp_poly_effect(dsp_poly* dsp1, dsp* dsp2)
-        :dsp_poly(dsp2), fPolyDSP(dsp1)
+        dsp_poly_effect(dsp_poly* voice, dsp* combined)
+        :dsp_poly(combined), fPolyDSP(voice)
         {}
         
         virtual ~dsp_poly_effect()
@@ -878,27 +938,38 @@ class dsp_poly_effect : public dsp_poly {
         {
             fPolyDSP->progChange(channel, pgm);
         }
-        
-        // Group API
-        void setGroup(bool group)
-        {
-            fPolyDSP->setGroup(group);
-        }
-        bool getGroup()
-        {
-            return fPolyDSP->getGroup();
-        }
+    
 };
 
 /**
  * Polyphonic DSP factory class. Helper code to support polyphonic DSP source with an integrated effect.
  */
-
 struct dsp_poly_factory : public dsp_factory {
     
     dsp_factory* fProcessFactory;
     dsp_factory* fEffectFactory;
     
+    dsp* adaptDSP(dsp* dsp, bool is_double)
+    {
+        return (is_double) ? new dsp_sample_adapter<double, float>(dsp) : dsp;
+    }
+
+    dsp_poly_factory(dsp_factory* process_factory = nullptr,
+                     dsp_factory* effect_factory = nullptr):
+    fProcessFactory(process_factory)
+    ,fEffectFactory(effect_factory)
+    {}
+
+    virtual ~dsp_poly_factory()
+    {}
+
+    virtual std::string getName() { return fProcessFactory->getName(); }
+    virtual std::string getSHAKey() { return fProcessFactory->getSHAKey(); }
+    virtual std::string getDSPCode() { return fProcessFactory->getDSPCode(); }
+    virtual std::string getCompileOptions() { return fProcessFactory->getCompileOptions(); }
+    virtual std::vector<std::string> getLibraryList() { return fProcessFactory->getLibraryList(); }
+    virtual std::vector<std::string> getIncludePathnames() { return fProcessFactory->getIncludePathnames(); }
+
     std::string getEffectCode(const std::string& dsp_content)
     {
         std::stringstream effect_code;
@@ -908,22 +979,6 @@ struct dsp_poly_factory : public dsp_factory {
         return effect_code.str();
     }
 
-    dsp_poly_factory(dsp_factory* process_factory = NULL,
-                     dsp_factory* effect_factory = NULL):
-    fProcessFactory(process_factory)
-    ,fEffectFactory(effect_factory)
-    {}
-    
-    virtual ~dsp_poly_factory()
-    {}
-    
-    virtual std::string getName() { return fProcessFactory->getName(); }
-    virtual std::string getSHAKey() { return fProcessFactory->getSHAKey(); }
-    virtual std::string getDSPCode() { return fProcessFactory->getDSPCode(); }
-    virtual std::string getCompileOptions() { return fProcessFactory->getCompileOptions(); }
-    virtual std::vector<std::string> getLibraryList() { return fProcessFactory->getLibraryList(); }
-    virtual std::vector<std::string> getIncludePathnames() { return fProcessFactory->getIncludePathnames(); }
-    
     virtual void setMemoryManager(dsp_memory_manager* manager)
     {
         fProcessFactory->setMemoryManager(manager);
@@ -932,7 +987,7 @@ struct dsp_poly_factory : public dsp_factory {
         }
     }
     virtual dsp_memory_manager* getMemoryManager() { return fProcessFactory->getMemoryManager(); }
-    
+
     /* Create a new polyphonic DSP instance with global effect, to be deleted with C++ 'delete'
      *
      * @param nvoices - number of polyphony voices, should be at least 1
@@ -941,25 +996,26 @@ struct dsp_poly_factory : public dsp_factory {
      * @param group - if true, voices are not individually accessible, a global "Voices" tab will automatically dispatch
      *                a given control on all voices, assuming GUI::updateAllGuis() is called.
      *                If false, all voices can be individually controlled.
+     * @param is_double - if true, internally allocated DSPs will be adapted to receive 'double' samples.
      */
-    dsp_poly* createPolyDSPInstance(int nvoices, bool control, bool group)
+    dsp_poly* createPolyDSPInstance(int nvoices, bool control, bool group, bool is_double = false)
     {
-        dsp_poly* dsp_poly = new mydsp_poly(fProcessFactory->createDSPInstance(), nvoices, control, group);
+        dsp_poly* dsp_poly = new mydsp_poly(adaptDSP(fProcessFactory->createDSPInstance(), is_double), nvoices, control, group);
         if (fEffectFactory) {
             // the 'dsp_poly' object has to be controlled with MIDI, so kept separated from new dsp_sequencer(...) object
-            return new dsp_poly_effect(dsp_poly, new dsp_sequencer(dsp_poly, fEffectFactory->createDSPInstance()));
+            return new dsp_poly_effect(dsp_poly, new dsp_sequencer(dsp_poly, adaptDSP(fEffectFactory->createDSPInstance(), is_double)));
         } else {
             return new dsp_poly_effect(dsp_poly, dsp_poly);
         }
     }
-    
+
     /* Create a new DSP instance, to be deleted with C++ 'delete' */
     dsp* createDSPInstance()
     {
         return fProcessFactory->createDSPInstance();
     }
-    
+
 };
 
 #endif // __poly_dsp__
-/**************************  END  poly-dsp.h **************************/
+/************************** END poly-dsp.h **************************/
