@@ -28,6 +28,7 @@
 #include <vector>
 #include <map>
 #include <string>
+#include <stdio.h>
 
 #include "faust/gui/UI.h"
 #include "faust/gui/PathBuilder.h"
@@ -48,8 +49,6 @@ class MapUI : public UI, public PathBuilder
         // Label zone map
         std::map<std::string, FAUSTFLOAT*> fLabelZoneMap;
     
-        std::string fNullStr = "";
-    
     public:
         
         MapUI() {}
@@ -119,13 +118,15 @@ class MapUI : public UI, public PathBuilder
         virtual void declare(FAUSTFLOAT* zone, const char* key, const char* val)
         {}
         
-        // set/get
+        // setParamValue/getParamValue
         void setParamValue(const std::string& path, FAUSTFLOAT value)
         {
             if (fPathZoneMap.find(path) != fPathZoneMap.end()) {
                 *fPathZoneMap[path] = value;
             } else if (fLabelZoneMap.find(path) != fLabelZoneMap.end()) {
                 *fLabelZoneMap[path] = value;
+            } else {
+                fprintf(stderr, "ERROR : setParamValue '%s' not found\n", path.c_str());
             }
         }
         
@@ -136,7 +137,8 @@ class MapUI : public UI, public PathBuilder
             } else if (fLabelZoneMap.find(path) != fLabelZoneMap.end()) {
                 return *fLabelZoneMap[path];
             } else {
-                return FAUSTFLOAT(0);
+                fprintf(stderr, "ERROR : getParamValue '%s' not found\n", path.c_str());
+                return 0;
             }
         }
     
@@ -145,31 +147,41 @@ class MapUI : public UI, public PathBuilder
         
         int getParamsCount() { return int(fPathZoneMap.size()); }
         
-        const std::string& getParamAddress(int index)
+        std::string getParamAddress(int index)
         {
             if (index < 0 || index > int(fPathZoneMap.size())) {
-                return fNullStr;
+                return "";
             } else {
                 auto it = fPathZoneMap.begin();
                 while (index-- > 0 && it++ != fPathZoneMap.end()) {}
                 return it->first;
             }
         }
-    
-        const std::string& getParamAddress(FAUSTFLOAT* zone)
+        
+        const char* getParamAddress1(int index)
         {
-            for (auto& it : fPathZoneMap) {
+            if (index < 0 || index > int(fPathZoneMap.size())) {
+                return nullptr;
+            } else {
+                auto it = fPathZoneMap.begin();
+                while (index-- > 0 && it++ != fPathZoneMap.end()) {}
+                return it->first.c_str();
+            }
+        }
+    
+        std::string getParamAddress(FAUSTFLOAT* zone)
+        {
+            for (const auto& it : fPathZoneMap) {
                 if (it.second == zone) return it.first;
             }
-            return fNullStr;
+            return "";
         }
     
         FAUSTFLOAT* getParamZone(const std::string& str)
         {
             if (fPathZoneMap.find(str) != fPathZoneMap.end()) {
                 return fPathZoneMap[str];
-            }
-            if (fLabelZoneMap.find(str) != fLabelZoneMap.end()) {
+            } else if (fLabelZoneMap.find(str) != fLabelZoneMap.end()) {
                 return fLabelZoneMap[str];
             }
             return nullptr;
