@@ -57,6 +57,11 @@
 #if QT_VERSION >= 0x050000
 #include <QtWidgets>
 #endif
+#if QT_VERSION >= 0x060000
+#define QTSetMargins(a) setContentsMargins(a, a, a, a)
+#else
+#define QTSetMargins(a) setMargin(a)
+#endif
 #include <QApplication>
 #include <QLabel>
 #include <QComboBox>
@@ -73,6 +78,14 @@
 #include "faust/gui/qrcodegen.h"
 #endif
 
+#if QT_VERSION >= 0x040300
+  #define QTColorLighter	lighter
+  #define QTColorDarker		darker
+#else
+  #define QTColorLighter	light
+  #define QTColorDarker		dark
+#endif
+
 // for compatibility
 #define minValue minimum
 #define maxValue maximum
@@ -132,7 +145,7 @@ public:
         
         QPalette pal = opt->palette;
         QColor knobColor = pal.mid().color();
-        QColor borderColor = knobColor.light();
+        QColor borderColor = knobColor.QTColorLighter();
         QColor meterColor = (dial->state & State_Enabled) ?
         QColor("orange") : pal.mid().color();
         // pal.highlight().color() : pal.mid().color();
@@ -143,9 +156,9 @@ public:
         
         // The bright metering bit...
         QConicalGradient meterShadow(xcenter, ycenter, -90);
-        meterShadow.setColorAt(0, meterColor.dark());
+        meterShadow.setColorAt(0, meterColor.QTColorDarker());
         meterShadow.setColorAt(0.5, meterColor);
-        meterShadow.setColorAt(1.0, meterColor.light().light());
+        meterShadow.setColorAt(1.0, meterColor.QTColorLighter().QTColorLighter());
         p->setBrush(meterShadow);
         p->setPen(Qt::transparent);
         p->drawPie(xcenter - meterWidth / 2, ycenter - meterWidth / 2,
@@ -169,9 +182,9 @@ public:
         
         QRadialGradient gradient(xcenter - shineCenter, ycenter - shineCenter,
                                  shineExtension,	xcenter - shineFocus, ycenter - shineFocus);
-        gradient.setColorAt(0.2, knobColor.light().light());
+        gradient.setColorAt(0.2, knobColor.QTColorLighter().QTColorLighter());
         gradient.setColorAt(0.5, knobColor);
-        gradient.setColorAt(1.0, knobColor.dark(150));
+        gradient.setColorAt(1.0, knobColor.QTColorDarker(150));
         QBrush knobBrush(gradient);
         p->setBrush(knobBrush);
         p->drawEllipse(xcenter - knobWidth / 2, ycenter - knobWidth / 2,
@@ -209,8 +222,8 @@ public:
         if (knobBorderWidth > 0) {
             QLinearGradient inShadow(xcenter - side / 4, ycenter - side / 4,
                                      xcenter + side / 4, ycenter + side / 4);
-            inShadow.setColorAt(0.0, borderColor.light());
-            inShadow.setColorAt(1.0, borderColor.dark());
+            inShadow.setColorAt(0.0, borderColor.QTColorLighter());
+            inShadow.setColorAt(1.0, borderColor.QTColorDarker());
             p->setPen(QPen(QBrush(inShadow), knobBorderWidth * 7 / 8));
             p->drawEllipse(xcenter - side / 2 + indent,
                            ycenter - side / 2 + indent,
@@ -220,8 +233,8 @@ public:
         // Scale shadow...
         QLinearGradient outShadow(xcenter - side / 3, ycenter - side / 3,
                                   xcenter + side / 3, ycenter + side / 3);
-        outShadow.setColorAt(0.0, background.dark().dark());
-        outShadow.setColorAt(1.0, background.light().light());
+        outShadow.setColorAt(0.0, background.QTColorDarker().QTColorDarker());
+        outShadow.setColorAt(1.0, background.QTColorLighter().QTColorLighter());
         p->setPen(QPen(QBrush(outShadow), scaleShadowWidth));
         p->drawArc(xcenter - side / 2 + scaleShadowWidth / 2,
                    ycenter - side / 2 + scaleShadowWidth / 2,
@@ -235,11 +248,11 @@ public:
         double y = ycenter + len * cos(angle);
         
         QColor pointerColor = pal.dark().color();
-        pen.setColor((dial->state & State_Enabled) ? pointerColor.dark(140) : pointerColor);
+        pen.setColor((dial->state & State_Enabled) ? pointerColor.QTColorDarker(140) : pointerColor);
         pen.setWidth(pointerWidth + 2);
         p->setPen(pen);
         p->drawLine(QLineF(xcenter, ycenter, x, y));
-        pen.setColor((dial->state & State_Enabled) ? pointerColor.light() : pointerColor.light(140));
+        pen.setColor((dial->state & State_Enabled) ? pointerColor.QTColorLighter() : pointerColor.QTColorLighter(140));
         pen.setWidth(pointerWidth);
         p->setPen(pen);
         p->drawLine(QLineF(xcenter - 1, ycenter - 1, x - 1, y - 1));
@@ -852,6 +865,7 @@ public:
     {
         fSlider->setMinimum(0);
         fSlider->setMaximum(10000);
+        //fSlider->setSingleStep(fStep);
         fSlider->setValue(int(0.5+fConverter->faust2ui(fCur)));
         *fZone = fCur;
     }
@@ -1261,7 +1275,7 @@ protected:
         std::map<std::string, std::string> metadata;
         std::string label;
         extractMetadata(fulllabel, label, metadata);
-        layout->setMargin(5);
+        layout->QTSetMargins(5);
         QWidget* box;
         
         label = startWith(label, "0x") ? "" : label;
@@ -1281,7 +1295,7 @@ protected:
                 
             } else {
                 // no label here we use simple widget
-                layout->setMargin(0);
+                layout->QTSetMargins(0);
                 box = new QWidget(this);
             }
             
@@ -1306,7 +1320,7 @@ protected:
                 
             } else {
                 // no label here we use simple widget
-                layout->setMargin(0);
+                layout->QTSetMargins(0);
                 box = new QWidget;
             }
             
@@ -1582,7 +1596,7 @@ public:
     virtual void addButton(const char* label, FAUSTFLOAT* zone)
     {
         QAbstractButton* w = new QPushButton(label);
-        w->setAttribute(Qt::WA_MacNoClickThrough);
+//        w->setAttribute(Qt::WA_MacNoClickThrough); // obsolete at least since Qt 5.11
         uiButton* c = new uiButton(this, zone, w);
         
         insert(label, w);
