@@ -412,13 +412,13 @@ class mspUI : public UI, public PathBuilder
     
         void clear()
         {
-            for (auto& it : fInputLabelTable) {
+            for (const auto& it : fInputLabelTable) {
                 delete it.second;
             }
             fInputLabelTable.clear();
             fInputPathTable.clear();
             
-            for (auto& it : fOutputLabelTable) {
+            for (const auto& it : fOutputLabelTable) {
                 delete it.second;
             }
             fOutputLabelTable.clear();
@@ -428,7 +428,7 @@ class mspUI : public UI, public PathBuilder
         void displayControls()
         {
             post("------- Range and path ----------");
-            for (auto& it : fInputPathTable) {
+            for (const auto& it : fInputPathTable) {
                 char param[STR_SIZE];
                 it.second->toString(param);
                 post(param);
@@ -461,11 +461,11 @@ class mspUI : public UI, public PathBuilder
 // MIDI handler
 //==============
 
-struct faustgen_midi : public midi_handler {
+struct max_midi : public midi_handler {
     
     void* m_midi_outlet = NULL;
     
-    faustgen_midi(void* midi_outlet = NULL):m_midi_outlet(midi_outlet)
+    max_midi(void* midi_outlet = NULL):m_midi_outlet(midi_outlet)
     {}
     
     void sendMessage(std::vector<unsigned char>& message)
