@@ -6,8 +6,7 @@
     each section for license and copyright information.
 *************************************************************************/
 
-/*******************BEGIN ARCHITECTURE SECTION (part 1/2)****************/
-
+/******************* BEGIN max-msp.cpp ****************/
 /************************************************************************
     FAUST Architecture File
     Copyright (C) 2004-2018 GRAME, Centre National de Creation Musicale
@@ -132,7 +131,7 @@ using namespace std;
 #define ASSIST_INLET 	1
 #define ASSIST_OUTLET 	2
 
-#define EXTERNAL_VERSION    "0.80"
+#define EXTERNAL_VERSION    "0.84"
 #define STR_SIZE            512
 
 #include "faust/gui/GUI.h"
@@ -140,7 +139,7 @@ using namespace std;
 #include "faust/gui/mspUI.h"
 #include "faust/dsp/poly-dsp.h"
 
-std::list<GUI*> GUI::fGuiList;
+list<GUI*> GUI::fGuiList;
 ztimedmap GUI::gTimedZoneMap;
 
 static t_class* faust_class;
@@ -156,7 +155,6 @@ typedef struct faust
     void** m_args;
     mspUI* m_dspUI;
     dsp* m_dsp;
-    mydsp_poly* m_dsp_poly;
     void* m_control_outlet;
     char* m_json;
     t_systhread_mutex m_mutex;    
@@ -165,7 +163,7 @@ typedef struct faust
     SaveUI* m_savedUI;
 #ifdef MIDICTRL
     MidiUI* m_midiUI;
-    faustgen_midi* m_midiHandler;
+    max_midi* m_midiHandler;
     void* m_midi_outlet;
 #endif
 #ifdef SOUNDFILE
@@ -188,48 +186,14 @@ void faust_allocate(t_faust* x, int nvoices)
     
     if (nvoices > 0) {
         post("polyphonic DSP voices = %d", nvoices);
-        x->m_dsp_poly = new mydsp_poly(new mydsp(), nvoices, true, true);
+        x->m_dsp = new mydsp_poly(new mydsp(), nvoices, true, true);
 #ifdef POLY2
-        x->m_dsp = new dsp_sequencer(x->m_dsp_poly, new effect());
-#else
-        x->m_dsp = x->m_dsp_poly;
-#endif
-#ifdef MIDICTRL
-        x->m_midiHandler->addMidiIn(x->m_dsp_poly);
+        x->m_dsp = new dsp_sequencer(x->m_dsp, new effect());
 #endif
     } else {
         post("monophonic DSP");
-        #if (DOWN_SAMPLING > 0)
-            #if (FILTER_TYPE == 0)
-                x->m_dsp = new dsp_down_sampler<Identity<Double<1,1>, DOWN_SAMPLING>>(new mydsp());
-            #elif (FILTER_TYPE == 1)
-                x->m_dsp = new dsp_down_sampler<LowPass3<Double<45,100>, DOWN_SAMPLING, double>>(new mydsp());
-            #elif (FILTER_TYPE == 2)
-                x->m_dsp = new dsp_down_sampler<LowPass4<Double<45,100>, DOWN_SAMPLING, double>>(new mydsp());
-            #elif (FILTER_TYPE == 3)
-                x->m_dsp = new dsp_down_sampler<LowPass3e<Double<45,100>, DOWN_SAMPLING, double>>(new mydsp());
-            #elif (FILTER_TYPE == 4)
-                x->m_dsp = new dsp_down_sampler<LowPass6eé<Double<45,100>, DOWN_SAMPLING, double>>(new mydsp());
-            #else
-                #error "ERROR : Filter type must be in [0..4] range"
-            #endif
-        #elif (UP_SAMPLING > 0)
-            #if (FILTER_TYPE == 0)
-                x->m_dsp = new dsp_up_sampler<Identity<Double<1,1>, UP_SAMPLING>>(new mydsp());
-            #elif (FILTER_TYPE == 1)
-                x->m_dsp = new dsp_up_sampler<LowPass3<Double<45,100>, UP_SAMPLING, double>>(new mydsp());
-            #elif (FILTER_TYPE == 2)
-                x->m_dsp = new dsp_up_sampler<LowPass4<Double<45,100>, UP_SAMPLING, double>>(new mydsp());
-            #elif (FILTER_TYPE == 3)
-                x->m_dsp = new dsp_up_sampler<LowPass3e<Double<45,100>, UP_SAMPLING, double>>(new mydsp());
-            #elif (FILTER_TYPE == 4)
-                x->m_dsp = new dsp_up_sampler<LowPass6e<Double<45,100>, UP_SAMPLING, double>>(new mydsp());
-            #else
-                #error "ERROR : Filter type must be in [0..4] range"
-            #endif
-        #else
-            x->m_dsp = new mydsp();
-        #endif
+        // Create a DS/US + Filter adapted DSP
+        x->m_dsp = createSRAdapter<FAUSTFLOAT>(new mydsp(), DOWN_SAMPLING, UP_SAMPLING, FILTER_TYPE);
     }
     
 #ifdef MIDICTRL
@@ -322,29 +286,7 @@ void faust_anything(t_faust* obj, t_symbol* s, short ac, t_atom* av)
 void faust_polyphony(t_faust* x, t_symbol* s, short ac, t_atom* av)
 {
     if (systhread_mutex_lock(x->m_mutex) == MAX_ERR_NONE) {
-        
-    #ifdef MIDICTRL
-        if (x->m_dsp_poly) {
-            x->m_midiHandler->removeMidiIn(x->m_dsp_poly);
-        }
-    #endif
-        
         faust_allocate(x, av[0].a_w.w_long);
-        
-        // Initialize at the system's sampling rate
-        x->m_dsp->init(long(sys_getsr()));
-        // Initialize User Interface (here connnection with controls)
-        x->m_dsp->buildUserInterface(x->m_dspUI);
-        
-        // Prepare JSON
-        faust_make_json(x);
-      
-        // Send JSON to JS script
-        faust_create_jsui(x);
-        
-        // Load old controller state
-        x->m_dsp->buildUserInterface(x->m_savedUI);
-        
         systhread_mutex_unlock(x->m_mutex);
     } else {
         post("Mutex lock cannot be taken...");
@@ -401,13 +343,13 @@ void faust_create_jsui(t_faust* x)
 /*--------------------------------------------------------------------------*/
 void faust_update_outputs(t_faust* x)
 {
-    for (auto& it1 : x->m_output_table) {
+    for (const auto& it1 : x->m_output_table) {
         bool new_val = false;
         FAUSTFLOAT value = x->m_dspUI->getOutputValue(it1.first, new_val);
         if (new_val) {
             t_atom at_value;
             atom_setfloat(&at_value, value);
-            for (auto& it2 : it1.second) {
+            for (const auto& it2 : it1.second) {
                 object_method_typed(it2, gensym("float"), 1, &at_value, 0);
             }
         }
@@ -430,88 +372,66 @@ void* faust_new(t_symbol* s, short ac, t_atom* av)
 {
     bool midi_sync = false;
     int nvoices = 0;
+    t_faust* x = (t_faust*)object_alloc(faust_class);
     
     mydsp* tmp_dsp = new mydsp();
     MidiMeta::analyse(tmp_dsp, midi_sync, nvoices);
-    delete tmp_dsp;
- 
-    t_faust* x = (t_faust*)object_alloc(faust_class);
-    
-    x->m_savedUI = new SaveLabelUI();
-    x->m_dspUI = NULL;
-    x->m_dsp = NULL;
-    x->m_dsp_poly = NULL;
-    x->m_json = NULL;
-    x->m_mute = false;
-    
-#ifdef MIDICTRL
-    x->m_midi_outlet = outlet_new((t_pxobject*)x, NULL);
-    x->m_midiHandler = new faustgen_midi(x->m_midi_outlet);
-    x->m_midiUI = new MidiUI(x->m_midiHandler);
-#endif
-
-    faust_allocate(x, nvoices);
-
-    x->m_Inputs = x->m_dsp->getNumInputs();
-    x->m_Outputs = x->m_dsp->getNumOutputs();
-   
-    x->m_dspUI = new mspUI();
-    x->m_control_outlet = outlet_new((t_pxobject*)x, (char*)"list");
-
-    // Initialize at the system's sampling rate
-    x->m_dsp->init(long(sys_getsr()));
-    // Initialize User Interface (here connnection with controls)
-    x->m_dsp->buildUserInterface(x->m_dspUI);
-    
-    t_max_err err = systhread_mutex_new(&x->m_mutex, SYSTHREAD_MUTEX_NORMAL);
-    if (err != MAX_ERR_NONE) {
-        post("Cannot allocate mutex...");
-    }
-    
-    // Prepare JSON
-    faust_make_json(x);
-    
-    x->m_args = (void**)calloc((x->m_dsp->getNumInputs() + x->m_dsp->getNumOutputs()) + 2, sizeof(void*));
-   /* Multi in */
-    dsp_setup((t_pxobject*)x, x->m_dsp->getNumInputs());
-
-    /* Multi out */
-    for (int i = 0; i < x->m_dsp->getNumOutputs(); i++) {
-        outlet_new((t_pxobject*)x, (char*)"signal");
-    }
-
-    ((t_pxobject*)x)->z_misc = Z_NO_INPLACE; // To assure input and output buffers are actually different
-    
 #ifdef SOUNDFILE
     Max_Meta3 meta3;
-    x->m_dsp->metadata(&meta3);
+    tmp_dsp->metadata(&meta3);
     string bundle_path_str = SoundUI::getBinaryPathFrom(meta3.fName);
     if (bundle_path_str == "") {
         post("Bundle_path '%s' cannot be found!", meta3.fName.c_str());
     }
     x->m_soundInterface = new SoundUI(bundle_path_str);
-    // SoundUI has to be dispatched on all internal voices
-    if (x->m_dsp_poly) x->m_dsp_poly->setGroup(false);
-    x->m_dsp->buildUserInterface(x->m_soundInterface);
-    if (x->m_dsp_poly) x->m_dsp_poly->setGroup(true);
+#endif
+    delete tmp_dsp;
+    
+    x->m_savedUI = new SaveLabelUI();
+    x->m_dspUI = new mspUI();
+    x->m_dsp = NULL;
+    x->m_json = NULL;
+    x->m_mute = false;
+    x->m_control_outlet = outlet_new((t_pxobject*)x, (char*)"list");
+    
+#ifdef MIDICTRL
+    x->m_midi_outlet = outlet_new((t_pxobject*)x, NULL);
+    x->m_midiHandler = new max_midi(x->m_midi_outlet);
+    x->m_midiUI = new MidiUI(x->m_midiHandler);
 #endif
     
-#ifdef OSCCTRL
-    x->m_oscInterface = NULL;
-#endif
+    faust_allocate(x, nvoices);
     
-    // Send JSON to JS script
-    faust_create_jsui(x);
+    if (systhread_mutex_new(&x->m_mutex, SYSTHREAD_MUTEX_NORMAL) != MAX_ERR_NONE) {
+        post("Cannot allocate mutex...");
+    }
     
-    // Load old controller state
-    x->m_dsp->buildUserInterface(x->m_savedUI);
+    int num_input;
+    if (x->m_dspUI->isMulti()) {
+        num_input = x->m_dsp->getNumInputs() + 1;
+    } else {
+        num_input = x->m_dsp->getNumInputs();
+    }
+    
+    x->m_args = (void**)calloc((num_input + x->m_dsp->getNumOutputs()) + 2, sizeof(void*));
+    
+    /* Multi in */
+    dsp_setup((t_pxobject*)x, num_input);
+    
+    /* Multi out */
+    for (int i = 0; i < x->m_dsp->getNumOutputs(); i++) {
+        outlet_new((t_pxobject*)x, (char*)"signal");
+    }
+    
+    ((t_pxobject*)x)->z_misc = Z_NO_INPLACE; // To assure input and output buffers are actually different
     
     // Display controls
+#ifdef POST
     x->m_dspUI->displayControls();
+#endif
     
     // Get attributes values
     attr_args_process(x, ac, av);
-    
     return x;
 }
 
@@ -644,9 +564,9 @@ void faust_assist(t_faust* x, void* b, long msg, long a, char* dst)
     if (msg == ASSIST_INLET) {
         if (a == 0) {
             if (x->m_dsp->getNumInputs() == 0) {
-                sprintf(dst, "(message) : Unused Input");
+                sprintf(dst, "(messages)");
             } else {
-                sprintf(dst, "(signal) : Audio Input %ld", (a+1));
+                sprintf(dst, "(messages/signal) : Audio Input %ld", (a+1));
             }
         } else if (a < x->m_dsp->getNumInputs()) {
             sprintf(dst, "(signal) : Audio Input %ld", (a+1));
@@ -654,8 +574,10 @@ void faust_assist(t_faust* x, void* b, long msg, long a, char* dst)
     } else if (msg == ASSIST_OUTLET) {
         if (a < x->m_dsp->getNumOutputs()) {
             sprintf(dst, "(signal) : Audio Output %ld", (a+1));
-        } else {
+        } else if (a == x->m_dsp->getNumOutputs()) {
             sprintf(dst, "(list) : [path, cur|init, min, max]*");
+        } else {
+            sprintf(dst, "(int) : raw MIDI bytes*");
         }
     }
 }
@@ -802,5 +724,4 @@ void ext_main(void* r)
 #endif
 }
 
-/********************END ARCHITECTURE SECTION (part 2/2)****************/
-
+/******************* END max-msp.cpp ****************/
