@@ -1,73 +1,70 @@
 /*
  faust2wasm: GRAME 2017-2019
 */
- 
+
 'use strict';
 
 function getBase64Mixer() { return "AGFzbQEAAAABj4CAgAACYAN/f38AYAR/f39/AX0CkoCAgAABBm1lbW9yeQZtZW1vcnkCAAIDg4CAgAACAAEHmoCAgAACC2NsZWFyT3V0cHV0AAAIbWl4Vm9pY2UAAQqKgoCAAALigICAAAEDfwJAQQAhBQNAAkAgAiAFQQJ0aigCACEDQQAhBANAAkAgAyAEQQJ0akMAAAAAOAIAIARBAWohBCAEIABIBEAMAgUMAQsACwsgBUEBaiEFIAUgAUgEQAwCBQwBCwALCwsLnYGAgAACBH8DfQJ9QQAhB0MAAAAAIQgDQAJAQQAhBiACIAdBAnRqKAIAIQQgAyAHQQJ0aigCACEFA0ACQCAEIAZBAnRqKgIAIQkgCCAJi5chCCAFIAZBAnRqKgIAIQogBSAGQQJ0aiAKIAmSOAIAIAZBAWohBiAGIABIBEAMAgUMAQsACwsgB0EBaiEHIAcgAUgEQAwCBQwBCwALCyAIDwsL"; }
 
 // Polyphonic Faust DSP
 class mydspPolyProcessor extends AudioWorkletProcessor {
-    
+
     // JSON parsing functions
-    static parse_ui(ui, obj, callback)
-    {
+    static parse_ui(ui, obj, callback) {
         for (var i = 0; i < ui.length; i++) {
-           	mydspPolyProcessor.parse_group(ui[i], obj, callback);
+            mydspPolyProcessor.parse_group(ui[i], obj, callback);
         }
     }
-    
-    static parse_group(group, obj, callback)
-    {
+
+    static parse_group(group, obj, callback) {
         if (group.items) {
             mydspPolyProcessor.parse_items(group.items, obj, callback);
         }
     }
-    
-    static parse_items(items, obj, callback)
-    {
+
+    static parse_items(items, obj, callback) {
         for (var i = 0; i < items.length; i++) {
             callback(items[i], obj, callback);
         }
     }
-    
-    static parse_item1(item, obj, callback)
-    {
+
+    static parse_item1(item, obj, callback) {
         if (item.type === "vgroup"
             || item.type === "hgroup"
             || item.type === "tgroup") {
             mydspPolyProcessor.parse_items(item.items, obj, callback);
         } else if (item.type === "hbargraph"
-                   || item.type === "vbargraph") {
+            || item.type === "vbargraph") {
             // Nothing
         } else if (item.type === "vslider"
-                   || item.type === "hslider"
-                   || item.type === "button"
-                   || item.type === "checkbox"
-                   || item.type === "nentry") {
-            obj.push({ name: item.address,
-                     defaultValue: item.init,
-                     minValue: item.min,
-                     maxValue: item.max });
+            || item.type === "hslider"
+            || item.type === "button"
+            || item.type === "checkbox"
+            || item.type === "nentry") {
+            obj.push({
+                name: item.address,
+                defaultValue: item.init,
+                minValue: item.min,
+                maxValue: item.max
+            });
         }
     }
-    
-    static parse_item2(item, obj, callback)
-    {
+
+    static parse_item2(item, obj, callback) {
         if (item.type === "vgroup"
             || item.type === "hgroup"
             || item.type === "tgroup") {
             mydspPolyProcessor.parse_items(item.items, obj, callback);
         } else if (item.type === "hbargraph"
-                   || item.type === "vbargraph") {
+            || item.type === "vbargraph") {
             // Keep bargraph adresses
             obj.outputs_items.push(item.address);
             obj.pathTable[item.address] = parseInt(item.index);
         } else if (item.type === "vslider"
-                   || item.type === "hslider"
-                   || item.type === "button"
-                   || item.type === "checkbox"
-                   || item.type === "nentry") {
+            || item.type === "hslider"
+            || item.type === "button"
+            || item.type === "checkbox"
+            || item.type === "nentry") {
             // Keep inputs adresses
             obj.inputs_items.push(item.address);
             obj.pathTable[item.address] = parseInt(item.index);
@@ -75,48 +72,50 @@ class mydspPolyProcessor extends AudioWorkletProcessor {
                 for (var i = 0; i < item.meta.length; i++) {
                     if (item.meta[i].midi !== undefined) {
                         if (item.meta[i].midi.trim() === "pitchwheel") {
-                            obj.fPitchwheelLabel.push({ path:item.address,
-                                  min:parseFloat(item.min),
-                                  max:parseFloat(item.max) });
+                            obj.fPitchwheelLabel.push({
+                                path: item.address,
+                                min: parseFloat(item.min),
+                                max: parseFloat(item.max)
+                            });
                         } else if (item.meta[i].midi.trim().split(" ")[0] === "ctrl") {
                             obj.fCtrlLabel[parseInt(item.meta[i].midi.trim().split(" ")[1])]
-                            .push({ path:item.address,
-                                  min:parseFloat(item.min),
-                                  max:parseFloat(item.max) });
+                                .push({
+                                    path: item.address,
+                                    min: parseFloat(item.min),
+                                    max: parseFloat(item.max)
+                                });
                         }
                     }
                 }
             }
         }
     }
-    
-    static b64ToUint6(nChr)
-    {
+
+    static b64ToUint6(nChr) {
         return nChr > 64 && nChr < 91 ?
-        nChr - 65
-        : nChr > 96 && nChr < 123 ?
-        nChr - 71
-        : nChr > 47 && nChr < 58 ?
-        nChr + 4
-        : nChr === 43 ?
-        62
-        : nChr === 47 ?
-        63
-        :
-        0;
+            nChr - 65
+            : nChr > 96 && nChr < 123 ?
+                nChr - 71
+                : nChr > 47 && nChr < 58 ?
+                    nChr + 4
+                    : nChr === 43 ?
+                        62
+                        : nChr === 47 ?
+                            63
+                            :
+                            0;
     }
-    
-    static atob(sBase64, nBlocksSize)
-    {
+
+    static atob(sBase64, nBlocksSize) {
         if (typeof atob === 'function') {
             return atob(sBase64);
         } else {
-            
+
             var sB64Enc = sBase64.replace(/[^A-Za-z0-9\+\/]/g, "");
             var nInLen = sB64Enc.length;
             var nOutLen = nBlocksSize ? Math.ceil((nInLen * 3 + 1 >> 2) / nBlocksSize) * nBlocksSize : nInLen * 3 + 1 >> 2;
             var taBytes = new Uint8Array(nOutLen);
-            
+
             for (var nMod3, nMod4, nUint24 = 0, nOutIdx = 0, nInIdx = 0; nInIdx < nInLen; nInIdx++) {
                 nMod4 = nInIdx & 3;
                 nUint24 |= mydspPolyProcessor.b64ToUint6(sB64Enc.charCodeAt(nInIdx)) << 18 - 6 * nMod4;
@@ -130,47 +129,43 @@ class mydspPolyProcessor extends AudioWorkletProcessor {
             return taBytes.buffer;
         }
     }
-    
-    static remap(v, mn0, mx0, mn1, mx1)
-    {
+
+    static remap(v, mn0, mx0, mn1, mx1) {
         return (1.0 * (v - mn0) / (mx0 - mn0)) * (mx1 - mn1) + mn1;
     }
 
-    static get parameterDescriptors() 
-    {
+    static get parameterDescriptors() {
         // Analyse JSON to generate AudioParam parameters
         var params = [];
-        
+
         // Add instrument parameters
         mydspPolyProcessor.parse_ui(JSON.parse(getJSONmydsp()).ui, params, mydspPolyProcessor.parse_item1);
-        
+
         // Possibly add effect parameters
         if (typeof (getJSONeffect) !== "undefined") {
-             mydspPolyProcessor.parse_ui(JSON.parse(getJSONeffect()).ui, params, mydspPolyProcessor.parse_item1);
+            mydspPolyProcessor.parse_ui(JSON.parse(getJSONeffect()).ui, params, mydspPolyProcessor.parse_item1);
         }
         return params;
     }
-    
-    static createMemory(buffer_size, polyphony) 
-    {
+
+    static createMemory(buffer_size, polyphony) {
         // Memory allocator
         var ptr_size = 4;
         var sample_size = 4;
-        
-        function pow2limit(x)
-        {
+
+        function pow2limit(x) {
             var n = 65536; // Minimum = 64 kB
             while (n < x) { n = 2 * n; }
             return n;
         }
-        
+
         var json_object = null;
         try {
             json_object = JSON.parse(getJSONmydsp());
         } catch (e) {
             return null;
         }
-        
+
         var effect_json_object_size = 0;
         if (typeof (getJSONeffect) !== "undefined") {
             var effect_json_object = null;
@@ -182,52 +177,51 @@ class mydspPolyProcessor extends AudioWorkletProcessor {
                 return null;
             }
         }
-        
+
         var memory_size = pow2limit(effect_json_object_size + parseInt(json_object.size) * polyphony + ((parseInt(json_object.inputs) + parseInt(json_object.outputs) * 2) * (ptr_size + (buffer_size * sample_size)))) / 65536;
         memory_size = Math.max(2, memory_size); // As least 2
         return new WebAssembly.Memory({ initial: memory_size, maximum: memory_size });
     }
-    
-    constructor(options)
-    {
+
+    constructor(options) {
         super(options);
         this.running = true;
-        
+
         this.json_object = JSON.parse(getJSONmydsp());
         if (typeof (getJSONeffect) !== "undefined") {
             this.effect_json_object = JSON.parse(getJSONeffect());
         }
-        
-        this.output_handler = function(path, value) { this.port.postMessage({ path: path, value: value }); };
-        
+
+        this.output_handler = function (path, value) { this.port.postMessage({ path: path, value: value }); };
+
         this.debug = false;
-        
+
         this.ins = null;
         this.outs = null;
         this.mixing = null;
         this.compute_handler = null;
-        
+
         this.dspInChannnels = [];
         this.dspOutChannnels = [];
-        
+
         this.fFreqLabel = [];
         this.fGateLabel = [];
         this.fGainLabel = [];
         this.fKeyFun = null;
         this.fVelFun = null;
         this.fDate = 0;
-        
+
         this.fPitchwheelLabel = [];
         this.fCtrlLabel = new Array(128);
         for (var i = 0; i < this.fCtrlLabel.length; i++) { this.fCtrlLabel[i] = []; }
-   
+
         this.numIn = parseInt(this.json_object.inputs);
         this.numOut = parseInt(this.json_object.outputs);
-        
+
         // Memory allocator
         this.ptr_size = 4;
         this.sample_size = 4;
-         
+
         var wasm_memory = mydspPolyProcessor.createMemory(mydspPolyProcessor.buffer_size, options.processorOptions.polyphony);
 
         // Create Mixer
@@ -238,10 +232,10 @@ class mydspPolyProcessor extends AudioWorkletProcessor {
             env: {
                 memoryBase: 0,
                 tableBase: 0,
-                    
+
                 // Integer version
                 _abs: Math.abs,
-                
+
                 // Float version
                 _acosf: Math.acos,
                 _asinf: Math.asin,
@@ -251,12 +245,12 @@ class mydspPolyProcessor extends AudioWorkletProcessor {
                 _cosf: Math.cos,
                 _expf: Math.exp,
                 _floorf: Math.floor,
-                _fmodf: function(x, y) { return x % y; },
+                _fmodf: function (x, y) { return x % y; },
                 _logf: Math.log,
                 _log10f: Math.log10,
                 _max_f: Math.max,
                 _min_f: Math.min,
-                _remainderf: function(x, y) { return x - Math.round(x/y) * y; },
+                _remainderf: function (x, y) { return x - Math.round(x / y) * y; },
                 _powf: Math.pow,
                 _roundf: Math.fround,
                 _sinf: Math.sin,
@@ -268,7 +262,10 @@ class mydspPolyProcessor extends AudioWorkletProcessor {
                 _coshf: Math.cosh,
                 _sinhf: Math.sinh,
                 _tanhf: Math.tanh,
-                   
+                _isnanf: Number.isNaN,
+                _isinff: function (x) { return !isFinite(x); },
+                _copysignf: function (x, y) { return Math.sign(x) === Math.sign(y) ? x : -x; },
+
                 // Double version
                 _acos: Math.acos,
                 _asin: Math.asin,
@@ -278,12 +275,12 @@ class mydspPolyProcessor extends AudioWorkletProcessor {
                 _cos: Math.cos,
                 _exp: Math.exp,
                 _floor: Math.floor,
-                _fmod: function(x, y) { return x % y; },
+                _fmod: function (x, y) { return x % y; },
                 _log: Math.log,
                 _log10: Math.log10,
                 _max_: Math.max,
                 _min_: Math.min,
-                _remainder:function(x, y) { return x - Math.round(x/y) * y; },
+                _remainder: function (x, y) { return x - Math.round(x / y) * y; },
                 _pow: Math.pow,
                 _round: Math.fround,
                 _sin: Math.sin,
@@ -295,9 +292,12 @@ class mydspPolyProcessor extends AudioWorkletProcessor {
                 _cosh: Math.cosh,
                 _sinh: Math.sinh,
                 _tanh: Math.tanh,
-                
+                _isnan: Number.isNaN,
+                _isinf: function (x) { return !isFinite(x); },
+                _copysign: function (x, y) { return Math.sign(x) === Math.sign(y) ? x : -x; },
+
                 memory: wasm_memory,
-                
+
                 table: new WebAssembly.Table({ initial: 0, element: 'anyfunc' })
             }
         }
@@ -307,103 +307,101 @@ class mydspPolyProcessor extends AudioWorkletProcessor {
 
         // wasm instance
         this.factory = new WebAssembly.Instance(mydspPolyProcessor.wasm_module, this.importObject).exports;
-        
+
         // wasm effect
         this.effect = (mydspPolyProcessor.wasm_effect_module) ? new WebAssembly.Instance(mydspPolyProcessor.wasm_effect_module, this.importObject).exports : null;
-        
+
         this.HEAP = wasm_memory.buffer;
         this.HEAP32 = new Int32Array(this.HEAP);
         this.HEAPF32 = new Float32Array(this.HEAP);
-        
+
         // Warning: keeps a ref on HEAP in Chrome and prevent proper GC
         //console.log(this.HEAP);
         //console.log(this.HEAP32);
         //console.log(this.HEAPF32);
-        
+
         // bargraph
         this.outputs_timer = 5;
         this.outputs_items = [];
-        
+
         // input items
         this.inputs_items = [];
-        
+
         // Start of HEAP index
         this.audio_heap_ptr = 0; // Fails when 0...
-        
+
         // Setup pointers offset
         this.audio_heap_ptr_inputs = this.audio_heap_ptr;
         this.audio_heap_ptr_outputs = this.audio_heap_ptr_inputs + (this.numIn * this.ptr_size);
         this.audio_heap_ptr_mixing = this.audio_heap_ptr_outputs + (this.numOut * this.ptr_size);
-        
+
         // Setup buffer offset
         this.audio_heap_inputs = this.audio_heap_ptr_mixing + (this.numOut * this.ptr_size);
         this.audio_heap_outputs = this.audio_heap_inputs + (this.numIn * mydspPolyProcessor.buffer_size * this.sample_size);
         this.audio_heap_mixing = this.audio_heap_outputs + (this.numOut * mydspPolyProcessor.buffer_size * this.sample_size);
-        
+
         // Setup DSP voices offset
         this.dsp_start = this.audio_heap_mixing + (this.numOut * mydspPolyProcessor.buffer_size * this.sample_size);
-        
+
         if (this.debug) {
             console.log(this.mixer);
             console.log(this.factory);
             console.log(this.effect);
         }
-        
+
         // Start of DSP memory ('polyphony' DSP voices)
         this.polyphony = options.processorOptions.polyphony;
         this.dsp_voices = [];
         this.dsp_voices_state = [];
         this.dsp_voices_level = [];
         this.dsp_voices_date = [];
-        
+
         this.kActiveVoice = 0;
         this.kFreeVoice = -1;
         this.kReleaseVoice = -2;
         this.kNoVoice = -3;
-     
+
         this.pathTable = [];
-        
+
         // Allocate table for 'setParamValue'
         this.value_table = [];
-        
+
         for (var i = 0; i < this.polyphony; i++) {
             this.dsp_voices[i] = this.dsp_start + i * parseInt(this.json_object.size);
             this.dsp_voices_state[i] = this.kFreeVoice;
             this.dsp_voices_level[i] = 0;
             this.dsp_voices_date[i] = 0;
         }
-        
+
         // Effect memory starts after last voice
         this.effect_start = this.dsp_voices[this.polyphony - 1] + parseInt(this.json_object.size);
-        
-        this.printMemory = function ()
-        {
+
+        this.printMemory = function () {
             console.log("============== Memory layout ==============");
             console.log("json_object.size: " + this.json_object.size);
-            
+
             console.log("audio_heap_ptr: " + this.audio_heap_ptr);
-            
+
             console.log("audio_heap_ptr_inputs: " + this.audio_heap_ptr_inputs);
             console.log("audio_heap_ptr_outputs: " + this.audio_heap_ptr_outputs);
             console.log("audio_heap_ptr_mixing: " + this.audio_heap_ptr_mixing);
-            
+
             console.log("audio_heap_inputs: " + this.audio_heap_inputs);
             console.log("audio_heap_outputs: " + this.audio_heap_outputs);
             console.log("audio_heap_mixing: " + this.audio_heap_mixing);
-            
+
             console.log("dsp_start: " + this.dsp_start);
-            for (var i = 0; i <  this.polyphony; i++) {
+            for (var i = 0; i < this.polyphony; i++) {
                 console.log("dsp_voices[i]: " + i + " " + this.dsp_voices[i]);
             }
             console.log("effect_start: " + this.effect_start);
         }
-    
-        this.getPlayingVoice = function(pitch)
-        {
+
+        this.getPlayingVoice = function (pitch) {
             var voice_playing = this.kNoVoice;
             var oldest_date_playing = Number.MAX_VALUE;
-            
-            for (var i = 0; i <  this.polyphony; i++) {
+
+            for (var i = 0; i < this.polyphony; i++) {
                 if (this.dsp_voices_state[i] === pitch) {
                     // Keeps oldest playing voice
                     if (this.dsp_voices_date[i] < oldest_date_playing) {
@@ -412,35 +410,33 @@ class mydspPolyProcessor extends AudioWorkletProcessor {
                     }
                 }
             }
-            
+
             return voice_playing;
         }
-        
+
         // Always returns a voice
-        this.allocVoice = function(voice)
-        {
+        this.allocVoice = function (voice) {
             // so that envelop is always re-initialized
             this.factory.instanceClear(this.dsp_voices[voice]);
             this.dsp_voices_date[voice] = this.fDate++;
             this.dsp_voices_state[voice] = this.kActiveVoice;
             return voice;
         }
-        
-        this.getFreeVoice = function()
-        {
-            for (var i = 0; i <  this.polyphony; i++) {
+
+        this.getFreeVoice = function () {
+            for (var i = 0; i < this.polyphony; i++) {
                 if (this.dsp_voices_state[i] === this.kFreeVoice) {
                     return this.allocVoice(i);
                 }
             }
-            
+
             var voice_release = this.kNoVoice;
             var voice_playing = this.kNoVoice;
             var oldest_date_release = Number.MAX_VALUE;
             var oldest_date_playing = Number.MAX_VALUE;
-            
+
             // Scan all voices
-            for (var i = 0; i <  this.polyphony; i++) {
+            for (var i = 0; i < this.polyphony; i++) {
                 // Try to steal a voice in kReleaseVoice mode...
                 if (this.dsp_voices_state[i] === this.kReleaseVoice) {
                     // Keeps oldest release voice
@@ -455,7 +451,7 @@ class mydspPolyProcessor extends AudioWorkletProcessor {
                     }
                 }
             }
-            
+
             // Then decide which one to steal
             if (oldest_date_release != Number.MAX_VALUE) {
                 if (this.debug) {
@@ -471,9 +467,8 @@ class mydspPolyProcessor extends AudioWorkletProcessor {
                 return this.kNoVoice;
             }
         }
-        
-        this.update_outputs = function ()
-        {
+
+        this.update_outputs = function () {
             if (this.outputs_items.length > 0 && this.output_handler && this.outputs_timer-- === 0) {
                 this.outputs_timer = 5;
                 for (var i = 0; i < this.outputs_items.length; i++) {
@@ -481,53 +476,51 @@ class mydspPolyProcessor extends AudioWorkletProcessor {
                 }
             }
         }
-        
-        this.midiToFreq = function (note)
-        {
+
+        this.midiToFreq = function (note) {
             return 440.0 * Math.pow(2.0, (note - 69.0) / 12.0);
         }
-        
-        this.initAux = function ()
-        {
+
+        this.initAux = function () {
             var i;
-            
+
             if (this.numIn > 0) {
                 this.ins = this.audio_heap_ptr_inputs;
                 for (i = 0; i < this.numIn; i++) {
                     this.HEAP32[(this.ins >> 2) + i] = this.audio_heap_inputs + ((mydspPolyProcessor.buffer_size * this.sample_size) * i);
                 }
-                
+
                 // Prepare Ins buffer tables
                 var dspInChans = this.HEAP32.subarray(this.ins >> 2, (this.ins + this.numIn * this.ptr_size) >> 2);
                 for (i = 0; i < this.numIn; i++) {
                     this.dspInChannnels[i] = this.HEAPF32.subarray(dspInChans[i] >> 2, (dspInChans[i] + mydspPolyProcessor.buffer_size * this.sample_size) >> 2);
                 }
             }
-            
+
             if (this.numOut > 0) {
                 // allocate memory for output and mixing arrays
                 this.outs = this.audio_heap_ptr_outputs;
                 this.mixing = this.audio_heap_ptr_mixing;
-                
+
                 for (i = 0; i < this.numOut; i++) {
                     this.HEAP32[(this.outs >> 2) + i] = this.audio_heap_outputs + ((mydspPolyProcessor.buffer_size * this.sample_size) * i);
                     this.HEAP32[(this.mixing >> 2) + i] = this.audio_heap_mixing + ((mydspPolyProcessor.buffer_size * this.sample_size) * i);
                 }
-                
+
                 // Prepare Out buffer tables
                 var dspOutChans = this.HEAP32.subarray(this.outs >> 2, (this.outs + this.numOut * this.ptr_size) >> 2);
                 for (i = 0; i < this.numOut; i++) {
                     this.dspOutChannnels[i] = this.HEAPF32.subarray(dspOutChans[i] >> 2, (dspOutChans[i] + mydspPolyProcessor.buffer_size * this.sample_size) >> 2);
                 }
             }
-            
+
             // Parse UI part
             mydspPolyProcessor.parse_ui(this.json_object.ui, this, mydspPolyProcessor.parse_item2);
-            
+
             if (this.effect) {
                 mydspPolyProcessor.parse_ui(this.effect_json_object.ui, this, mydspPolyProcessor.parse_item2);
             }
-     
+
             // keep 'keyOn/keyOff' labels
             for (i = 0; i < this.inputs_items.length; i++) {
                 if (this.inputs_items[i].endsWith("/gate")) {
@@ -539,30 +532,29 @@ class mydspPolyProcessor extends AudioWorkletProcessor {
                     this.fKeyFun = (pitch) => { return pitch; };
                     this.fFreqLabel.push(this.pathTable[this.inputs_items[i]]);
                 } else if (this.inputs_items[i].endsWith("/gain")) {
-                    this.fVelFun = (vel) => { return vel/127.0; };
+                    this.fVelFun = (vel) => { return vel / 127.0; };
                     this.fGainLabel.push(this.pathTable[this.inputs_items[i]]);
                 } else if (this.inputs_items[i].endsWith("/vel") || this.inputs_items[i].endsWith("/velocity")) {
                     this.fVelFun = (vel) => { return vel; };
                     this.fGainLabel.push(this.pathTable[this.inputs_items[i]]);
                 }
             }
-            
+
             // Init DSP voices
             for (i = 0; i < this.polyphony; i++) {
                 this.factory.init(this.dsp_voices[i], sampleRate);  // 'sampleRate' is defined in AudioWorkletGlobalScope
             }
-            
+
             // Init effect
             if (this.effect) {
                 this.effect.init(this.effect_start, sampleRate);
             }
-            
+
             // Print memory layout
             this.printMemory();
         }
-        
-        this.keyOn = function (channel, pitch, velocity)
-        {
+
+        this.keyOn = function (channel, pitch, velocity) {
             var voice = this.getFreeVoice();
             if (this.debug) {
                 console.log("keyOn voice %d", voice);
@@ -578,9 +570,8 @@ class mydspPolyProcessor extends AudioWorkletProcessor {
             }
             this.dsp_voices_state[voice] = pitch;
         }
-        
-        this.keyOff = function (channel, pitch, velocity)
-        {
+
+        this.keyOff = function (channel, pitch, velocity) {
             var voice = this.getPlayingVoice(pitch);
             if (voice !== this.kNoVoice) {
                 if (this.debug) {
@@ -598,9 +589,8 @@ class mydspPolyProcessor extends AudioWorkletProcessor {
                 }
             }
         }
-        
-        this.allNotesOff = function ()
-        {
+
+        this.allNotesOff = function () {
             for (var i = 0; i < this.polyphony; i++) {
                 for (var j = 0; j < this.fGateLabel.length; j++) {
                     this.factory.setParamValue(this.dsp_voices[i], this.fGateLabel[j], 0.0);
@@ -608,37 +598,34 @@ class mydspPolyProcessor extends AudioWorkletProcessor {
                 this.dsp_voices_state[i] = this.kReleaseVoice;
             }
         }
-        
-        this.ctrlChange = function (channel, ctrl, value)
-        {
+
+        this.ctrlChange = function (channel, ctrl, value) {
             if (ctrl === 123 || ctrl === 120) {
                 this.allNotesOff();
             }
-            
+
             if (this.fCtrlLabel[ctrl] !== []) {
                 for (var i = 0; i < this.fCtrlLabel[ctrl].length; i++) {
                     var path = this.fCtrlLabel[ctrl][i].path;
                     this.setParamValue(path, mydspPolyProcessor.remap(value, 0, 127, this.fCtrlLabel[ctrl][i].min, this.fCtrlLabel[ctrl][i].max));
                     if (this.output_handler) {
-                   		this.output_handler(path, this.getParamValue(path));
-                   	}
+                        this.output_handler(path, this.getParamValue(path));
+                    }
                 }
             }
         }
-        
-        this.pitchWheel = function (channel, wheel)
-        {
+
+        this.pitchWheel = function (channel, wheel) {
             for (var i = 0; i < this.fPitchwheelLabel.length; i++) {
                 var pw = this.fPitchwheelLabel[i];
                 this.setParamValue(pw.path, mydspPolyProcessor.remap(wheel, 0, 16383, pw.min, pw.max));
                 if (this.output_handler) {
-                   	this.output_handler(pw.path, this.getParamValue(pw.path));
+                    this.output_handler(pw.path, this.getParamValue(pw.path));
                 }
             }
         }
-        
-        this.setParamValue = function (path, val)
-        {
+
+        this.setParamValue = function (path, val) {
             if (this.effect && getJSONeffect().includes(path)) {
                 this.effect.setParamValue(this.effect_start, this.pathTable[path], val);
             } else {
@@ -648,24 +635,22 @@ class mydspPolyProcessor extends AudioWorkletProcessor {
             }
         }
 
-        this.getParamValue = function (path)
-        {
+        this.getParamValue = function (path) {
             if (this.effect && getJSONeffect().includes(path)) {
                 return this.effect.getParamValue(this.effect_start, this.pathTable[path]);
             } else {
                 return this.factory.getParamValue(this.dsp_voices[0], this.pathTable[path]);
             }
         }
-            
+
         // Init resulting DSP
         this.initAux();
-        
+
         // Set message handler
         this.port.onmessage = this.handleMessage.bind(this);
     }
-   
-    handleMessage(event) 
-    {
+
+    handleMessage(event) {
         var msg = event.data;
         switch (msg.type) {
             // Generic MIDI message
@@ -681,9 +666,8 @@ class mydspPolyProcessor extends AudioWorkletProcessor {
             case "destroy": this.running = false; break;
         }
     }
-  	
-    midiMessage(data)
-    {
+
+    midiMessage(data) {
         var cmd = data[0] >> 4;
         var channel = data[0] & 0xf;
         var data1 = data[1];
@@ -701,12 +685,11 @@ class mydspPolyProcessor extends AudioWorkletProcessor {
             this.pitchWheel(channel, (data2 * 128.0 + data1));
         }
     }
-    
-    process(inputs, outputs, parameters) 
-    {
+
+    process(inputs, outputs, parameters) {
         var input = inputs[0];
         var output = outputs[0];
-        
+
         // Check inputs
         if (this.numIn > 0 && (!input || !input[0] || input[0].length === 0)) {
             //console.log("Process input error");
@@ -717,7 +700,7 @@ class mydspPolyProcessor extends AudioWorkletProcessor {
             //console.log("Process output error");
             return true;
         }
-        
+
         // Copy inputs
         if (input !== undefined) {
             for (var chan = 0; chan < Math.min(this.numIn, input.length); ++chan) {
@@ -725,15 +708,15 @@ class mydspPolyProcessor extends AudioWorkletProcessor {
                 dspInput.set(input[chan]);
             }
         }
-       
+
         // Possibly call an externally given callback (for instance to synchronize playing a MIDIFile...)
         if (this.compute_handler) {
             this.compute_handler(mydspPolyProcessor.buffer_size);
         }
-         
+
         // First clear the outputs
         this.mixer.clearOutput(mydspPolyProcessor.buffer_size, this.numOut, this.outs);
-        
+
         // Compute all running voices
         try {
             for (var i = 0; i < this.polyphony; i++) {
@@ -753,13 +736,13 @@ class mydspPolyProcessor extends AudioWorkletProcessor {
             if (this.effect) {
                 this.effect.compute(this.effect_start, mydspPolyProcessor.buffer_size, this.outs, this.outs);
             }
-        } catch(e) {
-        	console.log("ERROR in compute (" + e + ")");
+        } catch (e) {
+            console.log("ERROR in compute (" + e + ")");
         }
-        
+
         // Update bargraph
         this.update_outputs();
-        
+
         // Copy outputs
         if (output !== undefined) {
             for (var chan = 0; chan < Math.min(this.numOut, output.length); ++chan) {
@@ -767,7 +750,7 @@ class mydspPolyProcessor extends AudioWorkletProcessor {
                 output[chan].set(dspOutput);
             }
         }
-        
+
         return this.running;
     }
 }
