@@ -28,6 +28,14 @@
 
 using namespace std;
 
+/*
+ C backend and module description:
+    - in -os mode:
+        - subcontainers are merged in the main class
+        - CScalarOneSampleCodeContainer1 (used in -os0) separates the DSP control state in iControl and fControl (possibly to be allocated elsewhere)
+        - CScalarOneSampleCodeContainer2 (used in -os1) separates the DSP control state in iControl and fControl and the DSP state in iZone and fZone (possibly to be allocated elsewhere)
+ */
+
 map<string, bool> CInstVisitor::gFunctionSymbolTable;
 
 dsp_factory_base* CCodeContainer::produceFactory()
@@ -39,9 +47,13 @@ dsp_factory_base* CCodeContainer::produceFactory()
 
 CodeContainer* CCodeContainer::createScalarContainer(const string& name, int sub_container_type)
 {
-    return (gGlobal->gOneSample)
-        ? new CScalarOneSampleCodeContainer(name, 0, 1, fOut, sub_container_type)
-        : new CScalarCodeContainer(name, 0, 1, fOut, sub_container_type);
+    if (gGlobal->gOneSample == 0) {
+        return new CScalarOneSampleCodeContainer1(name, 0, 1, fOut, sub_container_type);
+    } else if (gGlobal->gOneSample == 1) {
+        return new CScalarOneSampleCodeContainer2(name, 0, 1, fOut, sub_container_type);
+    } else {
+        return new CScalarCodeContainer(name, 0, 1, fOut, sub_container_type);
+    }
 }
 
 CodeContainer* CCodeContainer::createContainer(const string& name, int numInputs, int numOutputs, ostream* dst)
@@ -49,9 +61,6 @@ CodeContainer* CCodeContainer::createContainer(const string& name, int numInputs
     gGlobal->gDSPStruct = true;
     CodeContainer* container;
 
-    if (gGlobal->gMemoryManager) {
-        throw faustexception("ERROR : -mem not supported for C\n");
-    }
     if (gGlobal->gOpenCLSwitch) {
         throw faustexception("ERROR : OpenCL not supported for C\n");
     }
@@ -66,9 +75,13 @@ CodeContainer* CCodeContainer::createContainer(const string& name, int numInputs
     } else if (gGlobal->gVectorSwitch) {
         container = new CVectorCodeContainer(name, numInputs, numOutputs, dst);
     } else {
-        container = (gGlobal->gOneSample)
-            ? new CScalarOneSampleCodeContainer(name, numInputs, numOutputs, dst, kInt)
-            : new CScalarCodeContainer(name, numInputs, numOutputs, dst, kInt);
+        if (gGlobal->gOneSample == 0) {
+            container = new CScalarOneSampleCodeContainer1(name, numInputs, numOutputs, dst, kInt);
+        } else if (gGlobal->gOneSample == 1) {
+            container = new CScalarOneSampleCodeContainer2(name, numInputs, numOutputs, dst, kInt);
+        } else {
+            container = new CScalarCodeContainer(name, numInputs, numOutputs, dst, kInt);
+        }
     }
 
     return container;
@@ -149,6 +162,7 @@ void CCodeContainer::produceClass()
 {
     int n = 0;
 
+    tab(n, *fOut);
     *fOut << "#ifdef __cplusplus" << endl;
     *fOut << "extern \"C\" {" << endl;
     *fOut << "#endif" << endl;
@@ -158,10 +172,10 @@ void CCodeContainer::produceClass()
     printLibrary(*fOut);
     printIncludeFile(*fOut);
 
-    // Sub containers
+    // Generate gub containers
     generateSubContainers();
 
-    // Functions
+    // Global declarations
     tab(n, *fOut);
     fCodeProducer->Tab(n);
     generateGlobalDeclarations(fCodeProducer);
@@ -361,7 +375,8 @@ void CCodeContainer::produceClass()
     *fOut << "#endif" << endl;
 }
 
-void CScalarOneSampleCodeContainer::produceClass()
+// Used with -os0 option
+void CScalarOneSampleCodeContainer1::produceClass()
 {
     int n = 0;
     
@@ -377,30 +392,30 @@ void CScalarOneSampleCodeContainer::produceClass()
     *fOut << "#define RESTRICT __restrict__" << endl;
     *fOut << "#endif" << endl;
     tab(n, *fOut);
-
+    
     // Libraries
     printLibrary(*fOut);
     printIncludeFile(*fOut);
     
-    // Sub containers
+    // Sub containers are merged in the main class
     mergeSubContainers();
- 
-    // Functions
+    
+    // Global declarations
     tab(n, *fOut);
     fCodeProducer->Tab(n);
-    
     generateGlobalDeclarations(fCodeProducer);
     
     tab(n, *fOut);
     *fOut << "#ifndef FAUSTCLASS " << endl;
     *fOut << "#define FAUSTCLASS " << fKlassName << endl;
     *fOut << "#endif" << endl;
+    tab(n, *fOut);
     
     *fOut << "#ifdef __APPLE__ " << endl;
     *fOut << "#define exp10f __exp10f" << endl;
     *fOut << "#define exp10 __exp10" << endl;
     *fOut << "#endif" << endl;
-     
+    
     if (gGlobal->gLightMode) {
         tab(n, *fOut);
         *fOut << "#define max(a,b) ((a < b) ? b : a)\n";
@@ -420,17 +435,258 @@ void CScalarOneSampleCodeContainer::produceClass()
     *fOut << "} " << fKlassName << ";";
     tab(n, *fOut);
     
-    tab(n, *fOut);
-    *fOut << "#define FAUST_INPUTS " << fNumInputs << endl;
-    *fOut << "#define FAUST_OUTPUTS  " << fNumOutputs << endl;
     tab(n, *fOut);
     *fOut << "#define FAUST_INT_CONTROLS " << fInt32ControlNum << endl;
     *fOut << "#define FAUST_REAL_CONTROLS " << fRealControlNum << endl;
     tab(n, *fOut);
-    *fOut << "#define FAUST_INT_ZONE " << static_cast<CInstVisitor1*>(fCodeProducer)->getIntZoneSize() << endl;
-    *fOut << "#define FAUST_FLOAT_ZONE " << static_cast<CInstVisitor1*>(fCodeProducer)->getRealZoneSize();
+    
+    *fOut << "#ifndef TESTBENCH";
+    
+    // Memory methods
+    tab(n, *fOut);
+    if (!gGlobal->gLightMode) {
+        
+        if (fAllocateInstructions->fCode.size() > 0) {
+            tab(n, *fOut);
+            *fOut << "static void allocate" << fKlassName << "(" << fKlassName << "* dsp) {";
+            tab(n + 1, *fOut);
+            generateAllocate(fCodeProducer);
+            tab(n, *fOut);
+            *fOut << "}";
+        }
+        tab(n, *fOut);
+        
+        if (fDestroyInstructions->fCode.size() > 0) {
+            tab(n, *fOut);
+            *fOut << "static void destroy" << fKlassName << "(" << fKlassName << "* dsp) {";
+            tab(n + 1, *fOut);
+            generateDestroy(fCodeProducer);
+            tab(n, *fOut);
+            *fOut << "}";
+            tab(n, *fOut);
+        }
+        
+        *fOut << fKlassName << "* new" << fKlassName << "() { ";
+        tab(n + 1, *fOut);
+        *fOut << fKlassName << "* dsp = (" << fKlassName << "*)calloc(1, sizeof(" << fKlassName << "));";
+        if (fAllocateInstructions->fCode.size() > 0) {
+            tab(n + 1, *fOut);
+            *fOut << "allocate" << fKlassName << "(dsp);";
+        }
+        tab(n + 1, *fOut);
+        *fOut << "return dsp;";
+        tab(n, *fOut);
+        *fOut << "}";
+        
+        tab(n, *fOut);
+        tab(n, *fOut);
+        *fOut << "void delete" << fKlassName << "(" << fKlassName << "* dsp) { ";
+        if (fDestroyInstructions->fCode.size() > 0) {
+            tab(n + 1, *fOut);
+            *fOut << "destroy" << fKlassName << "(dsp);";
+        }
+        tab(n + 1, *fOut);
+        *fOut << "free(dsp);";
+        tab(n, *fOut);
+        *fOut << "}";
+        
+        // Print metadata declaration
+        tab(n, *fOut);
+        produceMetadata(n);
+    }
+    
+    // Get sample rate method
+    tab(n, *fOut);
+    fCodeProducer->Tab(n);
+    generateGetSampleRate("getSampleRate" + fKlassName, "dsp", false, false)->accept(fCodeProducer);
     
     tab(n, *fOut);
+    produceInfoFunctions(n, fKlassName, "dsp", false, false, fCodeProducer);
+    
+    // Inits
+    
+    // Dummy
+    tab(n, *fOut);
+    *fOut << "void classInit" << fKlassName << "(int sample_rate) {}";
+    tab(n, *fOut);
+    
+    tab(n, *fOut);
+    *fOut << "void staticInit" << fKlassName << "(" << fKlassName << "* dsp, int sample_rate) {";
+    {
+        tab(n + 1, *fOut);
+        fCodeProducer->Tab(n + 1);
+        // Rename 'sig' in 'dsp', remove 'dsp' allocation, inline subcontainers 'instanceInit' and 'fill' function call
+        inlineSubcontainersFunCalls(fStaticInitInstructions)->accept(fCodeProducer);
+    }
+    back(1, *fOut);
+    *fOut << "}";
+    
+    tab(n, *fOut);
+    tab(n, *fOut);
+    *fOut << "void instanceResetUserInterface" << fKlassName << "(" << fKlassName << "* dsp) {";
+    {
+        tab(n + 1, *fOut);
+        fCodeProducer->Tab(n + 1);
+        generateResetUserInterface(fCodeProducer);
+    }
+    back(1, *fOut);
+    *fOut << "}";
+    
+    tab(n, *fOut);
+    tab(n, *fOut);
+    *fOut << "void instanceClear" << fKlassName << "(" << fKlassName << "* dsp) {";
+    {
+        tab(n + 1, *fOut);
+        fCodeProducer->Tab(n + 1);
+        generateClear(fCodeProducer);
+    }
+    back(1, *fOut);
+    *fOut << "}";
+    
+    tab(n, *fOut);
+    tab(n, *fOut);
+    *fOut << "void instanceConstants" << fKlassName << "(" << fKlassName << "* dsp, int sample_rate) {";
+    {
+        tab(n + 1, *fOut);
+        fCodeProducer->Tab(n + 1);
+        // Rename 'sig' in 'dsp', remove 'dsp' allocation, inline subcontainers 'instanceInit' and 'fill' function call
+        inlineSubcontainersFunCalls(fInitInstructions)->accept(fCodeProducer);
+    }
+    back(1, *fOut);
+    *fOut << "}";
+    tab(n, *fOut);
+    
+    tab(n, *fOut);
+    *fOut << "void instanceInit" << fKlassName << "(" << fKlassName << "* dsp, int sample_rate) {";
+    // staticInit has to be called for each instance since the tables are actually not shared between instances
+    tab(n + 1, *fOut);
+    *fOut << "staticInit" << fKlassName << "(dsp, sample_rate);";
+    tab(n + 1, *fOut);
+    *fOut << "instanceConstants" << fKlassName << "(dsp, sample_rate);";
+    tab(n + 1, *fOut);
+    *fOut << "instanceResetUserInterface" << fKlassName << "(dsp);";
+    tab(n + 1, *fOut);
+    *fOut << "instanceClear" << fKlassName << "(dsp);";
+    tab(n, *fOut);
+    *fOut << "}";
+    tab(n, *fOut);
+    
+    tab(n, *fOut);
+    *fOut << "void init" << fKlassName << "(" << fKlassName << "* dsp, int sample_rate) {";
+    tab(n + 1, *fOut);
+    *fOut << "instanceInit" << fKlassName << "(dsp, sample_rate);";
+    tab(n, *fOut);
+    *fOut << "}";
+    
+    if (!gGlobal->gLightMode) {
+        // User interface
+        tab(n, *fOut);
+        tab(n, *fOut);
+        *fOut << "void buildUserInterface" << fKlassName << "(" << fKlassName << "* dsp, UIGlue* ui_interface) {";
+        tab(n + 1, *fOut);
+        fCodeProducer->Tab(n + 1);
+        generateUserInterface(fCodeProducer);
+        back(1, *fOut);
+        *fOut << "}";
+    }
+    
+    tab(n, *fOut);
+    tab(n, *fOut);
+    *fOut << "void control" << fKlassName << "(" << fKlassName << "* dsp, " << subst("int* RESTRICT iControl, $0* RESTRICT fControl) {", xfloat());
+    tab(n + 1, *fOut);
+    fCodeProducer->Tab(n + 1);
+    // Generates local variables declaration and setup
+    generateComputeBlock(fCodeProducer);
+    back(1, *fOut);
+    *fOut << "}" << endl;
+    
+    tab(n, *fOut);
+    *fOut << "int getNumIntControls" << fKlassName << "(" << fKlassName << "* dsp) { return "
+          << fInt32ControlNum << "; }";
+    tab(n, *fOut);
+    
+    tab(n, *fOut);
+    *fOut << "int getNumRealControls" << fKlassName << "(" << fKlassName << "* dsp) { return "
+          << fRealControlNum << "; }";
+    
+    // Compute
+    generateCompute(n);
+    
+    tab(n, *fOut);
+    *fOut << "#endif // TESTBENCH" << endl;
+    tab(n, *fOut);
+    
+    // Generate user interface macros if needed
+    printMacros(*fOut, n);
+    
+    *fOut << "#ifdef __cplusplus" << endl;
+    *fOut << "}" << endl;
+    *fOut << "#endif" << endl;
+}
+
+// Used with -os1 option
+void CScalarOneSampleCodeContainer2::produceClass()
+{
+    int n = 0;
+    
+    tab(n, *fOut);
+    *fOut << "#ifdef __cplusplus" << endl;
+    *fOut << "extern \"C\" {" << endl;
+    *fOut << "#endif" << endl;
+    tab(n, *fOut);
+    
+    *fOut << "#if defined(_WIN32)" << endl;
+    *fOut << "#define RESTRICT __restrict" << endl;
+    *fOut << "#else" << endl;
+    *fOut << "#define RESTRICT __restrict__" << endl;
+    *fOut << "#endif" << endl;
+    tab(n, *fOut);
+    
+    // Libraries
+    printLibrary(*fOut);
+    printIncludeFile(*fOut);
+    
+    // Sub containers are merged in the main class
+    mergeSubContainers();
+    
+    // Global declarations
+    tab(n, *fOut);
+    fCodeProducer->Tab(n);
+    generateGlobalDeclarations(fCodeProducer);
+    
+    tab(n, *fOut);
+    *fOut << "#ifndef FAUSTCLASS " << endl;
+    *fOut << "#define FAUSTCLASS " << fKlassName << endl;
+    *fOut << "#endif" << endl;
+    
+    *fOut << "#ifdef __APPLE__ " << endl;
+    *fOut << "#define exp10f __exp10f" << endl;
+    *fOut << "#define exp10 __exp10" << endl;
+    *fOut << "#endif" << endl;
+    
+    if (gGlobal->gLightMode) {
+        tab(n, *fOut);
+        *fOut << "#define max(a,b) ((a < b) ? b : a)\n";
+        *fOut << "#define min(a,b) ((a < b) ? a : b)\n";
+        tab(n, *fOut);
+    }
+    
+    tab(n, *fOut);
+    *fOut << "typedef struct {";
+    tab(n + 1, *fOut);
+    
+    // Fields
+    fCodeProducer->Tab(n + 1);
+    generateDeclarations(fCodeProducer);
+    
+    // Kept here because staticInit incorrectly change the size later on
+    int int_zone_size = static_cast<CInstVisitor1*>(fCodeProducer)->getIntZoneSize();
+    int real_zone_size = static_cast<CInstVisitor1*>(fCodeProducer)->getRealZoneSize();
+    
+    back(1, *fOut);
+    *fOut << "} " << fKlassName << ";";
+    tab(n, *fOut);
+    
     tab(n, *fOut);
     *fOut << "#ifndef TESTBENCH";
     
@@ -497,21 +753,13 @@ void CScalarOneSampleCodeContainer::produceClass()
     
     // Inits
     
-    // TODO
-    /*
-     CInstVisitor codeproducer1(fOut, "");
-     codeproducer1.Tab(n+1);
-     generateStaticInitFun("classInit" + fKlassName, false)->accept(&codeproducer1);
-     generateInstanceInitFun("instanceInit" + fKlassName, false, false)->accept(&codeproducer2);
-     */
-    
     // Dummy
     tab(n, *fOut);
     *fOut << "void classInit" << fKlassName << "(int sample_rate) {}";
     tab(n, *fOut);
     
     tab(n, *fOut);
-    *fOut << "void staticInit" << fKlassName << "(" << fKlassName << "* dsp, int sample_rate, " << subst("int* iZone, $0* fZone) {", xfloat());
+    *fOut << "void staticInit" << fKlassName << "(" << fKlassName << "* dsp, int sample_rate, " << subst("int* iZone, $0* fZone) {", ifloat());
     {
         tab(n + 1, *fOut);
         fCodeProducer->Tab(n + 1);
@@ -536,7 +784,7 @@ void CScalarOneSampleCodeContainer::produceClass()
     
     tab(n, *fOut);
     tab(n, *fOut);
-    *fOut << "void instanceClear" << fKlassName << subst("(int* iZone, $0* fZone) {", xfloat());
+    *fOut << "void instanceClear" << fKlassName << "(" << fKlassName << "* dsp, " << subst("int* iZone, $0* fZone) {", ifloat());
     {
         tab(n + 1, *fOut);
         fCodeProducer->Tab(n + 1);
@@ -547,7 +795,7 @@ void CScalarOneSampleCodeContainer::produceClass()
     
     tab(n, *fOut);
     tab(n, *fOut);
-    *fOut << "void instanceConstants" << fKlassName << "(" << fKlassName << "* dsp, int sample_rate, " << subst("int* iZone, $0* fZone) {", xfloat());
+    *fOut << "void instanceConstants" << fKlassName << "(" << fKlassName << "* dsp, int sample_rate, " << subst("int* iZone, $0* fZone) {", ifloat());
     {
         tab(n + 1, *fOut);
         fCodeProducer->Tab(n + 1);
@@ -559,7 +807,7 @@ void CScalarOneSampleCodeContainer::produceClass()
     
     tab(n, *fOut);
     tab(n, *fOut);
-    *fOut << "void instanceInit" << fKlassName << "(" << fKlassName << "* dsp, int sample_rate, " << subst("int* iZone, $0* fZone) {", xfloat());
+    *fOut << "void instanceInit" << fKlassName << "(" << fKlassName << "* dsp, int sample_rate, " << subst("int* iZone, $0* fZone) {", ifloat());
     // staticInit has to be called for each instance since the tables are actually not shared between instances
     tab(n + 1, *fOut);
     *fOut << "staticInit" << fKlassName << "(dsp, sample_rate, iZone, fZone);";
@@ -568,13 +816,13 @@ void CScalarOneSampleCodeContainer::produceClass()
     tab(n + 1, *fOut);
     *fOut << "instanceResetUserInterface" << fKlassName << "(dsp);";
     tab(n + 1, *fOut);
-    *fOut << "instanceClear" << fKlassName << "(iZone, fZone);";
+    *fOut << "instanceClear" << fKlassName << "(dsp, iZone, fZone);";
     tab(n, *fOut);
     *fOut << "}";
     
     tab(n, *fOut);
     tab(n, *fOut);
-    *fOut << "void init" << fKlassName << "(" << fKlassName << "* dsp, int sample_rate, " << subst("int* iZone, $0* fZone) {", xfloat());
+    *fOut << "void init" << fKlassName << "(" << fKlassName << "* dsp, int sample_rate, " << subst("int* iZone, $0* fZone) {", ifloat());
     tab(n + 1, *fOut);
     *fOut << "instanceInit" << fKlassName << "(dsp, sample_rate, iZone, fZone);";
     tab(n, *fOut);
@@ -593,7 +841,8 @@ void CScalarOneSampleCodeContainer::produceClass()
     }
     
     tab(n, *fOut);
-    *fOut << "void control" << fKlassName << "(" << fKlassName << "* dsp, " << subst("int* RESTRICT iControl, $0* RESTRICT fControl, int* RESTRICT iZone, $0* RESTRICT fZone) {", xfloat());
+    tab(n, *fOut);
+    *fOut << "void control" << fKlassName << "(" << fKlassName << "* dsp, " << subst("int* RESTRICT iControl, $0* RESTRICT fControl, int* RESTRICT iZone, $1* RESTRICT fZone) {", xfloat(), ifloat());
     tab(n + 1, *fOut);
     fCodeProducer->Tab(n + 1);
     // Generates local variables declaration and setup
@@ -605,21 +854,31 @@ void CScalarOneSampleCodeContainer::produceClass()
     *fOut << "int getNumIntControls" << fKlassName << "(" << fKlassName << "* dsp) { return "
           << fInt32ControlNum << "; }";
     tab(n, *fOut);
-    tab(n, *fOut);
+    
     *fOut << "int getNumRealControls" << fKlassName << "(" << fKlassName << "* dsp) { return "
           << fRealControlNum << "; }";
     tab(n, *fOut);
+    
     tab(n, *fOut);
     *fOut << "int getiZoneSize" << fKlassName << "(" << fKlassName << "* dsp) { return "
-          << static_cast<CInstVisitor1*>(fCodeProducer)->getIntZoneSize() << "; }";
-    tab(n, *fOut);
+          << int_zone_size << "; }";
     tab(n, *fOut);
+    
     *fOut << "int getfZoneSize" << fKlassName << "(" << fKlassName << "* dsp) { return "
-          << static_cast<CInstVisitor1*>(fCodeProducer)->getRealZoneSize() << "; }";
+          << real_zone_size << "; }";
     
     // Compute
     generateCompute(n);
     
+    tab(n, *fOut);
+    *fOut << "#define FAUST_INT_CONTROLS " << fInt32ControlNum << endl;
+    *fOut << "#define FAUST_REAL_CONTROLS " << fRealControlNum << endl;
+    
+    tab(n, *fOut);
+    *fOut << "#define FAUST_INT_ZONE " << int_zone_size << endl;
+    *fOut << "#define FAUST_FLOAT_ZONE " << real_zone_size;
+    tab(n, *fOut);
+    
     tab(n, *fOut);
     *fOut << "#endif // TESTBENCH" << endl;
     tab(n, *fOut);
@@ -638,7 +897,7 @@ void CCodeContainer::produceMetadata(int tabs)
     *fOut << "void metadata" << fKlassName << "(MetaGlue* m) { ";
 
     // We do not want to accumulate metadata from all hierachical levels, so the upper level only is kept
-    for (auto& i : gGlobal->gMetaDataSet) {
+    for (const auto& i : gGlobal->gMetaDataSet) {
         if (i.first != tree("author")) {
             tab(tabs + 1, *fOut);
             *fOut << "m->declare(m->metaInterface, \"" << *(i.first) << "\", " << **(i.second.begin()) << ");";
@@ -697,17 +956,47 @@ void CScalarCodeContainer::generateComputeAux(int n)
     *fOut << "}" << endl;
 }
 
-// Special version for -os generation mode
-void CScalarOneSampleCodeContainer::generateComputeAux(int n)
+// Used with -os0 option
+void CScalarOneSampleCodeContainer1::generateComputeAux(int n)
 {
     // Generates declaration
     tab(n, *fOut);
     if (gGlobal->gInPlace) {
         *fOut << "void compute" << fKlassName << "(" << fKlassName
-              << subst("* dsp, $0* inputs, $0* outputs, int* RESTRICT iControl, $0* RESTRICT fControl, int* RESTRICT iZone, $0* RESTRICT fZone) {", xfloat());
+        << subst("* dsp, $0* inputs, $0* outputs, int* RESTRICT iControl, $0* RESTRICT fControl) {", xfloat());
     } else {
         *fOut << "void compute" << fKlassName << "(" << fKlassName
-              << subst("* dsp, $0* RESTRICT inputs, $0* RESTRICT outputs, int* RESTRICT iControl, $0* RESTRICT fControl, int* RESTRICT iZone, $0* RESTRICT fZone) {", xfloat());
+        << subst("* dsp, $0* RESTRICT inputs, $0* RESTRICT outputs, int* RESTRICT iControl, $0* RESTRICT fControl) {", xfloat());
+    }
+    tab(n + 1, *fOut);
+    fCodeProducer->Tab(n + 1);
+    
+    // Generates one sample computation
+    BlockInst* block = fCurLoop->generateOneSample();
+    block->accept(fCodeProducer);
+    
+    /*
+     // TODO : atomic switch
+     // Currently for soundfile management
+     */
+    generatePostComputeBlock(fCodeProducer);
+    
+    back(1, *fOut);
+    *fOut << "}" << endl;
+}
+
+// Used with -os1 option
+void CScalarOneSampleCodeContainer2::generateComputeAux(int n)
+{
+    // Generates declaration
+    tab(n, *fOut);
+    if (gGlobal->gInPlace) {
+        *fOut << "void compute" << fKlassName << "(" << fKlassName
+              << subst("* dsp, $0* inputs, $0* outputs, int* RESTRICT iControl, $0* RESTRICT fControl, int* RESTRICT iZone, $1* RESTRICT fZone) {", xfloat(), ifloat());
+        
+    } else {
+        *fOut << "void compute" << fKlassName << "(" << fKlassName
+              << subst("* dsp, $0* RESTRICT inputs, $0* RESTRICT outputs, int* RESTRICT iControl, $0* RESTRICT fControl, int* RESTRICT iZone, $1* RESTRICT fZone) {", xfloat(), ifloat());
     }
     tab(n + 1, *fOut);
     fCodeProducer->Tab(n + 1);
