@@ -90,11 +90,11 @@ string VectorCompiler::CS(Tree sig)
             //cerr << "CASE SH : fBackwardLoopDependencies.insert : " << tl << " --depend(A)son--> " << ls << endl;
             tl->fBackwardLoopDependencies.insert(ls);
 
-        } else if (isSigFixDelay(sig, x, d) && fClass->getLoopProperty(x, ls)) {
+        } else if (isSigDelay(sig, x, d) && fClass->getLoopProperty(x, ls)) {
             //cerr << "CASE DL : fBackwardLoopDependencies.insert : " << tl << " --depend(B)son--> " << ls << endl;
             tl->fBackwardLoopDependencies.insert(ls);
 
-        } else if (isSigFixDelay(sig, x, d) && isProj(x, &i, r) && fClass->getLoopProperty(r, ls)) {
+        } else if (isSigDelay(sig, x, d) && isProj(x, &i, r) && fClass->getLoopProperty(r, ls)) {
             //cerr << "CASE DR : fBackwardLoopDependencies.insert : " << tl << " --depend(B)son--> " << ls << endl;
             tl->fBackwardLoopDependencies.insert(ls);
 
@@ -272,7 +272,7 @@ string VectorCompiler::generateCacheCode(Tree sig, const string& exp)
         } else {
             // not delayed
             Tree x, y;
-            if (sharing > 1 && isSigFixDelay(sig, x, y) && verySimple(y)) {
+            if (sharing > 1 && isSigDelay(sig, x, y) && verySimple(y)) {
                 // cerr << "SPECIAL CASE NO CACHE NEEDED : " << ppsig(sig) << endl;
                 return exp;
             } else if (sharing > 1 && !verySimple(sig)) {
@@ -310,7 +310,7 @@ bool VectorCompiler::needSeparateLoop(Tree sig)
         b = true;
     } else if (verySimple(sig) || t->variability() < kSamp) {
         b = false;  // non sample computation never require a loop
-    } else if (isSigFixDelay(sig, x, y)) {
+    } else if (isSigDelay(sig, x, y)) {
         b = false;  //
     } else if (isProj(sig, &i, x)) {
         // cerr << "REC "; // recursive expressions require a separate loop
@@ -352,9 +352,9 @@ string VectorCompiler::generateVariableStore(Tree sig, const string& exp)
  * the maximum delay attached to exp.
  */
 
-string VectorCompiler::generateFixDelay(Tree sig, Tree exp, Tree delay)
+string VectorCompiler::generateDelay(Tree sig, Tree exp, Tree delay)
 {
-    // cerr << "VectorCompiler::generateFixDelay " << ppsig(sig) << endl;
+    // cerr << "VectorCompiler::generateDelay " << ppsig(sig) << endl;
 
     string code = CS(exp);  // ensure exp is compiled to have a vector name
     int    d, mxd = fOccMarkup->retrieve(exp)->getMaxDelay();
