@@ -29,6 +29,14 @@
 
 using namespace std;
 
+/*
+ C++ backend and module description:
+    - in -os mode:
+        - subcontainers are merged in the main class
+        - CPPScalarOneSampleCodeContainer1 (used in -os0) separates the DSP control state in iControl and fControl (possibly to be allocated elsewhere)
+        - CPPScalarOneSampleCodeContainer2 (used in -os1) separates the DSP control state in iControl and fControl and the DSP state in iZone and fZone (possibly to be allocated elsewhere)
+ */
+
 map<string, bool> CPPInstVisitor::gFunctionSymbolTable;
 
 dsp_factory_base* CPPCodeContainer::produceFactory()
@@ -40,10 +48,14 @@ dsp_factory_base* CPPCodeContainer::produceFactory()
 
 CodeContainer* CPPCodeContainer::createScalarContainer(const string& name, int sub_container_type)
 {
-    return (gGlobal->gOneSample)
-        ? new CPPScalarOneSampleCodeContainer(name, "", 0, 1, fOut, sub_container_type)
-        : new CPPScalarCodeContainer(name, "", 0, 1, fOut, sub_container_type);
-}
+    if (gGlobal->gOneSample == 0) {
+        return new CPPScalarOneSampleCodeContainer1(name, "", 0, 1, fOut, sub_container_type);
+    } else if (gGlobal->gOneSample == 1) {
+        return new CPPScalarOneSampleCodeContainer1(name, "", 0, 1, fOut, sub_container_type);
+    } else {
+        return new CPPScalarCodeContainer(name, "", 0, 1, fOut, sub_container_type);
+    }
+ }
 
 CodeContainer* CPPCodeContainer::createContainer(const string& name, const string& super, int numInputs, int numOutputs,
                                                  ostream* dst)
@@ -75,21 +87,33 @@ CodeContainer* CPPCodeContainer::createContainer(const string& name, const strin
     } else if (gGlobal->gVectorSwitch) {
         container = new CPPVectorCodeContainer(name, super, numInputs, numOutputs, dst);
     } else {
-        container = (gGlobal->gOneSample)
-            ? new CPPScalarOneSampleCodeContainer(name, super, numInputs, numOutputs, dst, kInt)
-            : new CPPScalarCodeContainer(name, super, numInputs, numOutputs, dst, kInt);
+        if (gGlobal->gOneSample == 0) {
+            container = new CPPScalarOneSampleCodeContainer1(name, super, numInputs, numOutputs, dst, kInt);
+        } else if (gGlobal->gOneSample == 1) {
+            container = new CPPScalarOneSampleCodeContainer2(name, super, numInputs, numOutputs, dst, kInt);
+        } else {
+            container = new CPPScalarCodeContainer(name, super, numInputs, numOutputs, dst, kInt);
+        }
     }
 
     return container;
 }
 
+// Scalar
+CPPScalarCodeContainer::CPPScalarCodeContainer(const string& name, const string& super, int numInputs, int numOutputs,
+                                               std::ostream* out, int sub_container_type)
+: CPPCodeContainer(name, super, numInputs, numOutputs, out)
+{
+    fSubContainerType = sub_container_type;
+}
+
 void CPPCodeContainer::produceMetadata(int tabs)
 {
     tab(tabs, *fOut);
     *fOut << "void metadata(Meta* m) { ";
 
     // We do not want to accumulate metadata from all hierachical levels, so the upper level only is kept
-    for (auto& i : gGlobal->gMetaDataSet) {
+    for (const auto& i : gGlobal->gMetaDataSet) {
         if (i.first != tree("author")) {
             tab(tabs + 1, *fOut);
             *fOut << "m->declare(\"" << *(i.first) << "\", " << **(i.second.begin()) << ");";
@@ -149,8 +173,8 @@ void CPPCodeContainer::produceInternal()
     // Global declarations
     tab(n, *fOut);
 
-    fCodeProducer.Tab(n);
-    generateGlobalDeclarations(&fCodeProducer);
+    fCodeProducer->Tab(n);
+    generateGlobalDeclarations(fCodeProducer);
 
     *fOut << "class " << fKlassName << " {";
 
@@ -167,8 +191,8 @@ void CPPCodeContainer::produceInternal()
     tab(n + 1, *fOut);
 
     // Fields
-    fCodeProducer.Tab(n + 1);
-    generateDeclarations(&fCodeProducer);
+    fCodeProducer->Tab(n + 1);
+    generateDeclarations(fCodeProducer);
 
     tab(n, *fOut);
     *fOut << "  public:";
@@ -177,19 +201,19 @@ void CPPCodeContainer::produceInternal()
     tab(n + 1, *fOut);
 
     // fKlassName used in method naming for subclasses
-    produceInfoFunctions(n + 1, fKlassName, "dsp", true, false, &fCodeProducer);
+    produceInfoFunctions(n + 1, fKlassName, "dsp", true, false, fCodeProducer);
     
     // TODO
-    // generateInstanceInitFun("instanceInit" + fKlassName, true, false)->accept(&fCodeProducer);
+    // generateInstanceInitFun("instanceInit" + fKlassName, true, false)->accept(fCodeProducer);
 
     // Inits
     tab(n + 1, *fOut);
     *fOut << "void instanceInit" << fKlassName << "(int sample_rate) {";
     tab(n + 2, *fOut);
-    fCodeProducer.Tab(n + 2);
-    generateInit(&fCodeProducer);
-    generateResetUserInterface(&fCodeProducer);
-    generateClear(&fCodeProducer);
+    fCodeProducer->Tab(n + 2);
+    generateInit(fCodeProducer);
+    generateResetUserInterface(fCodeProducer);
+    generateClear(fCodeProducer);
     back(1, *fOut);
     *fOut << "}";
 
@@ -204,17 +228,17 @@ void CPPCodeContainer::produceInternal()
         *fOut << "void fill" << fKlassName << subst("(int $0, $1* " + fTableName + ") {", counter, ifloat());
     }
     tab(n + 2, *fOut);
-    fCodeProducer.Tab(n + 2);
-    generateComputeBlock(&fCodeProducer);
+    fCodeProducer->Tab(n + 2);
+    generateComputeBlock(fCodeProducer);
     ForLoopInst* loop = fCurLoop->generateScalarLoop(counter);
-    loop->accept(&fCodeProducer);
+    loop->accept(fCodeProducer);
     back(1, *fOut);
     *fOut << "}";
 
     /*
     tab(n+1, *fOut);
-    fCodeProducer.Tab(n+1);
-    generateFillFun("fill" + fKlassName, true, false)->accept(&fCodeProducer);
+    fCodeProducer->Tab(n+1);
+    generateFillFun("fill" + fKlassName, true, false)->accept(fCodeProducer);
     */
 
     tab(n, *fOut);
@@ -255,13 +279,13 @@ void CPPCodeContainer::produceClass()
         *fOut << "namespace " << gGlobal->gNameSpace << " {" << endl;
     }
 
-    // Sub containers
+    // Generate gub containers
     generateSubContainers();
 
     // Global declarations
     tab(n, *fOut);
-    fCodeProducer.Tab(n);
-    generateGlobalDeclarations(&fCodeProducer);
+    fCodeProducer->Tab(n);
+    generateGlobalDeclarations(fCodeProducer);
 
     tab(n, *fOut);
     *fOut << "#ifndef FAUSTCLASS " << endl;
@@ -276,7 +300,6 @@ void CPPCodeContainer::produceClass()
     
     tab(n, *fOut);
     *fOut << "class " << fKlassName << " : public " << fSuperKlassName << " {";
-    
     tab(n + 1, *fOut);
 
     if (gGlobal->gUIMacroSwitch) {
@@ -289,16 +312,16 @@ void CPPCodeContainer::produceClass()
     tab(n + 1, *fOut);
 
     // Fields
-    fCodeProducer.Tab(n + 1);
+    fCodeProducer->Tab(n + 1);
     tab(n + 1, *fOut);
-    generateDeclarations(&fCodeProducer);
+    generateDeclarations(fCodeProducer);
 
     if (fAllocateInstructions->fCode.size() > 0) {
         tab(n + 1, *fOut);
         *fOut << "void allocate() {";
         tab(n + 2, *fOut);
-        fCodeProducer.Tab(n + 2);
-        generateAllocate(&fCodeProducer);
+        fCodeProducer->Tab(n + 2);
+        generateAllocate(fCodeProducer);
         back(1, *fOut);
         *fOut << "}";
         tab(n + 1, *fOut);
@@ -308,8 +331,8 @@ void CPPCodeContainer::produceClass()
         tab(n + 1, *fOut);
         *fOut << "void destroy() {";
         tab(n + 2, *fOut);
-        fCodeProducer.Tab(n + 2);
-        generateDestroy(&fCodeProducer);
+        fCodeProducer->Tab(n + 2);
+        generateDestroy(fCodeProducer);
         back(1, *fOut);
         *fOut << "}";
         tab(n + 1, *fOut);
@@ -347,19 +370,19 @@ void CPPCodeContainer::produceClass()
 
     tab(n + 1, *fOut);
     // No class name for main class
-    produceInfoFunctions(n + 1, "", "dsp", true, true, &fCodeProducer);  // Inits
+    produceInfoFunctions(n + 1, "", "dsp", true, true, fCodeProducer);  // Inits
 
     // TODO
     /*
-    generateStaticInitFun("classInit", true)->accept(&fCodeProducer);
-    generateInstanceInitFun("instanceInit", true, true)->accept(&fCodeProducer);
+    generateStaticInitFun("classInit", true)->accept(fCodeProducer);
+    generateInstanceInitFun("instanceInit", true, true)->accept(fCodeProducer);
     */
 
     tab(n + 1, *fOut);
     *fOut << "static void classInit(int sample_rate) {";
     tab(n + 2, *fOut);
-    fCodeProducer.Tab(n + 2);
-    generateStaticInit(&fCodeProducer);
+    fCodeProducer->Tab(n + 2);
+    generateStaticInit(fCodeProducer);
     back(1, *fOut);
     *fOut << "}";
 
@@ -367,8 +390,8 @@ void CPPCodeContainer::produceClass()
         tab(n + 1, *fOut);
         *fOut << "static void classDestroy() {";
         tab(n + 2, *fOut);
-        fCodeProducer.Tab(n + 2);
-        generateStaticDestroy(&fCodeProducer);
+        fCodeProducer->Tab(n + 2);
+        generateStaticDestroy(fCodeProducer);
         back(1, *fOut);
         *fOut << "}";
     }
@@ -394,8 +417,8 @@ void CPPCodeContainer::produceClass()
         tab(n+1, *fOut); 
         *fOut << "static void classInit(int sample_rate) {";
             tab(n+2, *fOut);
-            fCodeProducer.Tab(n+2);
-            res1->accept(&fCodeProducer);
+            fCodeProducer->Tab(n+2);
+            res1->accept(fCodeProducer);
         tab(n+1, *fOut); *fOut << "}";
     }
     // End inline
@@ -405,8 +428,8 @@ void CPPCodeContainer::produceClass()
     tab(n + 1, *fOut);
     *fOut << "virtual void instanceConstants(int sample_rate) {";
     tab(n + 2, *fOut);
-    fCodeProducer.Tab(n + 2);
-    generateInit(&fCodeProducer);
+    fCodeProducer->Tab(n + 2);
+    generateInit(fCodeProducer);
     back(1, *fOut);
     *fOut << "}";
     tab(n + 1, *fOut);
@@ -414,8 +437,8 @@ void CPPCodeContainer::produceClass()
     tab(n + 1, *fOut);
     *fOut << "virtual void instanceResetUserInterface() {";
     tab(n + 2, *fOut);
-    fCodeProducer.Tab(n + 2);
-    generateResetUserInterface(&fCodeProducer);
+    fCodeProducer->Tab(n + 2);
+    generateResetUserInterface(fCodeProducer);
     back(1, *fOut);
     *fOut << "}";
     tab(n + 1, *fOut);
@@ -423,8 +446,8 @@ void CPPCodeContainer::produceClass()
     tab(n + 1, *fOut);
     *fOut << "virtual void instanceClear() {";
     tab(n + 2, *fOut);
-    fCodeProducer.Tab(n + 2);
-    generateClear(&fCodeProducer);
+    fCodeProducer->Tab(n + 2);
+    generateClear(fCodeProducer);
     back(1, *fOut);
     *fOut << "}";
     tab(n + 1, *fOut);
@@ -452,9 +475,9 @@ void CPPCodeContainer::produceClass()
         tab(n+1, *fOut); 
         *fOut << "virtual void instanceInit(int sample_rate) {";
         tab(n+2, *fOut);
-        fCodeProducer.Tab(n+2);
-        res1->accept(&fCodeProducer);
-        res2->accept(&fCodeProducer);
+        fCodeProducer->Tab(n+2);
+        res1->accept(fCodeProducer);
+        res2->accept(fCodeProducer);
         tab(n+1, *fOut); *fOut << "}";
     }
     // End inline
@@ -472,16 +495,16 @@ void CPPCodeContainer::produceClass()
     *fOut << "}";
 
     tab(n + 1, *fOut);
-    fCodeProducer.Tab(n + 1);
+    fCodeProducer->Tab(n + 1);
     tab(n + 1, *fOut);
-    generateGetSampleRate("getSampleRate", "dsp", true, true)->accept(&fCodeProducer);
+    generateGetSampleRate("getSampleRate", "dsp", true, true)->accept(fCodeProducer);
 
     // User interface
     tab(n + 1, *fOut);
     *fOut << "virtual void buildUserInterface(UI* ui_interface) {";
     tab(n + 2, *fOut);
-    fCodeProducer.Tab(n + 2);
-    generateUserInterface(&fCodeProducer);
+    fCodeProducer->Tab(n + 2);
+    generateUserInterface(fCodeProducer);
     back(1, *fOut);
     *fOut << "}";
 
@@ -506,70 +529,8 @@ void CPPCodeContainer::produceClass()
     }
 }
 
-// Scalar
-CPPScalarCodeContainer::CPPScalarCodeContainer(const string& name, const string& super, int numInputs, int numOutputs,
-                                               std::ostream* out, int sub_container_type)
-    : CPPCodeContainer(name, super, numInputs, numOutputs, out)
-{
-    fSubContainerType = sub_container_type;
-}
-
-void CPPScalarCodeContainer::generateCompute(int n)
-{
-    // Generates declaration
-    tab(n + 1, *fOut);
-    tab(n + 1, *fOut);
-    *fOut << subst("virtual void compute(int $0, $1** inputs, $1** outputs) {", fFullCount, xfloat());
-    tab(n + 2, *fOut);
-    fCodeProducer.Tab(n + 2);
-
-    // Generates local variables declaration and setup
-    generateComputeBlock(&fCodeProducer);
-   
-    // Generates one single scalar loop
-    ForLoopInst* loop = fCurLoop->generateScalarLoop(fFullCount);
-    loop->accept(&fCodeProducer);
-   
-    /*
-     // TODO : atomic switch
-     // Currently for soundfile management
-     */
-    generatePostComputeBlock(&fCodeProducer);
-
-    back(1, *fOut);
-    *fOut << "}";
-}
-
-// One Sample Scalar
-
-void CPPScalarOneSampleCodeContainer::generateCompute(int n)
-{
-    // Generates declaration
-    tab(n + 1, *fOut);
-    tab(n + 1, *fOut);
-    if (gGlobal->gInPlace) {
-        *fOut << subst("virtual void compute($0* inputs, $0* outputs, int* RESTRICT iControl, $0* RESTRICT fControl) {", xfloat());
-    } else {
-        *fOut << subst("virtual void compute($0* RESTRICT inputs, $0* RESTRICT outputs, int* RESTRICT iControl, $0* RESTRICT fControl) {", xfloat());
-    }
-    tab(n + 2, *fOut);
-    fCodeProducer.Tab(n + 2);
-    
-    // Generates one sample computation
-    BlockInst* block = fCurLoop->generateOneSample();
-    block->accept(&fCodeProducer);
-    
-    /*
-     // TODO : atomic switch
-     // Currently for soundfile management
-     */
-    generatePostComputeBlock(&fCodeProducer);
-    
-    back(1, *fOut);
-    *fOut << "}";
-}
-
-void CPPScalarOneSampleCodeContainer::produceClass()
+// Used with -os0 option
+void CPPScalarOneSampleCodeContainer1::produceClass()
 {
     int n = 0;
    
@@ -582,13 +543,13 @@ void CPPScalarOneSampleCodeContainer::produceClass()
         *fOut << "namespace " << gGlobal->gNameSpace << " {" << endl;
     }
  
-    // Sub containers
-    generateSubContainers();
+    // Sub containers are merged in the main class
+    mergeSubContainers();
     
     // Global declarations
     tab(n, *fOut);
-    fCodeProducer.Tab(n);
-    generateGlobalDeclarations(&fCodeProducer);
+    fCodeProducer->Tab(n);
+    generateGlobalDeclarations(fCodeProducer);
     
     tab(n, *fOut);
     *fOut << "#ifndef FAUSTCLASS " << endl;
@@ -609,14 +570,12 @@ void CPPScalarOneSampleCodeContainer::produceClass()
     *fOut << "#endif" << endl;
     tab(n, *fOut);
     
-    *fOut << "#define FAUST_INPUTS " << fNumInputs << endl;
-    *fOut << "#define FAUST_OUTPUTS  " << fNumOutputs << endl;
     *fOut << "#define FAUST_INT_CONTROLS " << fInt32ControlNum  << endl;
     *fOut << "#define FAUST_REAL_CONTROLS " << fRealControlNum;
     tab(n, *fOut);
-    fSuperKlassName = "one_sample_dsp";
     
     tab(n, *fOut);
+    fSuperKlassName = "one_sample_dsp";
     *fOut << "class " << fKlassName << " : public " << fSuperKlassName << " {";
     tab(n + 1, *fOut);
     
@@ -630,16 +589,16 @@ void CPPScalarOneSampleCodeContainer::produceClass()
     tab(n + 1, *fOut);
     
     // Fields
-    fCodeProducer.Tab(n + 1);
+    fCodeProducer->Tab(n + 1);
     tab(n + 1, *fOut);
-    generateDeclarations(&fCodeProducer);
+    generateDeclarations(fCodeProducer);
     
     if (fAllocateInstructions->fCode.size() > 0) {
         tab(n + 1, *fOut);
         *fOut << "void allocate() {";
         tab(n + 2, *fOut);
-        fCodeProducer.Tab(n + 2);
-        generateAllocate(&fCodeProducer);
+        fCodeProducer->Tab(n + 2);
+        generateAllocate(fCodeProducer);
         back(1, *fOut);
         *fOut << "}";
         tab(n + 1, *fOut);
@@ -649,8 +608,8 @@ void CPPScalarOneSampleCodeContainer::produceClass()
         tab(n + 1, *fOut);
         *fOut << "void destroy() {";
         tab(n + 2, *fOut);
-        fCodeProducer.Tab(n + 2);
-        generateDestroy(&fCodeProducer);
+        fCodeProducer->Tab(n + 2);
+        generateDestroy(fCodeProducer);
         back(1, *fOut);
         *fOut << "}";
         tab(n + 1, *fOut);
@@ -686,15 +645,31 @@ void CPPScalarOneSampleCodeContainer::produceClass()
         *fOut << "}" << endl;
     }
     
+    // Constructor
+    tab(n + 1, *fOut);
+    *fOut << fKlassName << "() {}";
+    tab(n + 1, *fOut);
+    *fOut << fKlassName << "(int* icontrol, FAUSTFLOAT* fcontrol)";
+    *fOut << ":one_sample_dsp(icontrol, fcontrol) {}";
     tab(n + 1, *fOut);
-    // No class name for main class
-    produceInfoFunctions(n + 1, "", "dsp", true, true, &fCodeProducer);  // Inits
     
     tab(n + 1, *fOut);
-    *fOut << "static void classInit(int sample_rate) {";
-    tab(n + 2, *fOut);
-    fCodeProducer.Tab(n + 2);
-    generateStaticInit(&fCodeProducer);
+    // No class name for main class
+    produceInfoFunctions(n + 1, "", "dsp", true, true, fCodeProducer);  // Inits
+    
+    // Dummy
+    tab(n + 1, *fOut);
+    *fOut << "static void classInit(int sample_rate) {}";
+    tab(n, *fOut);
+    
+    tab(n + 1, *fOut);
+    *fOut << "void staticInit(int sample_rate) {";
+    {
+        tab(n + 2, *fOut);
+        fCodeProducer->Tab(n + 2);
+        // Rename 'sig' in 'dsp', remove 'dsp' allocation, inline subcontainers 'instanceInit' and 'fill' function call
+        inlineSubcontainersFunCalls(fStaticInitInstructions)->accept(fCodeProducer);
+    }
     back(1, *fOut);
     *fOut << "}";
     
@@ -702,8 +677,8 @@ void CPPScalarOneSampleCodeContainer::produceClass()
         tab(n + 1, *fOut);
         *fOut << "static void classDestroy() {";
         tab(n + 2, *fOut);
-        fCodeProducer.Tab(n + 2);
-        generateStaticDestroy(&fCodeProducer);
+        fCodeProducer->Tab(n + 2);
+        generateStaticDestroy(fCodeProducer);
         back(1, *fOut);
         *fOut << "}";
     }
@@ -712,8 +687,8 @@ void CPPScalarOneSampleCodeContainer::produceClass()
     tab(n + 1, *fOut);
     *fOut << "virtual void instanceConstants(int sample_rate) {";
     tab(n + 2, *fOut);
-    fCodeProducer.Tab(n + 2);
-    generateInit(&fCodeProducer);
+    fCodeProducer->Tab(n + 2);
+    generateInit(fCodeProducer);
     back(1, *fOut);
     *fOut << "}";
     tab(n + 1, *fOut);
@@ -721,8 +696,8 @@ void CPPScalarOneSampleCodeContainer::produceClass()
     tab(n + 1, *fOut);
     *fOut << "virtual void instanceResetUserInterface() {";
     tab(n + 2, *fOut);
-    fCodeProducer.Tab(n + 2);
-    generateResetUserInterface(&fCodeProducer);
+    fCodeProducer->Tab(n + 2);
+    generateResetUserInterface(fCodeProducer);
     back(1, *fOut);
     *fOut << "}";
     tab(n + 1, *fOut);
@@ -730,16 +705,41 @@ void CPPScalarOneSampleCodeContainer::produceClass()
     tab(n + 1, *fOut);
     *fOut << "virtual void instanceClear() {";
     tab(n + 2, *fOut);
-    fCodeProducer.Tab(n + 2);
-    generateClear(&fCodeProducer);
+    fCodeProducer->Tab(n + 2);
+    generateClear(fCodeProducer);
     back(1, *fOut);
     *fOut << "}";
     tab(n + 1, *fOut);
     
     // Init
-    produceInit(n + 1);
+    if (gGlobal->gMemoryManager) {
+        tab(n + 1, *fOut);
+        *fOut << "virtual void init(int sample_rate) {}";
+    } else {
+        tab(n + 1, *fOut);
+        *fOut << "virtual void init(int sample_rate) {";
+        tab(n + 2, *fOut);
+        *fOut << "instanceInit(sample_rate);";
+        tab(n + 1, *fOut);
+        *fOut << "}";
+    }
+    tab(n + 1, *fOut);
     
     tab(n + 1, *fOut);
+    *fOut << "virtual void instanceInit(int sample_rate) {";
+    // staticInit has to be called for each instance since the tables are actually not shared between instances
+    tab(n + 2, *fOut);
+    *fOut << "staticInit(sample_rate);";
+    tab(n + 2, *fOut);
+    *fOut << "instanceConstants(sample_rate);";
+    tab(n + 2, *fOut);
+    *fOut << "instanceResetUserInterface();";
+    tab(n + 2, *fOut);
+    *fOut << "instanceClear();";
+    tab(n + 1, *fOut);
+    *fOut << "}";
+    tab(n + 1, *fOut);
+    
     tab(n + 1, *fOut);
     *fOut << "virtual " << fKlassName << "* clone() {";
     tab(n + 2, *fOut);
@@ -748,25 +748,26 @@ void CPPScalarOneSampleCodeContainer::produceClass()
     *fOut << "}";
     
     tab(n + 1, *fOut);
-    fCodeProducer.Tab(n + 1);
+    fCodeProducer->Tab(n + 1);
     tab(n + 1, *fOut);
-    generateGetSampleRate("getSampleRate", "dsp", true, true)->accept(&fCodeProducer);
+    generateGetSampleRate("getSampleRate", "dsp", true, true)->accept(fCodeProducer);
     
     // User interface
     tab(n + 1, *fOut);
     *fOut << "virtual void buildUserInterface(UI* ui_interface) {";
     tab(n + 2, *fOut);
-    fCodeProducer.Tab(n + 2);
-    generateUserInterface(&fCodeProducer);
+    fCodeProducer->Tab(n + 2);
+    generateUserInterface(fCodeProducer);
     back(1, *fOut);
     *fOut << "}";
+    tab(n + 1, *fOut);
     
     tab(n + 1, *fOut);
     *fOut << subst("virtual void control(int* RESTRICT iControl, $0* RESTRICT fControl) {", xfloat());
     tab(n + 2, *fOut);
-    fCodeProducer.Tab(n + 2);
+    fCodeProducer->Tab(n + 2);
     // Generates local variables declaration and setup
-    generateComputeBlock(&fCodeProducer);
+    generateComputeBlock(fCodeProducer);
     back(1, *fOut);
     *fOut << "}" << endl;
     
@@ -796,6 +797,374 @@ void CPPScalarOneSampleCodeContainer::produceClass()
     }
 }
 
+// Used with -os1 option
+void CPPScalarOneSampleCodeContainer2::produceClass()
+{
+    int n = 0;
+    
+    // Libraries
+    printLibrary(*fOut);
+    printIncludeFile(*fOut);
+    
+    if (gGlobal->gNameSpace != "" && gGlobal->gArchFile == "") {
+        tab(n, *fOut);
+        *fOut << "namespace " << gGlobal->gNameSpace << " {" << endl;
+    }
+    
+    // Sub containers are merged in the main class
+    mergeSubContainers();
+    
+    // Global declarations
+    tab(n, *fOut);
+    fCodeProducer->Tab(n);
+    generateGlobalDeclarations(fCodeProducer);
+    
+    tab(n, *fOut);
+    *fOut << "#ifndef FAUSTCLASS " << endl;
+    *fOut << "#define FAUSTCLASS " << fKlassName << endl;
+    *fOut << "#endif" << endl;
+    tab(n, *fOut);
+    
+    *fOut << "#ifdef __APPLE__ " << endl;
+    *fOut << "#define exp10f __exp10f" << endl;
+    *fOut << "#define exp10 __exp10" << endl;
+    *fOut << "#endif" << endl;
+    tab(n, *fOut);
+    
+    *fOut << "#if defined(_WIN32)" << endl;
+    *fOut << "#define RESTRICT __restrict" << endl;
+    *fOut << "#else" << endl;
+    *fOut << "#define RESTRICT __restrict__" << endl;
+    *fOut << "#endif" << endl;
+    
+    tab(n, *fOut);
+    fSuperKlassName = "one_sample_dsp_real<" + string(ifloat()) + ">";
+    *fOut << "class " << fKlassName << " : public " << fSuperKlassName << " {";
+    tab(n + 1, *fOut);
+    
+    if (gGlobal->gUIMacroSwitch) {
+        tab(n, *fOut);
+        *fOut << " public:";
+    } else {
+        tab(n, *fOut);
+        *fOut << " private:";
+    }
+    tab(n + 1, *fOut);
+    
+    // Fields
+    fCodeProducer->Tab(n + 1);
+    tab(n + 1, *fOut);
+    generateDeclarations(fCodeProducer);
+    
+    // Kept here because staticInit incorrectly change the size later on
+    int int_zone_size = static_cast<CPPInstVisitor1*>(fCodeProducer)->getIntZoneSize();
+    int real_zone_size = static_cast<CPPInstVisitor1*>(fCodeProducer)->getRealZoneSize();
+    
+    if (fAllocateInstructions->fCode.size() > 0) {
+        tab(n + 1, *fOut);
+        *fOut << "void allocate() {";
+        tab(n + 2, *fOut);
+        fCodeProducer->Tab(n + 2);
+        generateAllocate(fCodeProducer);
+        back(1, *fOut);
+        *fOut << "}";
+        tab(n + 1, *fOut);
+    }
+    
+    if (fDestroyInstructions->fCode.size() > 0) {
+        tab(n + 1, *fOut);
+        *fOut << "void destroy() {";
+        tab(n + 2, *fOut);
+        fCodeProducer->Tab(n + 2);
+        generateDestroy(fCodeProducer);
+        back(1, *fOut);
+        *fOut << "}";
+        tab(n + 1, *fOut);
+    }
+    
+    tab(n, *fOut);
+    *fOut << " public:";
+    
+    if (gGlobal->gMemoryManager) {
+        tab(n + 1, *fOut);
+        *fOut << "static dsp_memory_manager* fManager;";
+    }
+    
+    // Print metadata declaration
+    tab(n + 1, *fOut);
+    produceMetadata(n + 1);
+    
+    if (fAllocateInstructions->fCode.size() > 0) {
+        tab(n + 1, *fOut);
+        *fOut << fKlassName << "() {";
+        tab(n + 2, *fOut);
+        *fOut << "allocate();";
+        tab(n + 1, *fOut);
+        *fOut << "}" << endl;
+    }
+    
+    if (fDestroyInstructions->fCode.size() > 0) {
+        tab(n + 1, *fOut);
+        *fOut << "virtual ~" << fKlassName << "() {";
+        tab(n + 2, *fOut);
+        *fOut << "destroy();";
+        tab(n + 1, *fOut);
+        *fOut << "}" << endl;
+    }
+    
+    // Constructor
+    tab(n + 1, *fOut);
+    *fOut << fKlassName << "() {}";
+    tab(n + 1, *fOut);
+    *fOut << fKlassName << "(int* icontrol, FAUSTFLOAT* fcontrol, int* izone, " << ifloat() << "* fzone)";
+    *fOut << ":one_sample_dsp_real(icontrol, fcontrol, izone, fzone) {}";
+    tab(n + 1, *fOut);
+    
+    tab(n + 1, *fOut);
+    // No class name for main class
+    produceInfoFunctions(n + 1, "", "dsp", true, true, fCodeProducer);  // Inits
+    
+    // Dummy
+    tab(n + 1, *fOut);
+    *fOut << "static void classInit(int sample_rate) {}";
+    tab(n + 1, *fOut);
+    
+    tab(n + 1, *fOut);
+    *fOut << "void staticInit(int sample_rate, " << subst("int* iZone, $0* fZone) {", ifloat());
+    {
+        tab(n + 2, *fOut);
+        fCodeProducer->Tab(n + 2);
+        // For waveform
+        MoveVariablesInFront3().getCode(fGlobalDeclarationInstructions)->accept(fCodeProducer);
+        // Rename 'sig' in 'dsp', remove 'dsp' allocation, inline subcontainers 'instanceInit' and 'fill' function call
+        inlineSubcontainersFunCalls(fStaticInitInstructions)->accept(fCodeProducer);
+    }
+    back(1, *fOut);
+    *fOut << "}";
+    
+    if (gGlobal->gMemoryManager) {
+        tab(n + 1, *fOut);
+        *fOut << "static void classDestroy() {";
+        tab(n + 2, *fOut);
+        fCodeProducer->Tab(n + 2);
+        generateStaticDestroy(fCodeProducer);
+        back(1, *fOut);
+        *fOut << "}";
+    }
+    
+    tab(n + 1, *fOut);
+    tab(n + 1, *fOut);
+    *fOut << "virtual void instanceConstants(int sample_rate, " << subst("int* iZone, $0* fZone) {", ifloat());
+    tab(n + 2, *fOut);
+    fCodeProducer->Tab(n + 2);
+    generateInit(fCodeProducer);
+    back(1, *fOut);
+    *fOut << "}";
+    tab(n + 1, *fOut);
+    
+    tab(n + 1, *fOut);
+    *fOut << "virtual void instanceResetUserInterface() {";
+    tab(n + 2, *fOut);
+    fCodeProducer->Tab(n + 2);
+    generateResetUserInterface(fCodeProducer);
+    back(1, *fOut);
+    *fOut << "}";
+    tab(n + 1, *fOut);
+    
+    tab(n + 1, *fOut);
+    *fOut << "virtual void instanceClear(" << subst("int* iZone, $0* fZone) {", ifloat());
+    tab(n + 2, *fOut);
+    fCodeProducer->Tab(n + 2);
+    generateClear(fCodeProducer);
+    back(1, *fOut);
+    *fOut << "}";
+    tab(n + 1, *fOut);
+    
+    // Init
+    if (gGlobal->gMemoryManager) {
+        tab(n + 1, *fOut);
+        *fOut << "virtual void init(int sample_rate, " << subst("int* iZone, $0* fZone) {", ifloat());
+    } else {
+        tab(n + 1, *fOut);
+        *fOut << "virtual void init(int sample_rate, " << subst("int* iZone, $0* fZone) {", ifloat());
+        tab(n + 2, *fOut);
+        *fOut << "instanceInit(sample_rate, iZone, fZone);";
+        tab(n + 1, *fOut);
+        *fOut << "}";
+    }
+    tab(n + 1, *fOut);
+    
+    tab(n + 1, *fOut);
+    *fOut << "virtual void instanceInit(int sample_rate, " << subst("int* iZone, $0* fZone) {", ifloat());
+    // staticInit has to be called for each instance since the tables are actually not shared between instances
+    tab(n + 2, *fOut);
+    *fOut << "staticInit(sample_rate, iZone, fZone);";
+    tab(n + 2, *fOut);
+    *fOut << "instanceConstants(sample_rate, iZone, fZone);";
+    tab(n + 2, *fOut);
+    *fOut << "instanceResetUserInterface();";
+    tab(n + 2, *fOut);
+    *fOut << "instanceClear(iZone, fZone);";
+    tab(n + 1, *fOut);
+    *fOut << "}";
+    tab(n + 1, *fOut);
+    
+    tab(n + 1, *fOut);
+    *fOut << "virtual " << fKlassName << "* clone() {";
+    tab(n + 2, *fOut);
+    *fOut << "return new " << fKlassName << "();";
+    tab(n + 1, *fOut);
+    *fOut << "}";
+    
+    tab(n + 1, *fOut);
+    fCodeProducer->Tab(n + 1);
+    tab(n + 1, *fOut);
+    generateGetSampleRate("getSampleRate", "dsp", true, true)->accept(fCodeProducer);
+    
+    // User interface
+    tab(n + 1, *fOut);
+    *fOut << "virtual void buildUserInterface(UI* ui_interface) {";
+    tab(n + 2, *fOut);
+    fCodeProducer->Tab(n + 2);
+    generateUserInterface(fCodeProducer);
+    back(1, *fOut);
+    *fOut << "}";
+    tab(n + 1, *fOut);
+    
+    tab(n + 1, *fOut);
+    *fOut << subst("virtual void control(int* RESTRICT iControl, $0* RESTRICT fControl, int* RESTRICT iZone, $1* RESTRICT fZone) {", xfloat(), ifloat());
+    tab(n + 2, *fOut);
+    fCodeProducer->Tab(n + 2);
+    // Generates local variables declaration and setup
+    generateComputeBlock(fCodeProducer);
+    back(1, *fOut);
+    *fOut << "}" << endl;
+    
+    tab(n + 1, *fOut);
+    *fOut << "virtual int getNumIntControls() { return " << fInt32ControlNum << "; }";
+    tab(n + 1, *fOut);
+    
+    *fOut << "virtual int getNumRealControls() { return " << fRealControlNum << "; }";
+    tab(n + 1, *fOut);
+    
+    tab(n + 1, *fOut);
+    *fOut << "virtual int getiZoneSize() { return " << int_zone_size << "; }";
+    
+    tab(n + 1, *fOut);
+    *fOut << "virtual int getfZoneSize() { return " << real_zone_size << "; }";
+    
+    // Compute
+    generateCompute(n);
+    tab(n, *fOut);
+    tab(n, *fOut);
+    *fOut << "};" << endl;
+    
+    tab(n, *fOut);
+    *fOut << "#define FAUST_INT_CONTROLS " << fInt32ControlNum << endl;
+    *fOut << "#define FAUST_REAL_CONTROLS " << fRealControlNum << endl;
+    
+    tab(n, *fOut);
+    *fOut << "#define FAUST_INT_ZONE " << int_zone_size << endl;
+    *fOut << "#define FAUST_FLOAT_ZONE " << real_zone_size;
+    tab(n, *fOut);
+    
+    // To improve (generalization for all backends...)
+    if (gGlobal->gMemoryManager) {
+        tab(n, *fOut);
+        *fOut << "dsp_memory_manager* " << fKlassName << "::fManager = 0;" << endl;
+    }
+    
+    // Generate user interface macros if needed
+    printMacros(*fOut, n);
+    
+    if (gGlobal->gNameSpace != "" && gGlobal->gArchFile == "") {
+        tab(n, *fOut);
+        *fOut << "} // namespace " << gGlobal->gNameSpace << endl;
+    }
+}
+
+void CPPScalarCodeContainer::generateCompute(int n)
+{
+    // Generates declaration
+    tab(n + 1, *fOut);
+    tab(n + 1, *fOut);
+    *fOut << subst("virtual void compute(int $0, $1** inputs, $1** outputs) {", fFullCount, xfloat());
+    tab(n + 2, *fOut);
+    fCodeProducer->Tab(n + 2);
+    
+    // Generates local variables declaration and setup
+    generateComputeBlock(fCodeProducer);
+    
+    // Generates one single scalar loop
+    ForLoopInst* loop = fCurLoop->generateScalarLoop(fFullCount);
+    loop->accept(fCodeProducer);
+    
+    /*
+     // TODO : atomic switch
+     // Currently for soundfile management
+     */
+    generatePostComputeBlock(fCodeProducer);
+    
+    back(1, *fOut);
+    *fOut << "}";
+}
+
+// Used with -os0 option
+void CPPScalarOneSampleCodeContainer1::generateCompute(int n)
+{
+    // Generates declaration
+    tab(n + 1, *fOut);
+    tab(n + 1, *fOut);
+    if (gGlobal->gInPlace) {
+        *fOut << subst("virtual void compute($0* inputs, $0* outputs, int* RESTRICT iControl, $0* RESTRICT fControl) {", xfloat());
+    } else {
+        *fOut << subst("virtual void compute($0* RESTRICT inputs, $0* RESTRICT outputs, int* RESTRICT iControl, $0* RESTRICT fControl) {", xfloat());
+    }
+    tab(n + 2, *fOut);
+    fCodeProducer->Tab(n + 2);
+    
+    // Generates one sample computation
+    BlockInst* block = fCurLoop->generateOneSample();
+    block->accept(fCodeProducer);
+    
+    /*
+     // TODO : atomic switch
+     // Currently for soundfile management
+     */
+    generatePostComputeBlock(fCodeProducer);
+    
+    back(1, *fOut);
+    *fOut << "}";
+}
+
+// Used with -os1 option
+void CPPScalarOneSampleCodeContainer2::generateCompute(int n)
+{
+    // Generates declaration
+    tab(n + 1, *fOut);
+    tab(n + 1, *fOut);
+    if (gGlobal->gInPlace) {
+        *fOut << subst("virtual void compute($0* inputs, $0* outputs, int* RESTRICT iControl, $0* RESTRICT fControl, int* RESTRICT iZone, $1* RESTRICT fZone) {", xfloat(), ifloat());
+    } else {
+        *fOut << subst("virtual void compute($0* RESTRICT inputs, $0* RESTRICT outputs, int* RESTRICT iControl, $0* RESTRICT fControl, int* RESTRICT iZone, $1* RESTRICT fZone) {", xfloat(), ifloat());
+    }
+    tab(n + 2, *fOut);
+    fCodeProducer->Tab(n + 2);
+    
+    // Generates one sample computation
+    BlockInst* block = fCurLoop->generateOneSample();
+    block->accept(fCodeProducer);
+    
+    /*
+     // TODO : atomic switch
+     // Currently for soundfile management
+     */
+    generatePostComputeBlock(fCodeProducer);
+    
+    back(1, *fOut);
+    *fOut << "}";
+}
+
 // Vector
 CPPVectorCodeContainer::CPPVectorCodeContainer(const string& name, const string& super, int numInputs, int numOutputs,
                                                std::ostream* out)
@@ -806,21 +1175,21 @@ CPPVectorCodeContainer::CPPVectorCodeContainer(const string& name, const string&
 void CPPVectorCodeContainer::generateCompute(int n)
 {
     // Possibly generate separated functions
-    fCodeProducer.Tab(n + 1);
+    fCodeProducer->Tab(n + 1);
     tab(n + 1, *fOut);
-    generateComputeFunctions(&fCodeProducer);
+    generateComputeFunctions(fCodeProducer);
 
     // Generates declaration
     tab(n + 1, *fOut);
     *fOut << subst("virtual void compute(int $0, $1** inputs, $1** outputs) {", fFullCount, xfloat());
     tab(n + 2, *fOut);
-    fCodeProducer.Tab(n + 2);
+    fCodeProducer->Tab(n + 2);
 
     // Generates local variables declaration and setup
-    generateComputeBlock(&fCodeProducer);
+    generateComputeBlock(fCodeProducer);
 
     // Generates the DSP loop
-    fDAGBlock->accept(&fCodeProducer);
+    fDAGBlock->accept(fCodeProducer);
 
     back(1, *fOut);
     *fOut << "}";
@@ -836,21 +1205,21 @@ CPPOpenMPCodeContainer::CPPOpenMPCodeContainer(const string& name, const string&
 void CPPOpenMPCodeContainer::generateCompute(int n)
 {
     // Possibly generate separated functions
-    fCodeProducer.Tab(n + 1);
+    fCodeProducer->Tab(n + 1);
     tab(n + 1, *fOut);
-    generateComputeFunctions(&fCodeProducer);
+    generateComputeFunctions(fCodeProducer);
 
     // Generates declaration
     tab(n + 1, *fOut);
     *fOut << subst("virtual void compute(int $0, $1** inputs, $1** outputs) {", fFullCount, xfloat());
     tab(n + 2, *fOut);
-    fCodeProducer.Tab(n + 2);
+    fCodeProducer->Tab(n + 2);
 
     // Generates local variables declaration and setup
-    generateComputeBlock(&fCodeProducer);
+    generateComputeBlock(fCodeProducer);
 
     // Generate DSP loop
-    fGlobalLoopBlock->accept(&fCodeProducer);
+    fGlobalLoopBlock->accept(fCodeProducer);
 
     back(1, *fOut);
     *fOut << "}";
@@ -881,18 +1250,18 @@ void CPPWorkStealingCodeContainer::produceClass()
 void CPPWorkStealingCodeContainer::generateCompute(int n)
 {
     // Possibly generate separated functions
-    fCodeProducer.Tab(n + 1);
+    fCodeProducer->Tab(n + 1);
     tab(n + 1, *fOut);
-    generateComputeFunctions(&fCodeProducer);
+    generateComputeFunctions(fCodeProducer);
 
     // Generates declaration
     tab(n + 1, *fOut);
     *fOut << subst("virtual void compute(int $0, $1** inputs, $1** outputs) {", fFullCount, xfloat());
     tab(n + 2, *fOut);
-    fCodeProducer.Tab(n + 2);
+    fCodeProducer->Tab(n + 2);
 
     // Generates local variables declaration and setup
-    generateComputeBlock(&fCodeProducer);
+    generateComputeBlock(fCodeProducer);
 
     //tab(n + 1, *fOut);
     back(1, *fOut);
@@ -902,11 +1271,11 @@ void CPPWorkStealingCodeContainer::generateCompute(int n)
     tab(n + 1, *fOut);
     *fOut << "void computeThread" << fKlassName << "(int num_thread) {";
     tab(n + 2, *fOut);
-    fCodeProducer.Tab(n + 2);
+    fCodeProducer->Tab(n + 2);
 
     // Generate it
     faustassert(fThreadLoopBlock);
-    fThreadLoopBlock->accept(&fCodeProducer);
+    fThreadLoopBlock->accept(fCodeProducer);
 
     back(1, *fOut);
     *fOut << "}";
