@@ -66,16 +66,17 @@ Typed::VarType ctType(Type t)
     return (t->nature() == kInt) ? Typed::kInt32 : Typed::kFloat;
 }
 
-string Typed::gTypeString[] = {"kInt32",          "kInt32_ptr",    "kInt32_vec",      "kInt32_vec_ptr",
-                               "kInt64",          "kInt64_ptr",    "kInt64_vec",      "kInt64_vec_ptr",
-                               "kBool",           "kBool_ptr",     "kBool_vec",       "kBool_vec_ptr",
-                               "kFloat",          "kFloat_ptr",    "kFloat_ptr_ptr",  "kFloat_vec",
-                               "kFloat_vec_ptr",  "kFloatMacro",   "kFloatMacro_ptr", "kFloatMacro_ptr_ptr",
-                               "kDouble",         "kDouble_ptr",   "kDouble_ptr_ptr", "kDouble_vec",
-                               "kDouble_vec_ptr", "kQuad",         "kQuad_ptr",       "kQuad_ptr_ptr",
-                               "kQuad_vec",       "kQuad_vec_ptr", "kVoid",           "kVoid_ptr",
-                               "kVoid_ptr_ptr",   "kObj",          "kObj_ptr",        "kSound",
-                               "kSound_ptr",      "kUint_ptr",     "kNoType"};
+string Typed::gTypeString[] = {"kInt32",          "kInt32_ptr",      "kInt32_vec",          "kInt32_vec_ptr",
+                               "kInt64",          "kInt64_ptr",      "kInt64_vec",          "kInt64_vec_ptr",
+                               "kBool",           "kBool_ptr",       "kBool_vec",           "kBool_vec_ptr",
+                               "kFloat",          "kFloat_ptr",      "kFloat_ptr_ptr",      "kFloat_vec",
+                               "kFloat_vec_ptr",  "kFloatMacro",     "kFloatMacro_ptr",     "kFloatMacro_ptr_ptr",
+                               "kDouble",         "kDouble_ptr",     "kDouble_ptr_ptr",     "kDouble_vec",       "kDouble_vec_ptr",
+                               "kQuad",           "kQuad_ptr",       "kQuad_ptr_ptr",       "kQuad_vec",         "kQuad_vec_ptr",
+                               "kFixedPoint",     "kFixedPoint_ptr", "kFixedPoint_ptr_ptr", "kFixedPoint_vec",   "kFixedPoint_vec_ptr",
+                               "kVoid",           "kVoid_ptr",       "kVoid_ptr_ptr",
+                               "kObj",            "kObj_ptr",        "kSound",
+                               "kSound_ptr",      "kUint_ptr",       "kNoType"};
 
 void BasicTyped::cleanup()
 {
@@ -140,14 +141,7 @@ DeclareFunInst::DeclareFunInst(const string& name, FunTyped* type, BlockInst* co
 
 BasicTyped* InstBuilder::genBasicTyped(Typed::VarType type)
 {
-    // Possibly force FAUSTFLOAT type (= kFloatMacro) to internal real
-    Typed::VarType new_type = ((type == Typed::kFloatMacro) && gGlobal->gFAUSTFLOAT2Internal) ? itfloat() : type;
-
-    // If not defined, add the type in the table
-    if (gGlobal->gTypeTable.find(new_type) == gGlobal->gTypeTable.end()) {
-        gGlobal->gTypeTable[new_type] = new BasicTyped(new_type);
-    }
-    return gGlobal->gTypeTable[new_type];
+    return gGlobal->genBasicTyped(type);
 }
 
 int BasicTyped::getSize() const
@@ -352,6 +346,13 @@ DeclareFunInst* InstBuilder::genFunction6(const string& name, Typed::VarType res
     return InstBuilder::genDeclareFunInst(name, fun_type, code);
 }
 
+bool LoadVarInst::isSimpleValue() const
+{
+    NamedAddress* named = dynamic_cast<NamedAddress*>(fAddress);
+    IndexedAddress* indexed = dynamic_cast<IndexedAddress*>(fAddress);
+    return named || (indexed && dynamic_cast<Int32NumInst*>(indexed->getIndex()));
+}
+
 void ScalVecDispatcherVisitor::Dispatch2Visitor(ValueInst* inst)
 {
     std::cout << "Dispatch2Visitor %d\n";
