@@ -65,7 +65,7 @@
 #include <llvm/Bitcode/BitcodeReader.h>
 #include <llvm/Bitcode/BitcodeWriter.h>
 
-#if defined(LLVM_100) || defined(LLVM_110) || defined(LLVM_120)
+#if defined(LLVM_100) || defined(LLVM_110) || defined(LLVM_120) || defined(LLVM_130) || defined(LLVM_140)
 #include <llvm/InitializePasses.h>
 #include <llvm/Support/CodeGen.h>
 #endif
@@ -98,14 +98,14 @@ static string getParam(int argc, const char* argv[], const string& param, const
     return def;
 }
 
-static Module* ParseBitcodeFile(MEMORY_BUFFER Buffer, LLVMContext& Context, string* ErrMsg)
+static Module* ParseBitcodeFile(MEMORY_BUFFER buffer, LLVMContext& context, string& error_msg)
 {
-    Expected<unique_ptr<Module>> ModuleOrErr = parseBitcodeFile(Buffer, Context);
-    if (!ModuleOrErr) {
-        if (ErrMsg) *ErrMsg = "Failed to read bitcode";
+    Expected<unique_ptr<Module>> module_or_err = parseBitcodeFile(buffer, context);
+    if (error_code ec = errorToErrorCode(module_or_err.takeError())) {
+        error_msg = "ERROR : failed to read bitcode\n";
         return nullptr;
     } else {
-        return ModuleOrErr.get().release();
+        return module_or_err.get().release();
     }
 }
 
@@ -114,7 +114,8 @@ void llvm_dynamic_dsp_factory_aux::write(ostream* out, bool binary, bool small)
     string res;
     raw_string_ostream out_str(res);
     if (binary) {
-#if defined(LLVM_80) || defined(LLVM_90) || defined(LLVM_100) || defined(LLVM_110) || defined(LLVM_120)
+#if defined(LLVM_80) || defined(LLVM_90) || defined(LLVM_100) || defined(LLVM_110) || defined(LLVM_120) || \
+    defined(LLVM_130) || defined(LLVM_140)
         WriteBitcodeToFile(*fModule, out_str);
 #else
         WriteBitcodeToFile(fModule, out_str);
@@ -130,7 +131,8 @@ string llvm_dynamic_dsp_factory_aux::writeDSPFactoryToBitcode()
 {
     string res;
     raw_string_ostream out(res);
-#if defined(LLVM_80) || defined(LLVM_90) || defined(LLVM_100) || defined(LLVM_110) || defined(LLVM_120)
+#if defined(LLVM_80) || defined(LLVM_90) || defined(LLVM_100) || defined(LLVM_110) || defined(LLVM_120) || \
+    defined(LLVM_130) || defined(LLVM_140)
     WriteBitcodeToFile(*fModule, out);
 #else
     WriteBitcodeToFile(fModule, out);
@@ -147,7 +149,8 @@ bool llvm_dynamic_dsp_factory_aux::writeDSPFactoryToBitcodeFile(const string& bi
         cerr << "ERROR : writeDSPFactoryToBitcodeFile could not open file : " << err.message();
         return false;
     }
-#if defined(LLVM_80) || defined(LLVM_90) || defined(LLVM_100) || defined(LLVM_110) || defined(LLVM_120)
+#if defined(LLVM_80) || defined(LLVM_90) || defined(LLVM_100) || defined(LLVM_110) || defined(LLVM_120) || \
+    defined(LLVM_130) || defined(LLVM_140)
     WriteBitcodeToFile(*fModule, out);
 #else
     WriteBitcodeToFile(fModule, out);
@@ -285,11 +288,12 @@ bool llvm_dynamic_dsp_factory_aux::initJIT(string& error_msg)
     targetOptions.GuaranteedTailCallOpt = true;
     targetOptions.NoTrappingFPMath      = true;
     
-#if defined(LLVM_90) || defined(LLVM_100) || defined(LLVM_110) || defined(LLVM_120)
+#if defined(LLVM_90) || defined(LLVM_100) || defined(LLVM_110) || defined(LLVM_120) || defined(LLVM_130) || \
+    defined(LLVM_140)
     targetOptions.NoSignedZerosFPMath   = true;
 #endif
     
-#if defined(LLVM_110) || defined(LLVM_120)
+#if defined(LLVM_110) || defined(LLVM_120) || defined(LLVM_130) || defined(LLVM_140)
     targetOptions.setFPDenormalMode(DenormalMode::getIEEE());
 #else
     targetOptions.FPDenormalMode = FPDenormal::IEEE;
@@ -299,9 +303,9 @@ bool llvm_dynamic_dsp_factory_aux::initJIT(string& error_msg)
     
     string debug_var = (getenv("FAUST_DEBUG")) ? string(getenv("FAUST_DEBUG")) : "";
     if ((debug_var != "") && (debug_var.find("FAUST_LLVM3") != string::npos)) {
-    #if !defined(LLVM_120)
+#if !defined(LLVM_120) && !defined(LLVM_130) && !defined(LLVM_140)
         targetOptions.PrintMachineCode = true;
-    #endif
+#endif
     }
 
     builder.setTargetOptions(targetOptions);
@@ -361,7 +365,7 @@ bool llvm_dynamic_dsp_factory_aux::initJIT(string& error_msg)
 
     fObjectCache = new FaustObjectCache();
     fJIT->setObjectCache(fObjectCache);
-    return initJITAux(error_msg);
+    return initJITAux();
 }
 
 // Bitcode <==> string
@@ -369,7 +373,6 @@ static llvm_dsp_factory* readDSPFactoryFromBitcodeAux(MEMORY_BUFFER buffer, cons
                                                       int opt_level)
 {
     string sha_key = generateSHA1(MEMORY_BUFFER_GET(buffer).str());
-
     dsp_factory_table<SDsp_factory>::factory_iterator it;
 
     if (llvm_dsp_factory_aux::gLLVMFactoryTable.getFactory(sha_key, it)) {
@@ -377,21 +380,23 @@ static llvm_dsp_factory* readDSPFactoryFromBitcodeAux(MEMORY_BUFFER buffer, cons
         sfactory->addReference();
         return sfactory;
     } else {
-        LLVMContext* context = new LLVMContext();
-        Module*      module  = ParseBitcodeFile(buffer, *context, &error_msg);
-        if (!module) return nullptr;
-
-        llvm_dynamic_dsp_factory_aux* factory_aux =
-            new llvm_dynamic_dsp_factory_aux(sha_key, module, context, target, opt_level);
-
-        if (factory_aux->initJIT(error_msg)) {
-            llvm_dsp_factory* factory = new llvm_dsp_factory(factory_aux);
-            llvm_dsp_factory_aux::gLLVMFactoryTable.setFactory(factory);
-            factory->setSHAKey(sha_key);
-            return factory;
-        } else {
-            error_msg = "ERROR : " + error_msg;
-            delete factory_aux;
+        try {
+            LLVMContext* context = new LLVMContext();
+            Module*      module  = ParseBitcodeFile(buffer, *context, error_msg);
+            if (!module) return nullptr;
+            llvm_dynamic_dsp_factory_aux* factory_aux =
+                new llvm_dynamic_dsp_factory_aux(sha_key, module, context, target, opt_level);
+            if (factory_aux->initJIT(error_msg)) {
+                llvm_dsp_factory* factory = new llvm_dsp_factory(factory_aux);
+                llvm_dsp_factory_aux::gLLVMFactoryTable.setFactory(factory);
+                factory->setSHAKey(sha_key);
+                return factory;
+            } else {
+                delete factory_aux;
+                return nullptr;
+            }
+        } catch (faustexception& e) {
+            error_msg = e.what();
             return nullptr;
         }
     }
@@ -402,7 +407,6 @@ bool llvm_dynamic_dsp_factory_aux::writeDSPFactoryToObjectcodeFileAux(const stri
 {
     auto TargetTriple = sys::getDefaultTargetTriple();
     fModule->setTargetTriple(TargetTriple);
-
     string Error;
     auto Target = TargetRegistry::lookupTarget(TargetTriple, Error);
 
@@ -416,25 +420,20 @@ bool llvm_dynamic_dsp_factory_aux::writeDSPFactoryToObjectcodeFileAux(const stri
 
     StringRef CPU = sys::getHostCPUName();
     string Features;
-
     TargetOptions opt;
-    
     auto RM = Optional<Reloc::Model>();
     auto TheTargetMachine = Target->createTargetMachine(TargetTriple, CPU, Features, opt, RM);
-
     fModule->setDataLayout(TheTargetMachine->createDataLayout());
 
     error_code EC;
-    raw_fd_ostream  dest(object_code_path.c_str(), EC, sys::fs::F_None);
-
+    raw_fd_ostream  dest(object_code_path.c_str(), EC, sys::fs::OF_None);
     if (EC) {
         errs() << "ERROR : writeDSPFactoryToObjectcodeFile could not open file : " << EC.message();
         return false;
     }
 
     legacy::PassManager pass;
- 
-#if defined(LLVM_100) || defined(LLVM_110) || defined(LLVM_120)
+#if defined(LLVM_100) || defined(LLVM_110) || defined(LLVM_120) || defined(LLVM_130) || defined(LLVM_140)
     if (TheTargetMachine->addPassesToEmitFile(pass, dest, nullptr, CGFT_ObjectFile)) {
 #elif defined(LLVM_80) || defined(LLVM_90)
     if (TheTargetMachine->addPassesToEmitFile(pass, dest, nullptr, TargetMachine::CGFT_ObjectFile)) {
@@ -476,7 +475,6 @@ static llvm_dsp_factory* readDSPFactoryFromIRAux(MEMORY_BUFFER buffer, const str
                                                  int opt_level)
 {
     string sha_key = generateSHA1(MEMORY_BUFFER_GET(buffer).str());
-    
     dsp_factory_table<SDsp_factory>::factory_iterator it;
     
     if (llvm_dsp_factory_aux::gLLVMFactoryTable.getFactory(sha_key, it)) {
@@ -484,36 +482,37 @@ static llvm_dsp_factory* readDSPFactoryFromIRAux(MEMORY_BUFFER buffer, const str
         sfactory->addReference();
         return sfactory;
     } else {
-        char* tmp_local = setlocale(LC_ALL, NULL);
-        if (tmp_local != NULL) {
-            tmp_local = strdup(tmp_local);
-        }
-        setlocale(LC_ALL, "C");
-        LLVMContext* context = new LLVMContext();
-        SMDiagnostic err;
-        // parseIR takes ownership of the given buffer, so don't delete it
-        Module* module = parseIR(buffer, err, *context).release();
-        if (!module) {
-            error_msg = "ERROR : " + string(err.getMessage().data()) + "\n";
-            return nullptr;
-        }
-        if (tmp_local != NULL) {
-            setlocale(LC_ALL, tmp_local);
-            free(tmp_local);
-        }
-        string error_msg;
-        
-        llvm_dynamic_dsp_factory_aux* factory_aux =
-        new llvm_dynamic_dsp_factory_aux(sha_key, module, context, target, opt_level);
-        
-        if (factory_aux->initJIT(error_msg)) {
-            llvm_dsp_factory* factory = new llvm_dsp_factory(factory_aux);
-            llvm_dsp_factory_aux::gLLVMFactoryTable.setFactory(factory);
-            factory->setSHAKey(sha_key);
-            return factory;
-        } else {
-            error_msg = "ERROR : " + error_msg;
-            delete factory_aux;
+        try {
+            char* tmp_local = setlocale(LC_ALL, NULL);
+            if (tmp_local) {
+                tmp_local = strdup(tmp_local);
+            }
+            setlocale(LC_ALL, "C");
+            LLVMContext* context = new LLVMContext();
+            SMDiagnostic err;
+            // parseIR takes ownership of the given buffer, so don't delete it
+            Module* module = parseIR(buffer, err, *context).release();
+            if (!module) {
+                error_msg = "ERROR : " + string(err.getMessage().data()) + "\n";
+                return nullptr;
+            }
+            if (tmp_local) {
+                setlocale(LC_ALL, tmp_local);
+                free(tmp_local);
+            }
+            llvm_dynamic_dsp_factory_aux* factory_aux
+                = new llvm_dynamic_dsp_factory_aux(sha_key, module, context, target, opt_level);
+            if (factory_aux->initJIT(error_msg)) {
+                llvm_dsp_factory* factory = new llvm_dsp_factory(factory_aux);
+                llvm_dsp_factory_aux::gLLVMFactoryTable.setFactory(factory);
+                factory->setSHAKey(sha_key);
+                return factory;
+            } else {
+                delete factory_aux;
+                return nullptr;
+            }
+        } catch (faustexception& e) {
+            error_msg = e.what();
             return nullptr;
         }
     }
@@ -595,60 +594,113 @@ EXPORT llvm_dsp_factory* createDSPFactoryFromString(const string& name_app, cons
 {
     LOCK_API
     string expanded_dsp_content, sha_key;
-    
-    //if ((expanded_dsp_content = expandDSPFromString(name_app, dsp_content, argc, argv, sha_key, error_msg)) == "") {
+   
     if ((expanded_dsp_content = sha1FromDSP(name_app, dsp_content, argc, argv, sha_key)) == "") {
         return nullptr;
     } else {
         
         dsp_factory_table<SDsp_factory>::factory_iterator it;
-        llvm_dsp_factory* factory = nullptr;
-        
         if (llvm_dsp_factory_aux::gLLVMFactoryTable.getFactory(sha_key, it)) {
             SDsp_factory sfactory = (*it).first;
             sfactory->addReference();
             return sfactory;
         } else {
-            int         argc1 = 0;
-            const char* argv1[64];
-            argv1[argc1++] = "faust";
-            argv1[argc1++] = "-lang";
-            // argv1[argc1++] = "cllvm";
-            argv1[argc1++] = "llvm";
-            argv1[argc1++] = "-o";
-            argv1[argc1++] = "string";
-            // Copy arguments
-            for (int i = 0; i < argc; i++) {
-                argv1[argc1++] = argv[i];
-            }
-            argv1[argc1] = nullptr;  // NULL terminated argv
-            
-            llvm_dynamic_dsp_factory_aux* factory_aux = nullptr;
             try {
-                factory_aux = static_cast<llvm_dynamic_dsp_factory_aux*>(
-                                                                         compileFaustFactory(argc1, argv1, name_app.c_str(), dsp_content.c_str(), error_msg, true));
-                if (factory_aux) {
+                int         argc1 = 0;
+                const char* argv1[64];
+                argv1[argc1++] = "faust";
+                argv1[argc1++] = "-lang";
+                argv1[argc1++] = "llvm";
+                argv1[argc1++] = "-o";
+                argv1[argc1++] = "string";
+                // Copy arguments
+                for (int i = 0; i < argc; i++) {
+                    argv1[argc1++] = argv[i];
+                }
+                argv1[argc1] = nullptr;  // NULL terminated argv
+                
+                llvm_dynamic_dsp_factory_aux* factory_aux
+                    = static_cast<llvm_dynamic_dsp_factory_aux*>(createFactory(name_app.c_str(),
+                                                                               dsp_content.c_str(),
+                                                                               argc1, argv1,
+                                                                               error_msg,
+                                                                               true));
+                if (factory_aux && factory_aux->initJIT(error_msg)) {
                     factory_aux->setTarget(target);
                     factory_aux->setOptlevel(opt_level);
                     factory_aux->setClassName(getParam(argc, argv, "-cn", "mydsp"));
                     factory_aux->setName(name_app);
-                    if (!factory_aux->initJIT(error_msg)) {
-                        goto error;
-                    }
-                    factory = new llvm_dsp_factory(factory_aux);
+                    llvm_dsp_factory* factory = new llvm_dsp_factory(factory_aux);
                     llvm_dsp_factory_aux::gLLVMFactoryTable.setFactory(factory);
                     factory->setSHAKey(sha_key);
                     factory->setDSPCode(expanded_dsp_content);
                     return factory;
+                } else {
+                    delete factory_aux;
+                    return nullptr;
                 }
             } catch (faustexception& e) {
                 error_msg = e.what();
-                goto error;
+                return nullptr;
             }
-        error:
+        }
+    }
+}
+        
+EXPORT llvm_dsp_factory* createDSPFactoryFromSignals(const std::string& name_app, tvec signals,
+                                                    int argc, const char* argv[],
+                                                    const std::string& target,
+                                                    std::string& error_msg,
+                                                    int opt_level)
+{
+    LOCK_API
+    try {
+        int         argc1 = 0;
+        const char* argv1[64];
+        argv1[argc1++] = "faust";
+        argv1[argc1++] = "-lang";
+        argv1[argc1++] = "llvm";
+        argv1[argc1++] = "-o";
+        argv1[argc1++] = "string";
+            // Copy arguments
+        for (int i = 0; i < argc; i++) {
+            argv1[argc1++] = argv[i];
+        }
+        argv1[argc1] = nullptr;  // NULL terminated argv
+        
+        llvm_dynamic_dsp_factory_aux* factory_aux
+            = static_cast<llvm_dynamic_dsp_factory_aux*>(createFactory(name_app.c_str(), signals, argc1, argv1, error_msg));
+        if (factory_aux && factory_aux->initJIT(error_msg)) {
+            factory_aux->setTarget(target);
+            factory_aux->setOptlevel(opt_level);
+            factory_aux->setClassName(getParam(argc, argv, "-cn", "mydsp"));
+            factory_aux->setName(name_app);
+            llvm_dsp_factory* factory = new llvm_dsp_factory(factory_aux);
+            llvm_dsp_factory_aux::gLLVMFactoryTable.setFactory(factory);
+            return factory;
+        } else {
             delete factory_aux;
             return nullptr;
         }
+    } catch (faustexception& e) {
+        error_msg = e.what();
+        return nullptr;
+    }
+}
+        
+EXPORT llvm_dsp_factory* createDSPFactoryFromBoxes(const std::string& name_app, Tree box,
+                                                   int argc, const char* argv[],
+                                                   const std::string& target,
+                                                   std::string& error_msg,
+                                                   int opt_level)
+{
+    LOCK_API
+    try {
+        tvec signals = boxesToSignalsAux(box);
+        return createDSPFactoryFromSignals(name_app, signals, argc, argv, target, error_msg, opt_level);
+    } catch (faustexception& e) {
+        error_msg = e.Message();
+        return nullptr;
     }
 }
 
@@ -705,7 +757,6 @@ EXPORT llvm_dsp_factory* readDSPFactoryFromIRFile(const string& ir_code_path, co
 {
     LOCK_API
     ErrorOr<OwningPtr<MemoryBuffer>> buffer = MemoryBuffer::getFileOrSTDIN(ir_code_path);
-
     if (error_code ec = buffer.getError()) {
         error_msg = "ERROR : " + ec.message() + "\n";
         return nullptr;
@@ -730,7 +781,6 @@ EXPORT llvm_dsp_factory* createCDSPFactoryFromFile(const char* filename, int arg
                                                    const char* target, char* error_msg, int opt_level)
 {
     string error_msg_aux;
-    
     llvm_dsp_factory* factory = createDSPFactoryFromFile(filename, argc, argv, target, error_msg_aux, opt_level);
     strncpy(error_msg, error_msg_aux.c_str(), 4096);
     return factory;
@@ -741,18 +791,46 @@ EXPORT llvm_dsp_factory* createCDSPFactoryFromString(const char* name_app, const
                                                      int opt_level)
 {
     string error_msg_aux;
-    
     llvm_dsp_factory* factory =
         createDSPFactoryFromString(name_app, dsp_content, argc, argv, target, error_msg_aux, opt_level);
     strncpy(error_msg, error_msg_aux.c_str(), 4096);
     return factory;
 }
+    
+EXPORT llvm_dsp_factory* createCDSPFactoryFromSignals(const char* name_app, Signal* signals_aux,
+                                                      int argc, const char* argv[],
+                                                      const char* target,
+                                                      char* error_msg,
+                                                      int opt_level)
+{
+    string error_msg_aux;
+    tvec signals;
+    int i = 0;
+    while (signals_aux[i]) { signals.push_back(signals_aux[i]); i++; }
+    llvm_dsp_factory* factory =
+        createDSPFactoryFromSignals(name_app, signals, argc, argv, target, error_msg_aux, opt_level);
+    strncpy(error_msg, error_msg_aux.c_str(), 4096);
+    return factory;
+}
+
+EXPORT llvm_dsp_factory* createCDSPFactoryFromBoxes(const char* name_app,
+                                                    Tree box,
+                                                    int argc, const char* argv[],
+                                                    const char* target,
+                                                    char* error_msg,
+                                                    int opt_level)
+{
+    string error_msg_aux;
+    llvm_dsp_factory* factory =
+        createDSPFactoryFromBoxes(name_app, box, argc, argv, target, error_msg_aux, opt_level);
+    strncpy(error_msg, error_msg_aux.c_str(), 4096);
+    return factory;
+}
 
 EXPORT llvm_dsp_factory* readCDSPFactoryFromBitcode(const char* bit_code, const char* target, char* error_msg,
                                                     int opt_level)
 {
     string error_msg_aux;
-    
     llvm_dsp_factory* factory = readDSPFactoryFromBitcode(bit_code, target, error_msg_aux, opt_level);
     strncpy(error_msg, error_msg_aux.c_str(), 4096);
     return factory;
@@ -767,7 +845,6 @@ EXPORT llvm_dsp_factory* readCDSPFactoryFromBitcodeFile(const char* bit_code_pat
                                                         int opt_level)
 {
     string error_msg_aux;
-    
     llvm_dsp_factory* factory = readDSPFactoryFromBitcodeFile(bit_code_path, target, error_msg_aux, opt_level);
     strncpy(error_msg, error_msg_aux.c_str(), 4096);
     return factory;
@@ -781,7 +858,6 @@ EXPORT bool writeCDSPFactoryToBitcodeFile(llvm_dsp_factory* factory, const char*
 EXPORT llvm_dsp_factory* readCDSPFactoryFromIR(const char* ir_code, const char* target, char* error_msg, int opt_level)
 {
     string error_msg_aux;
-    
     llvm_dsp_factory* factory = readDSPFactoryFromIR(ir_code, target, error_msg_aux, opt_level);
     strncpy(error_msg, error_msg_aux.c_str(), 4096);
     return factory;
