@@ -46,6 +46,11 @@ static int xVariability(int v, int r)
 //	Occurences methods
 //-------------------------------------------------
 
+int old_Occurences::getOccurence(int variability) const
+{
+    return fOccurences[variability];
+}
+
 old_Occurences::old_Occurences(int v, int r, Tree xc) : fXVariability(xVariability(v, r))
 {
     for (int i = 0; i < 4; i++) fOccurences[i] = 0;
@@ -122,12 +127,7 @@ void old_OccMarkup::mark(Tree root)
 
 old_Occurences* old_OccMarkup::retrieve(Tree t)
 {
-    old_Occurences* p = getOcc(t);
-    if (p == 0) {
-        // cerr << "No Occurences info attached to : " << *t << endl;
-        // exit(1);
-    }
-    return p;
+    return getOcc(t);
 }
 
 //------------------------------------------------------------------------------
@@ -151,8 +151,8 @@ void old_OccMarkup::incOcc(Tree env, int v, int r, int d, Tree xc, Tree t)
         setOcc(t, occ);
 
         // We mark the subtrees of t
-        Tree c, x, y, z;
-        if (isSigFixDelay(t, x, y)) {
+        Tree x, y;
+        if (isSigDelay(t, x, y)) {
             Type g2 = getCertifiedSigType(y);
             int  d2 = checkDelayInterval(g2);
             faustassert(d2 >= 0);
@@ -161,15 +161,6 @@ void old_OccMarkup::incOcc(Tree env, int v, int r, int d, Tree xc, Tree t)
         } else if (isSigPrefix(t, y, x)) {
             incOcc(env, v0, r0, 1, c0, x);
             incOcc(env, v0, r0, 0, c0, y);
-        } else if (isSigSelect3(t, c, y, x, z)) {
-            // make a special case for select3 implemented with real if
-            // because the c expression will be used twice in the C++
-            // translation
-            incOcc(env, v0, r0, 0, c0, c);
-            incOcc(env, v0, r0, 0, c0, c);
-            incOcc(env, v0, r0, 0, c0, x);
-            incOcc(env, v0, r0, 0, c0, y);
-            incOcc(env, v0, r0, 0, c0, z);
         } else {
             vector<Tree> br;
             int          n = getSubSignals(t, br);
