@@ -60,9 +60,6 @@ CodeContainer* RustCodeContainer::createContainer(const string& name, int numInp
     gGlobal->gDSPStruct = true;
     CodeContainer* container;
 
-    if (gGlobal->gMemoryManager) {
-        throw faustexception("ERROR : -mem not supported for Rust\n");
-    }
     if (gGlobal->gFloatSize == 3) {
         throw faustexception("ERROR : quad format not supported for Rust\n");
     }
@@ -74,10 +71,8 @@ CodeContainer* RustCodeContainer::createContainer(const string& name, int numInp
     }
 
     if (gGlobal->gOpenMPSwitch) {
-        // container = new RustOpenMPCodeContainer(name, numInputs, numOutputs, dst);
         throw faustexception("ERROR : OpenMP not supported for Rust\n");
     } else if (gGlobal->gSchedulerSwitch) {
-        // container = new RustWorkStealingCodeContainer(name, numInputs, numOutputs, dst);
         throw faustexception("ERROR : Scheduler not supported for Rust\n");
     } else if (gGlobal->gVectorSwitch) {
         // container = new RustVectorCodeContainer(name, numInputs, numOutputs, dst);
@@ -100,7 +95,6 @@ void RustCodeContainer::produceInternal()
 
     tab(n, *fOut);
     *fOut << "pub struct " << fKlassName << " {";
-
     tab(n + 1, *fOut);
 
     // Fields
@@ -171,7 +165,7 @@ void RustCodeContainer::produceClass()
 {
     int n = 0;
 
-    // Sub containers
+    // Generate gub containers
     generateSubContainers();
 
     // Functions
@@ -355,6 +349,9 @@ void RustCodeContainer::produceClass()
     tab(n, *fOut);
     *fOut << "}" << endl;
     tab(n, *fOut);
+    
+    // Generate user interface macros if needed
+    printMacros(*fOut, n);
 }
 
 void RustCodeContainer::produceMetadata(int n)
@@ -363,7 +360,7 @@ void RustCodeContainer::produceMetadata(int n)
     *fOut << "fn metadata(&self, m: &mut dyn Meta) { ";
 
     // We do not want to accumulate metadata from all hierachical levels, so the upper level only is kept
-    for (auto& i : gGlobal->gMetaDataSet) {
+    for (const auto& i : gGlobal->gMetaDataSet) {
         if (i.first != tree("author")) {
             tab(n + 1, *fOut);
             *fOut << "m.declare(\"" << *(i.first) << "\", " << **(i.second.begin()) << ");";
@@ -394,10 +391,6 @@ void RustCodeContainer::produceInfoFunctions(int tabs, const string& classname,
     producer->Tab(tabs);
     generateGetInputs(subst("get_num_inputs$0", classname), obj, false, false)->accept(&fCodeProducer);
     generateGetOutputs(subst("get_num_outputs$0", classname), obj, false, false)->accept(&fCodeProducer);
-    producer->Tab(tabs);
-    generateGetInputRate(subst("get_input_rate$0", classname), obj, false, false)->accept(&fCodeProducer);
-    producer->Tab(tabs);
-    generateGetOutputRate(subst("get_output_rate$0", classname), obj, false, false)->accept(&fCodeProducer);
 }
 
 void RustCodeContainer::produceParameterGetterSetter(int tabs, map<string, int> parameterLookup)
@@ -468,10 +461,13 @@ void RustScalarCodeContainer::generateCompute(int n)
         iterators.push_back("inputs" + std::to_string(i));
     }
     for (int i = 0; i < fNumOutputs; ++i) {
-        iterators.push_back("outputs"+ std::to_string(i));
+        iterators.push_back("outputs" + std::to_string(i));
     }
     IteratorForLoopInst* loop = fCurLoop->generateSimpleScalarLoop(iterators);
     loop->accept(&fCodeProducer);
+    
+    // Currently for soundfile management
+    generatePostComputeBlock(&fCodeProducer);
 
     back(1, *fOut);
     *fOut << "}" << endl;
