@@ -36,6 +36,7 @@ using namespace std;
     - see the SOULInstVisitor fIntAsBool variable.
  - the 'fillXXX' function needs to generate the actual size of the table argument type. This is done using the
  TableSizeVisitor class.
+ - bargraphs use 'output event' type and are outputting values at 50 Hz. The code is conditionally generated.
 */
 
 dsp_factory_base* SOULCodeContainer::produceFactory()
@@ -107,7 +108,7 @@ void SOULCodeContainer::produceInternal()
     // Retrieves the table size kept in gTableSizeVisitor to generate the correct table type
     string fun_name = "fill" + fKlassName;
     // We possibly have to generate several versions of the function with a different table size
-    for (auto& it : gGlobal->gTableSizeVisitor->fSizeTable) {
+    for (const auto& it : gGlobal->gTableSizeVisitor->fSizeTable) {
         string fun_name_aux = it.first;
         int    table_size   = it.second;
         if (startWith(fun_name_aux, fun_name)) {
@@ -152,6 +153,10 @@ void SOULCodeContainer::produceInit(int tabs)
     tab(tabs + 1, *fOut);
     *fOut << "let sample_rate = int(processor.frequency);";
     tab(tabs + 1, *fOut);
+    if (fUIVisitor.fHasBargraph) {
+        *fOut << "fControlSlice = int (processor.frequency) / 50;";
+        tab(tabs + 1, *fOut);
+    }
     *fOut << "// classInit is not called here since the tables are actually not shared between instances";
     tab(tabs + 1, *fOut);
     *fOut << "instanceInit (sample_rate);";
@@ -181,11 +186,11 @@ void SOULCodeContainer::produceInit(int tabs)
 void SOULCodeContainer::produceClass()
 {
     int n = 0;
-
+   
     // Look for the "fillXXX" function
     generateStaticInit(gGlobal->gTableSizeVisitor);
     generateInit(gGlobal->gTableSizeVisitor);
-
+ 
     // Processor generation
     tab(n, *fOut);
     *fOut << "processor " << fKlassName;
@@ -218,14 +223,17 @@ void SOULCodeContainer::produceClass()
         tab(n + 1, *fOut);
     }
     
-    SOULInstUIVisitor ui_visitor(n + 1);
-    generateUserInterface(&ui_visitor);
-    *fOut << ui_visitor.fOut.str();
+    generateUserInterface(&fUIVisitor);
+    *fOut << fUIVisitor.fOut.str();
     generateDeclarations(&fCodeProducer);
   
     // Control
     *fOut << "bool fUpdated;";
     tab(n + 1, *fOut);
+    if (fUIVisitor.fHasBargraph) {
+        *fOut << "int fControlSlice;";
+        tab(n + 1, *fOut);
+    }
  
     // For control computation
     if (fInt32ControlNum > 0) {
@@ -285,10 +293,20 @@ void SOULCodeContainer::produceClass()
         tab(n + 1, *fOut);
     }
   
-    // Sub containers
+    // Generate gub containers
     generateSubContainers();
+    
+    // Missing math functions
+    tab(n + 1, *fOut);
+    if (gGlobal->gFloatSize == 1) {
+        *fOut << "float32 copysign(float32 x, float32 y) { return abs(x) * ((y < 0.0f) ? -1.0f : 1.0f); }";
+    } else if (gGlobal->gFloatSize == 2) {
+        *fOut << "float64 copysign(float64 x, float64 y) { return abs(x) * ((y < 0.0) ? -1.0 : 1.0); }";
+    }
+    tab(n + 1, *fOut);
 
     // inputs/outputs
+    tab(n + 1, *fOut);
     *fOut << "int getNumInputs() { return " << fNumInputs << "; }";
     tab(n + 1, *fOut);
     tab(n + 1, *fOut);
@@ -392,8 +410,6 @@ void SOULScalarCodeContainer::generateCompute(int n)
     *fOut << "loop";
     tab(n + 1, *fOut);
     *fOut << "{";
-    tab(n + 2, *fOut);
-
     tab(n + 2, *fOut);
     *fOut << "if (fUpdated) { fUpdated = false; control(); }";
     tab(n + 2, *fOut);
@@ -410,6 +426,12 @@ void SOULScalarCodeContainer::generateCompute(int n)
     generatePostComputeBlock(&fCodeProducer);
 
     tab(n + 2, *fOut);
+    if (fUIVisitor.fHasBargraph) {
+        *fOut << "// Updates fControlSlice once per sample";
+        tab(n + 2, *fOut);
+        *fOut << "if (fControlSlice-- == 0) { fControlSlice = int (processor.frequency) / 50; }";
+        tab(n + 2, *fOut);
+    }
     *fOut << "// Moves all streams forward by one 'tick'";
     tab(n + 2, *fOut);
     *fOut << "advance();";
