@@ -101,14 +101,13 @@ EXPORT wasm_dsp_factory* createWasmDSPFactoryFromString(const string& name_app,
         argv1[argc1++] = (internal_memory) ? "wasm-ib" : "wasm-eb";
         argv1[argc1++] = "-o";
         argv1[argc1++] = "binary";
+        // Copy argument
         for (int i = 0; i < argc; i++) {
             argv1[argc1++] = argv[i];
         }
         argv1[argc1] = nullptr;  // NULL terminated argv
 
-        dsp_factory_base* dsp_factory_aux =
-            compileFaustFactory(argc1, argv1, name_app.c_str(), dsp_content.c_str(), error_msg, true);
-
+        dsp_factory_base* dsp_factory_aux = createFactory(name_app.c_str(), dsp_content.c_str(), argc1, argv1, error_msg, true);
         if (dsp_factory_aux) {
             dsp_factory_aux->setName(name_app);
             wasm_dsp_factory* factory = new wasm_dsp_factory(dsp_factory_aux);
@@ -122,6 +121,35 @@ EXPORT wasm_dsp_factory* createWasmDSPFactoryFromString(const string& name_app,
     }
 }
 
+EXPORT wasm_dsp_factory* createWasmDSPFactoryFromSignals(const std::string& name_app, tvec signals,
+                                                         int argc, const char* argv[], std::string& error_msg,
+                                                         bool internal_memory)
+{
+    int         argc1 = 0;
+    const char* argv1[64];
+    argv1[argc1++] = "faust";
+    argv1[argc1++] = "-lang";
+    // argv1[argc1++] = (internal_memory) ? "wasm-i" : "wasm-e";
+    argv1[argc1++] = (internal_memory) ? "wasm-ib" : "wasm-eb";
+    argv1[argc1++] = "-o";
+    argv1[argc1++] = "binary";
+    // Copy argument
+    for (int i = 0; i < argc; i++) {
+        argv1[argc1++] = argv[i];
+    }
+    argv1[argc1] = nullptr;  // NULL terminated argv
+    
+    dsp_factory_base* dsp_factory_aux = createFactory(name_app.c_str(), signals, argc1, argv1, error_msg);
+    if (dsp_factory_aux) {
+        dsp_factory_aux->setName(name_app);
+        wasm_dsp_factory* factory = new wasm_dsp_factory(dsp_factory_aux);
+        wasm_dsp_factory::gWasmFactoryTable.setFactory(factory);
+        return factory;
+    } else {
+        return nullptr;
+    }
+}
+
 EXPORT std::string generateWasmFromString(const string& name_app, const string& dsp_content, int argc,
                                          const char* argv[], string& error_msg, bool internal_memory)
 {
@@ -138,7 +166,7 @@ EXPORT std::string generateWasmFromString(const string& name_app, const string&
     }
     argv1[argc1] = nullptr;  // NULL terminated argv
     
-    dsp_factory_base* dsp_factory_aux = compileFaustFactory(argc1, argv1, name_app.c_str(), dsp_content.c_str(), error_msg, true);
+    dsp_factory_base* dsp_factory_aux = createFactory(name_app.c_str(), dsp_content.c_str(), argc1, argv1, error_msg, true);
     return (dsp_factory_aux) ? dsp_factory_aux->getBinaryCode() : "";
 }
 
@@ -154,7 +182,7 @@ EXPORT void deleteAllWasmCDSPFactories()
 EXPORT wasm_dsp_factory* createWasmCDSPFactoryFromFile2(const char* filename, int argc, const char* argv[],
                                                         char* error_msg, bool internal_memory)
 {
-    string            error_msg_aux;
+    string error_msg_aux;
     wasm_dsp_factory* factory = createWasmDSPFactoryFromFile(filename, argc, argv, error_msg_aux, internal_memory);
     strncpy(error_msg, error_msg_aux.c_str(), 4096);
     return factory;
@@ -163,12 +191,23 @@ EXPORT wasm_dsp_factory* createWasmCDSPFactoryFromFile2(const char* filename, in
 EXPORT wasm_dsp_factory* createWasmCDSPFactoryFromString2(const char* name_app, const char* dsp_content, int argc,
                                                           const char* argv[], char* error_msg, bool internal_memory)
 {
-    string            error_msg_aux;
+    string error_msg_aux;
     wasm_dsp_factory* factory =
         createWasmDSPFactoryFromString(name_app, dsp_content, argc, argv, error_msg_aux, internal_memory);
     strncpy(error_msg, error_msg_aux.c_str(), 4096);
     return factory;
 }
+    
+EXPORT wasm_dsp_factory* createWasmCDSPFactoryFromSignals2(const char* name_app, tvec signals,
+                                                           int argc, const char* argv[], char* error_msg,
+                                                           bool internal_memory)
+{
+    string error_msg_aux;
+    wasm_dsp_factory* factory =
+        createWasmDSPFactoryFromSignals(name_app, signals, argc, argv, error_msg_aux, internal_memory);
+    strncpy(error_msg, error_msg_aux.c_str(), 4096);
+    return factory;
+}
 
 static WasmModule* createWasmCDSPFactoryAux(wasm_dsp_factory* factory, const string& error_msg_aux, char* error_msg)
 {
