@@ -32,6 +32,7 @@
 #include "cosprim.hh"
 #include "exp10prim.hh"
 #include "expprim.hh"
+#include "floats.hh"
 #include "floorprim.hh"
 #include "fmodprim.hh"
 #include "ftzprim.hh"
@@ -74,6 +75,10 @@
 #include "java_code_container.hh"
 #endif
 
+#ifdef CSHARP_BUILD
+#include "csharp_code_container.hh"
+#endif
+
 #ifdef RUST_BUILD
 #include "rust_code_container.hh"
 #endif
@@ -103,9 +108,9 @@ global::global() : TABBER(1), gLoopDetector(1024, 400), gStackOverflowDetector(M
     EVALPROPERTY   = symbol("EvalProperty");
     PMPROPERTYNODE = symbol("PMPROPERTY");
 
-    gResult          = 0;
-    gResult2         = 0;
-    gExpandedDefList = 0;
+    gResult          = nullptr;
+    gResult2         = nullptr;
+    gExpandedDefList = nullptr;
 
     gDetailsSwitch    = false;
     gDrawSignals      = false;
@@ -147,14 +152,16 @@ global::global() : TABBER(1), gLoopDetector(1024, 400), gStackOverflowDetector(M
     gLightMode  = false;
     gClang      = false;
     gCheckTable = "";
+    
+    gMathExceptions = false;
 
     gClassName      = "mydsp";
     gSuperClassName = "dsp";
     gProcessName    = "process";
 
-    gDSPFactory = 0;
+    gDSPFactory = nullptr;
 
-    gInputString = 0;
+    gInputString = nullptr;
 
     // Backend configuration : default values
     gAllowForeignFunction = true;
@@ -172,12 +179,15 @@ global::global() : TABBER(1), gLoopDetector(1024, 400), gStackOverflowDetector(M
     gMathApprox           = false;
     gNeedManualPow        = true;
     gRemoveVarAddress     = false;
-    gOneSample            = false;
+    gOneSample            = -1;
     gOneSampleControl     = false;
     gComputeMix           = false;
     gFastMathLib          = "default";
     gNameSpace            = "";
 
+    gNarrowingLimit = 0;
+    gWideningLimit = 0;
+
     // Fastmath mapping float version
     gFastMathLibTable["fabsf"]      = "fast_fabsf";
     gFastMathLibTable["acosf"]      = "fast_acosf";
@@ -247,9 +257,9 @@ global::global() : TABBER(1), gLoopDetector(1024, 400), gStackOverflowDetector(M
     gLocalCausalityCheck = false;
     gCausality           = false;
 
-    gOccurrences = 0;
+    gOccurrences = nullptr;
     gFoldingFlag = false;
-    gDevSuffix   = 0;
+    gDevSuffix   = nullptr;
 
     gAbsPrim       = new AbsPrim();
     gAcosPrim      = new AcosPrim();
@@ -343,9 +353,10 @@ global::global() : TABBER(1), gLoopDetector(1024, 400), gStackOverflowDetector(M
     FFUN        = symbol("ForeignFunction");
 
     SIGINPUT           = symbol("SigInput");
+    gMaxInputs         = 0;
     SIGOUTPUT          = symbol("SigOutput");
     SIGDELAY1          = symbol("SigDelay1");
-    SIGFIXDELAY        = symbol("SigFixDelay");
+    SIGDELAY           = symbol("SigDelay");
     SIGPREFIX          = symbol("SigPrefix");
     SIGIOTA            = symbol("SigIota");
     SIGRDTBL           = symbol("SigRDTbl");
@@ -356,7 +367,9 @@ global::global() : TABBER(1), gLoopDetector(1024, 400), gStackOverflowDetector(M
     SIGDOCWRITETBL     = symbol("SigDocWriteTbl");
     SIGDOCACCESSTBL    = symbol("SigDocAccessTbl");
     SIGSELECT2         = symbol("SigSelect2");
-    SIGSELECT3         = symbol("SigSelect3");
+    SIGASSERTBOUNDS    = symbol("sigAssertBounds");
+    SIGHIGHEST         = symbol("sigHighest");
+    SIGLOWEST          = symbol("sigLowest");
     SIGBINOP           = symbol("SigBinOp");
     SIGFFUN            = symbol("SigFFun");
     SIGFCONST          = symbol("SigFConst");
@@ -411,16 +424,20 @@ global::global() : TABBER(1), gLoopDetector(1024, 400), gStackOverflowDetector(M
     gOutputLang          = "";
 
 #ifdef WASM_BUILD
-    gWASMVisitor = 0;  // Will be (possibly) allocated in WebAssembly backend
-    gWASTVisitor = 0;  // Will be (possibly) allocated in WebAssembly backend
+    gWASMVisitor = nullptr;  // Will be (possibly) allocated in WebAssembly backend
+    gWASTVisitor = nullptr;  // Will be (possibly) allocated in WebAssembly backend
 #endif
 
 #ifdef INTERP_BUILD
-    gInterpreterVisitor = 0;  // Will be (possibly) allocated in Interp backend
+    gInterpreterVisitor = nullptr;  // Will be (possibly) allocated in Interp backend
+#endif
+
+#ifdef JULIA_BUILD
+    gJuliaVisitor = nullptr;  // Will be (possibly) allocated in Julia backend
 #endif
 
 #ifdef SOUL_BUILD
-    gTableSizeVisitor = 0;  // Will be (possibly) allocated in SOUL backend
+    gTableSizeVisitor = nullptr;  // Will be (possibly) allocated in SOUL backend
 #endif
 
     gHelpSwitch       = false;
@@ -433,6 +450,9 @@ global::global() : TABBER(1), gLoopDetector(1024, 400), gStackOverflowDetector(M
     gGraphSwitch      = false;
     gDrawPSSwitch     = false;
     gDrawSVGSwitch    = false;
+    gVHDLSwitch       = false;
+    gVHDLTrace        = false;
+    gElementarySwitch = false;
     gPrintXMLSwitch   = false;
     gPrintJSONSwitch  = false;
     gPrintDocSwitch   = false;
@@ -442,8 +462,8 @@ global::global() : TABBER(1), gLoopDetector(1024, 400), gStackOverflowDetector(M
     gTimeout = 120;  // Time out to abort compiler (in seconds)
 
     // Globals to transfer results in thread based evaluation
-    gProcessTree  = 0;
-    gLsignalsTree = 0;
+    gProcessTree  = nullptr;
+    gLsignalsTree = nullptr;
     gNumInputs    = 0;
     gNumOutputs   = 0;
     gErrorMessage = "";
@@ -462,9 +482,9 @@ void global::init()
     gSymListProp           = new property<Tree>();
 
     // Essential predefined types
-    gMemoizedTypes   = new property<AudioType*>();
-    gAllocationCount = 0;
-    gMaskDelayLineThreshold  = INT_MAX;
+    gMemoizedTypes          = new property<AudioType*>();
+    gAllocationCount        = 0;
+    gMaskDelayLineThreshold = INT_MAX;
 
     // True by default but only usable with -lang ocpp backend
     gEnableFlag = true;
@@ -481,12 +501,17 @@ void global::init()
     TEXEC = makeSimpleType(kInt, kKonst, kExec, kVect, kNum, interval());
 
     // More predefined types
-    TINPUT   = makeSimpleType(kReal, kSamp, kExec, kVect, kNum, interval());
+    TINPUT   = makeSimpleType(kReal, kSamp, kExec, kVect, kNum, interval(-1, 1));
     TGUI     = makeSimpleType(kReal, kBlock, kExec, kVect, kNum, interval());
     TGUI01   = makeSimpleType(kReal, kBlock, kExec, kVect, kNum, interval(0, 1));
     INT_TGUI = makeSimpleType(kInt, kBlock, kExec, kVect, kNum, interval());
 
-    TREC = makeSimpleType(kInt, kSamp, kInit, kScal, kNum, interval());
+    TREC = makeSimpleType(kInt, kSamp, kInit, kScal, kNum, interval(0, 0));
+    // !!! TRECMAX Maximal only in the last component of the type lattice
+    TRECMAX = makeSimpleType(kInt, kSamp, kInit, kScal, kNum, interval(-HUGE_VAL,HUGE_VAL));
+
+    // empty Predefined bit depth
+    RES = res();
 
     // Predefined symbols CONS and NIL
     CONS = symbol("cons");
@@ -519,7 +544,7 @@ void global::init()
 
     // yyfilename is defined in errormsg.cpp but must be redefined at each compilation.
     yyfilename = "";
-    yyin       = 0;
+    yyin       = nullptr;
 
     gLatexheaderfilename = "latexheader.tex";
     gDocTextsDefaultFile = "mathdoctexts-default.txt";
@@ -573,18 +598,40 @@ void global::init()
     gMathForeignFunctions["tanhf"] = true;
     gMathForeignFunctions["tanh"]  = true;
     gMathForeignFunctions["tanhl"] = true;
-    
+
     gMathForeignFunctions["isnanf"] = true;
-    gMathForeignFunctions["isnan"] = true;
+    gMathForeignFunctions["isnan"]  = true;
     gMathForeignFunctions["isnanl"] = true;
-    
+
     gMathForeignFunctions["isinff"] = true;
-    gMathForeignFunctions["isinf"] = true;
+    gMathForeignFunctions["isinf"]  = true;
     gMathForeignFunctions["isinfl"] = true;
+    
+    gMathForeignFunctions["copysignf"] = true;
+    gMathForeignFunctions["copysign"]  = true;
+    gMathForeignFunctions["copysignl"] = true;
+}
+
+static string printFloat()
+{
+    switch (gGlobal->gFloatSize) {
+        case 1:
+            return "-single ";
+        case 2:
+            return "-double ";
+        case 3:
+            return "-quad ";
+        case 4:
+            return "-fp ";
+        default:
+            faustassert(false);
+            return "";
+    }
 }
 
 void global::printCompilationOptions(stringstream& dst, bool backend)
 {
+    if (gArchFile != "") dst << "-a " << gArchFile << " ";
     if (backend) {
 #ifdef LLVM_BUILD
         if (gOutputLang == "llvm") {
@@ -597,7 +644,7 @@ void global::printCompilationOptions(stringstream& dst, bool backend)
 #endif
     }
     if (gInPlace) dst << "-inpl ";
-    if (gOneSample) dst << "-os ";
+    if (gOneSample >= 0) dst << "-os" << gOneSample << " ";
     if (gLightMode) dst << "-light ";
     if (gMemoryManager) dst << "-mem ";
     if (gComputeMix) dst << "-cm ";
@@ -606,20 +653,18 @@ void global::printCompilationOptions(stringstream& dst, bool backend)
     if (gMaskDelayLineThreshold != INT_MAX) dst << "-dtl " << gMaskDelayLineThreshold << " ";
     dst << "-es " << gEnableFlag << " ";
     if (gHasExp10) dst << "-exp10 ";
-    if (gOneSample) dst << "-os ";
     if (gSchedulerSwitch) dst << "-sch ";
     if (gOpenMPSwitch) dst << "-omp " << ((gOpenMPLoop) ? "-pl " : "");
     if (gVectorSwitch) {
-        dst << "-vec"
-            << " -lv " << gVectorLoopVariant << " -vs " << gVecSize << ((gFunTaskSwitch) ? " -fun" : "")
-            << ((gGroupTaskSwitch) ? " -g" : "") << ((gDeepFirstSwitch) ? " -dfs" : "")
-            << ((gFloatSize == 2) ? " -double" : (gFloatSize == 3) ? " -quad" : "") << " -ftz " << gFTZMode << " -mcd "
-            << gGlobal->gMaxCopyDelay;
+        dst << "-vec "
+            << "-lv " << gVectorLoopVariant << " "
+            << "-vs " << gVecSize << " " << ((gFunTaskSwitch) ? "-fun " : "") << ((gGroupTaskSwitch) ? "-g " : "")
+            << ((gDeepFirstSwitch) ? "-dfs " : "") << printFloat() << "-ftz " << gFTZMode << " "
+            << "-mcd " << gGlobal->gMaxCopyDelay;
     } else {
-        dst << ((gFloatSize == 1) ? "-scal" : ((gFloatSize == 2) ? "-double" : (gFloatSize == 3) ? "-quad" : ""))
-            << " -ftz " << gFTZMode;
+        dst << printFloat() << "-ftz " << gFTZMode;
     }
-    
+
     // Add 'compile_options' metadata
     gGlobal->gMetaDataSet[tree("compile_options")].insert(tree("\"" + dst.str() + "\""));
 }
@@ -672,6 +717,28 @@ int global::audioSampleSize()
     return int(pow(2.f, float(gFloatSize + 1)));
 }
 
+BasicTyped* global::genBasicTyped(Typed::VarType type)
+{
+    // Possibly force FAUSTFLOAT type (= kFloatMacro) to internal real
+    Typed::VarType new_type = ((type == Typed::kFloatMacro) && gFAUSTFLOAT2Internal) ? itfloat() : type;
+
+    // If not defined, add the type in the table
+    if (gTypeTable.find(new_type) == gTypeTable.end()) {
+        gTypeTable[new_type] = new BasicTyped(new_type);
+    }
+    return gTypeTable[new_type];
+}
+
+void global::setVarType(const string& name, Typed::VarType type)
+{
+    gVarTypeTable[name] = genBasicTyped(type);
+}
+
+Typed::VarType global::getVarType(const string& name)
+{
+    return gVarTypeTable[name]->getType();
+}
+
 global::~global()
 {
     Garbageable::cleanup();
@@ -687,18 +754,24 @@ global::~global()
 #ifdef CPP_BUILD
     CPPInstVisitor::cleanup();
 #endif
+#ifdef CSHARP_BUILD
+    CSharpInstVisitor::cleanup();
+#endif
+#ifdef DLANG_BUILD
+    DLangInstVisitor::cleanup();
+#endif
 #ifdef FIR_BUILD
     FIRInstVisitor::cleanup();
 #endif
 #ifdef JAVA_BUILD
     JAVAInstVisitor::cleanup();
 #endif
+#ifdef JULIS_BUILD
+    JuliaInstVisitor::cleanup();
+#endif
 #ifdef RUST_BUILD
     RustInstVisitor::cleanup();
 #endif
-#ifdef DLANG_BUILD
-    DLangInstVisitor::cleanup();
-#endif
 }
 
 void global::allocate()
@@ -716,7 +789,7 @@ void global::destroy()
     }
 #endif
     delete gGlobal;
-    gGlobal = NULL;
+    gGlobal = nullptr;
 }
 
 string global::makeDrawPath()
