@@ -32,12 +32,12 @@
 #include "signals.hh"
 #include "xtended.hh"
 
-Tree sigWriteReadTable(Tree n, Tree init, Tree widx, Tree wsig, Tree ridx)
+EXPORT Tree sigWriteReadTable(Tree n, Tree init, Tree widx, Tree wsig, Tree ridx)
 {
     return sigRDTbl(sigWRTbl(gGlobal->nil, sigTable(gGlobal->nil, n, sigGen(init)), widx, wsig), ridx);
 }
 
-Tree sigReadOnlyTable(Tree n, Tree init, Tree ridx)
+EXPORT Tree sigReadOnlyTable(Tree n, Tree init, Tree ridx)
 {
     return sigRDTbl(sigTable(gGlobal->nil, n, sigGen(init)), ridx);
 }
@@ -45,7 +45,6 @@ Tree sigReadOnlyTable(Tree n, Tree init, Tree ridx)
 ////////////////////////////////////////////////////////////////////////
 /**
  * Signals
- *
  */
 ///////////////////////////////////////////////////////////////////////
 
@@ -59,7 +58,7 @@ Tree sigRem(Tree x, Tree y)
     return sigBinOp(kRem, x, y);
 }
 
-Tree sigInt(int i)
+EXPORT Tree sigInt(int i)
 {
     return tree(i);
 }
@@ -68,7 +67,7 @@ bool isSigInt(Tree t, int* i)
     return isInt(t->node(), i);
 }
 
-Tree sigReal(double r)
+EXPORT Tree sigReal(double r)
 {
     return tree(r);
 }
@@ -77,8 +76,10 @@ bool isSigReal(Tree t, double* r)
     return isDouble(t->node(), r);
 }
 
-Tree sigInput(int i)
+EXPORT Tree sigInput(int i)
 {
+    // Keep the max input number (used with the signal API)
+    gGlobal->gMaxInputs = std::max(gGlobal->gMaxInputs, i+1);
     return tree(gGlobal->SIGINPUT, tree(i));
 }
 bool isSigInput(Tree t, int* i)
@@ -99,7 +100,7 @@ bool isSigOutput(Tree t, int* i, Tree& t0)
 
 Tree sigDelay0(Tree t0)
 {
-    return sigFixDelay(t0, sigInt(0));
+    return sigDelay(t0, sigInt(0));
 }
 
 Tree sigDelay1(Tree t0)
@@ -111,13 +112,13 @@ bool isSigDelay1(Tree t, Tree& t0)
     return isTree(t, gGlobal->SIGDELAY1, t0);
 }
 
-Tree sigFixDelay(Tree t0, Tree t1)
+EXPORT Tree sigDelay(Tree t0, Tree t1)
 {
-    return tree(gGlobal->SIGFIXDELAY, t0, sigIntCast(t1));
+    return tree(gGlobal->SIGDELAY, t0, sigIntCast(t1));
 }
-bool isSigFixDelay(Tree t, Tree& t0, Tree& t1)
+bool isSigDelay(Tree t, Tree& t0, Tree& t1)
 {
-    return isTree(t, gGlobal->SIGFIXDELAY, t0, t1);
+    return isTree(t, gGlobal->SIGDELAY, t0, t1);
 }
 
 Tree sigPrefix(Tree t0, Tree t1)
@@ -213,7 +214,7 @@ bool isSigDocAccessTbl(Tree t, Tree& tbl, Tree& ridx)
 
 // Select on signal among severals
 
-Tree sigSelect2(Tree selector, Tree s1, Tree s2)
+EXPORT Tree sigSelect2(Tree selector, Tree s1, Tree s2)
 {
     return tree(gGlobal->SIGSELECT2, sigIntCast(selector), s1, s2);
 }
@@ -222,20 +223,46 @@ bool isSigSelect2(Tree t, Tree& selector, Tree& s1, Tree& s2)
     return isTree(t, gGlobal->SIGSELECT2, selector, s1, s2);
 }
 
-//  "select3" expresses with "select2"
-Tree sigSelect3(Tree selector, Tree s1, Tree s2, Tree s3)
+//  "select3" expressed with "select2"
+EXPORT Tree sigSelect3(Tree selector, Tree s1, Tree s2, Tree s3)
 {
     return sigSelect2(sigBinOp(kEQ, sigIntCast(selector), sigInt(0)),
                       sigSelect2(sigBinOp(kEQ, sigIntCast(selector), sigInt(1)), s3, s2), s1);
 }
-bool isSigSelect3(Tree t, Tree& selector, Tree& s1, Tree& s2, Tree& s3)
+
+Tree sigAssertBounds(Tree s1, Tree s2, Tree s3)
 {
-    return isTree(t, gGlobal->SIGSELECT3, selector, s1, s2, s3);
+    return tree(gGlobal->SIGASSERTBOUNDS, s1, s2, s3);
+}
+
+bool isSigAssertBounds(Tree t, Tree& s1, Tree& s2, Tree& s3)
+{
+    return isTree(t, gGlobal->SIGASSERTBOUNDS, s1, s2, s3);
+}
+
+Tree sigHighest(Tree s)
+{
+    return tree(gGlobal->SIGHIGHEST, s);
+}
+
+bool isSigHighest(Tree t, Tree& s)
+{
+    return isTree(t, gGlobal->SIGHIGHEST, s);
+}
+
+Tree sigLowest(Tree s)
+{
+    return tree(gGlobal->SIGLOWEST, s);
+}
+
+bool isSigLowest(Tree t, Tree& s)
+{
+    return isTree(t, gGlobal->SIGLOWEST, s);
 }
 
 // Arithmetical operations
 
-Tree sigBinOp(int op, Tree x, Tree y)
+EXPORT Tree sigBinOp(int op, Tree x, Tree y)
 {
     return tree(gGlobal->SIGBINOP, tree(op), x, y);
 }
@@ -256,7 +283,7 @@ bool isSigFFun(Tree s, Tree& ff, Tree& largs)
     return isTree(s, gGlobal->SIGFFUN, ff, largs);
 }
 
-Tree sigFConst(Tree type, Tree name, Tree file)
+EXPORT Tree sigFConst(Tree type, Tree name, Tree file)
 {
     return tree(gGlobal->SIGFCONST, type, name, file);
 }
@@ -270,7 +297,7 @@ bool isSigFConst(Tree s, Tree& type, Tree& name, Tree& file)
     return isTree(s, gGlobal->SIGFCONST, type, name, file);
 }
 
-Tree sigFVar(Tree type, Tree name, Tree file)
+EXPORT Tree sigFVar(Tree type, Tree name, Tree file)
 {
     return tree(gGlobal->SIGFVAR, type, name, file);
 }
@@ -284,7 +311,7 @@ bool isSigFVar(Tree s, Tree& type, Tree& name, Tree& file)
     return isTree(s, gGlobal->SIGFVAR, type, name, file);
 }
 
-// nouvelle version utilisant rec et ref
+// New version using rec and ref
 
 Tree sigProj(int i, Tree rgroup)
 {
@@ -298,7 +325,7 @@ bool isProj(Tree t, int* i, Tree& rgroup)
 
 // Int and Float casting
 
-Tree sigIntCast(Tree t)
+EXPORT Tree sigIntCast(Tree t)
 {
     Node n = t->node();
 
@@ -311,7 +338,7 @@ Tree sigIntCast(Tree t)
     return tree(gGlobal->SIGINTCAST, t);
 }
 
-Tree sigFloatCast(Tree t)
+EXPORT Tree sigFloatCast(Tree t)
 {
     // cerr << "sigFloatCast(" << ppsig(t) << ")" << endl;
     Node n = t->node();
@@ -326,7 +353,7 @@ Tree sigFloatCast(Tree t)
     return tree(gGlobal->SIGFLOATCAST, t);
 }
 
-// Tree  sigFloatCast(Tree t)						{ return isSigFloatCast(t)? t : tree(gGlobal->SIGFLOATCAST, t); }
+// Tree sigFloatCast(Tree t) { return isSigFloatCast(t)? t : tree(gGlobal->SIGFLOATCAST, t); }
 
 bool isSigIntCast(Tree t)
 {
@@ -348,11 +375,81 @@ bool isSigFloatCast(Tree t, Tree& x)
     return isTree(t, gGlobal->SIGFLOATCAST, x);
 }
 
+// Emulation of all fonctions
+EXPORT Tree sigAdd(Tree x, Tree y)
+{
+    return sigBinOp(kAdd, x, y);
+}
+EXPORT Tree sigSub(Tree x, Tree y)
+{
+    return sigBinOp(kSub, x, y);
+}
+EXPORT Tree sigMul(Tree x, Tree y)
+{
+    return sigBinOp(kMul, x, y);
+}
+EXPORT Tree sigDiv(Tree x, Tree y)
+{
+    return sigBinOp(kDiv, x, y);
+}
+Tree sigRem(Tree x, Tree y);
+
+EXPORT Tree sigAND(Tree x, Tree y)
+{
+    return sigBinOp(kAND, x, y);
+}
+EXPORT Tree sigOR(Tree x, Tree y)
+{
+    return sigBinOp(kOR, x, y);
+}
+EXPORT Tree sigXOR(Tree x, Tree y)
+{
+    return sigBinOp(kXOR, x, y);
+}
+
+EXPORT Tree sigLeftShift(Tree x, Tree y)
+{
+    return sigBinOp(kLsh, x, y);
+}
+EXPORT Tree sigARightShift(Tree x, Tree y)
+{
+    return sigBinOp(kARsh, x, y);
+}
+EXPORT Tree sigLRightShift(Tree x, Tree y)
+{
+    return sigBinOp(kLRsh, x, y);
+}
+
+EXPORT Tree sigGT(Tree x, Tree y)
+{
+    return sigBinOp(kGT, x, y);
+}
+EXPORT Tree sigLT(Tree x, Tree y)
+{
+    return sigBinOp(kLT, x, y);
+}
+EXPORT Tree sigGE(Tree x, Tree y)
+{
+    return sigBinOp(kGE, x, y);
+}
+EXPORT Tree sigLE(Tree x, Tree y)
+{
+    return sigBinOp(kLE, x, y);
+}
+EXPORT Tree sigEQ(Tree x, Tree y)
+{
+    return sigBinOp(kEQ, x, y);
+}
+EXPORT Tree sigNE(Tree x, Tree y)
+{
+    return sigBinOp(kNE, x, y);
+}
+
 /*****************************************************************************
                              User Interface Elements
 *****************************************************************************/
 
-Tree sigButton(Tree lbl)
+EXPORT Tree sigButton(Tree lbl)
 {
     return tree(gGlobal->SIGBUTTON, lbl);
 }
@@ -366,7 +463,7 @@ bool isSigButton(Tree s, Tree& lbl)
     return isTree(s, gGlobal->SIGBUTTON, lbl);
 }
 
-Tree sigCheckbox(Tree lbl)
+EXPORT Tree sigCheckbox(Tree lbl)
 {
     return tree(gGlobal->SIGCHECKBOX, lbl);
 }
@@ -380,7 +477,7 @@ bool isSigCheckbox(Tree s, Tree& lbl)
     return isTree(s, gGlobal->SIGCHECKBOX, lbl);
 }
 
-Tree sigWaveform(const tvec& wf)
+EXPORT Tree sigWaveform(const tvec& wf)
 {
     return tree(gGlobal->SIGWAVEFORM, wf);
 }
@@ -389,9 +486,9 @@ bool isSigWaveform(Tree s)
     return isTree(s, gGlobal->SIGWAVEFORM);
 }
 
-Tree sigHSlider(Tree lbl, Tree cur, Tree min, Tree max, Tree step)
+EXPORT Tree sigHSlider(Tree lbl, Tree init, Tree min, Tree max, Tree step)
 {
-    return tree(gGlobal->SIGHSLIDER, lbl, list4(cur, min, max, step));
+    return tree(gGlobal->SIGHSLIDER, lbl, list4(init, min, max, step));
 }
 bool isSigHSlider(Tree s)
 {
@@ -399,11 +496,11 @@ bool isSigHSlider(Tree s)
     return isTree(s, gGlobal->SIGHSLIDER, lbl, params);
 }
 
-bool isSigHSlider(Tree s, Tree& lbl, Tree& cur, Tree& min, Tree& max, Tree& step)
+bool isSigHSlider(Tree s, Tree& lbl, Tree& init, Tree& min, Tree& max, Tree& step)
 {
     Tree params;
     if (isTree(s, gGlobal->SIGHSLIDER, lbl, params)) {
-        cur  = nth(params, 0);
+        init  = nth(params, 0);
         min  = nth(params, 1);
         max  = nth(params, 2);
         step = nth(params, 3);
@@ -413,9 +510,9 @@ bool isSigHSlider(Tree s, Tree& lbl, Tree& cur, Tree& min, Tree& max, Tree& step
     }
 }
 
-Tree sigVSlider(Tree lbl, Tree cur, Tree min, Tree max, Tree step)
+EXPORT Tree sigVSlider(Tree lbl, Tree init, Tree min, Tree max, Tree step)
 {
-    return tree(gGlobal->SIGVSLIDER, lbl, list4(cur, min, max, step));
+    return tree(gGlobal->SIGVSLIDER, lbl, list4(init, min, max, step));
 }
 bool isSigVSlider(Tree s)
 {
@@ -423,11 +520,11 @@ bool isSigVSlider(Tree s)
     return isTree(s, gGlobal->SIGVSLIDER, lbl, params);
 }
 
-bool isSigVSlider(Tree s, Tree& lbl, Tree& cur, Tree& min, Tree& max, Tree& step)
+bool isSigVSlider(Tree s, Tree& lbl, Tree& init, Tree& min, Tree& max, Tree& step)
 {
     Tree params;
     if (isTree(s, gGlobal->SIGVSLIDER, lbl, params)) {
-        cur  = nth(params, 0);
+        init  = nth(params, 0);
         min  = nth(params, 1);
         max  = nth(params, 2);
         step = nth(params, 3);
@@ -437,9 +534,9 @@ bool isSigVSlider(Tree s, Tree& lbl, Tree& cur, Tree& min, Tree& max, Tree& step
     }
 }
 
-Tree sigNumEntry(Tree lbl, Tree cur, Tree min, Tree max, Tree step)
+EXPORT Tree sigNumEntry(Tree lbl, Tree init, Tree min, Tree max, Tree step)
 {
-    return tree(gGlobal->SIGNUMENTRY, lbl, list4(cur, min, max, step));
+    return tree(gGlobal->SIGNUMENTRY, lbl, list4(init, min, max, step));
 }
 bool isSigNumEntry(Tree s)
 {
@@ -447,11 +544,11 @@ bool isSigNumEntry(Tree s)
     return isTree(s, gGlobal->SIGNUMENTRY, lbl, params);
 }
 
-bool isSigNumEntry(Tree s, Tree& lbl, Tree& cur, Tree& min, Tree& max, Tree& step)
+bool isSigNumEntry(Tree s, Tree& lbl, Tree& init, Tree& min, Tree& max, Tree& step)
 {
     Tree params;
     if (isTree(s, gGlobal->SIGNUMENTRY, lbl, params)) {
-        cur  = nth(params, 0);
+        init  = nth(params, 0);
         min  = nth(params, 1);
         max  = nth(params, 2);
         step = nth(params, 3);
@@ -461,9 +558,9 @@ bool isSigNumEntry(Tree s, Tree& lbl, Tree& cur, Tree& min, Tree& max, Tree& ste
     }
 }
 
-// output elements
+// Output elements
 
-Tree sigHBargraph(Tree lbl, Tree min, Tree max, Tree x)
+EXPORT Tree sigHBargraph(Tree lbl, Tree min, Tree max, Tree x)
 {
     return tree(gGlobal->SIGHBARGRAPH, lbl, min, max, x);
 }
@@ -477,7 +574,7 @@ bool isSigHBargraph(Tree s, Tree& lbl, Tree& min, Tree& max, Tree& x)
     return isTree(s, gGlobal->SIGHBARGRAPH, lbl, min, max, x);
 }
 
-Tree sigVBargraph(Tree lbl, Tree min, Tree max, Tree x)
+EXPORT Tree sigVBargraph(Tree lbl, Tree min, Tree max, Tree x)
 {
     return tree(gGlobal->SIGVBARGRAPH, lbl, min, max, x);
 }
@@ -518,6 +615,127 @@ bool isSigControl(Tree t, Tree& t0, Tree& t1)
     return isTree(t, gGlobal->SIGCONTROL, t0, t1);
 }
 
+// Extended math functions
+static Tree sigExtended1(Tree sig, Tree x)
+{
+    tvec args;
+    args.push_back(x);
+    return ((xtended*)getUserData(sig))->computeSigOutput(args);
+}
+
+static Tree sigExtended2(Tree sig, Tree x, Tree y)
+{
+    tvec args;
+    args.push_back(x);
+    args.push_back(y);
+    return ((xtended*)getUserData(sig))->computeSigOutput(args);
+}
+
+EXPORT Tree sigAbs(Tree x)
+{
+    return sigExtended1(gGlobal->gAbsPrim->box(), x);
+}
+
+EXPORT Tree sigAcos(Tree x)
+{
+    return sigExtended1(gGlobal->gAcosPrim->box(), x);
+}
+
+EXPORT Tree sigTan(Tree x)
+{
+    return sigExtended1(gGlobal->gTanPrim->box(), x);
+}
+
+EXPORT Tree sigSqrt(Tree x)
+{
+    return sigExtended1(gGlobal->gSqrtPrim->box(), x);
+}
+
+EXPORT Tree sigSin(Tree x)
+{
+    return sigExtended1(gGlobal->gSinPrim->box(), x);
+}
+
+EXPORT Tree sigRint(Tree x)
+{
+    return sigExtended1(gGlobal->gRintPrim->box(), x);
+}
+
+EXPORT Tree sigRemainder(Tree x, Tree y)
+{
+    return sigExtended2(gGlobal->gRemainderPrim->box(), x, y);
+}
+
+EXPORT Tree sigPow(Tree x, Tree y)
+{
+    return sigExtended2(gGlobal->gPowPrim->box(), x, y);
+}
+
+EXPORT Tree sigMin(Tree x, Tree y)
+{
+    return sigExtended2(gGlobal->gMinPrim->box(), x, y);
+}
+
+EXPORT Tree sigMax(Tree x, Tree y)
+{
+    return sigExtended2(gGlobal->gMaxPrim->box(), x, y);
+}
+
+EXPORT Tree sigLog(Tree x)
+{
+    return sigExtended1(gGlobal->gLogPrim->box(), x);
+}
+
+EXPORT Tree sigLog10(Tree x)
+{
+    return sigExtended1(gGlobal->gLog10Prim->box(), x);
+}
+
+EXPORT Tree sigFmod(Tree x, Tree y)
+{
+    return sigExtended2(gGlobal->gFmodPrim->box(), x, y);
+}
+
+EXPORT Tree sigFloor(Tree x)
+{
+    return sigExtended1(gGlobal->gFloorPrim->box(), x);
+}
+
+EXPORT Tree sigExp(Tree x)
+{
+    return sigExtended1(gGlobal->gExpPrim->box(), x);
+}
+
+EXPORT Tree sigExp10(Tree x)
+{
+    return sigExtended1(gGlobal->gExp10Prim->box(), x);
+}
+
+EXPORT Tree sigCos(Tree x)
+{
+    return sigExtended1(gGlobal->gCosPrim->box(), x);
+}
+
+EXPORT Tree sigCeil(Tree x)
+{
+    return sigExtended1(gGlobal->gCeilPrim->box(), x);
+}
+
+EXPORT Tree sigAtan(Tree x)
+{
+    return sigExtended1(gGlobal->gAtanPrim->box(), x);
+}
+
+EXPORT Tree sigAtan2(Tree x, Tree y)
+{
+    return sigExtended2(gGlobal->gAtan2Prim->box(), x, y);
+}
+
+EXPORT Tree sigAsin(Tree x)
+{
+    return sigExtended1(gGlobal->gAsinPrim->box(), x);
+}
+
 bool sameMagnitude(Tree a, Tree b)
 {
     return sameMagnitude(a->node(), b->node());
@@ -598,27 +816,27 @@ bool isSigDiv(Tree a, Tree& x, Tree& y)
 }
 
 /*****************************************************************************
-                             Sounfiles
+                             Soundfiles
 *****************************************************************************/
 /*
- A boxSounfile(label,c) has 2 inputs and c+3 outputs:
+ A boxSoundfile(label,c) has 2 inputs and c+3 outputs:
  0   sigSoundfileLength(label, part):  the number of frames of the soundfile part (NK)
  1   sigSoundfileRate(label, part): the sampling rate encoded in the file (NK)
  2   sigSoundfileBuffer(label, c, part, ridx): the cth channel content (RK ou RS)
 */
-Tree sigSoundfile(Tree label)
+EXPORT Tree sigSoundfile(Tree label)
 {
     return tree(gGlobal->SIGSOUNDFILE, label);
 }
-Tree sigSoundfileLength(Tree sf, Tree part)
+EXPORT Tree sigSoundfileLength(Tree sf, Tree part)
 {
     return tree(gGlobal->SIGSOUNDFILELENGTH, sf, part);
 }
-Tree sigSoundfileRate(Tree sf, Tree part)
+EXPORT Tree sigSoundfileRate(Tree sf, Tree part)
 {
     return tree(gGlobal->SIGSOUNDFILERATE, sf, part);
 }
-Tree sigSoundfileBuffer(Tree sf, Tree chan, Tree part, Tree ridx)
+EXPORT Tree sigSoundfileBuffer(Tree sf, Tree chan, Tree part, Tree ridx)
 {
     return tree(gGlobal->SIGSOUNDFILEBUFFER, sf, chan, part, ridx);
 }
@@ -717,7 +935,7 @@ Tree sigFTZ(Tree s)
 /*****************************************************************************
  *                          sigList2vectInt
  *****************************************************************************/
-// convert a list of signals (representing numbers) into a vector of ints
+// Convert a list of signals (representing numbers) into a vector of ints
 // the result is true if the conversion was possible
 
 bool sigList2vecInt(Tree ls, vector<int>& v)
@@ -737,4 +955,16 @@ bool sigList2vecInt(Tree ls, vector<int>& v)
         }
     }
     return true;
-}
\ No newline at end of file
+}
+
+/**
+ * Convert an stl vector of signals into a tree list of signals
+ */
+Tree listConvert(const siglist& a)
+{
+    int  n = (int)a.size();
+    Tree t = gGlobal->nil;
+    
+    while (n--) t = cons(a[n], t);
+    return t;
+}
