@@ -49,7 +49,7 @@
 static int infereSigOrder(Tree sig);
 
 /**
- * retrieve the order annotation (between 0 and 3) of a signal.
+ * Retrieve the order annotation (between 0 and 3) of a signal.
  * (compute the order the first time). Orders have the following meanings
  *	0 : numbers
  *	1 : constants
@@ -87,7 +87,6 @@ static int infereSigOrder(Tree sig)
     xtended* xt = (xtended*)getUserData(sig);
     // primitive elements
     if (xt) {
-        // return 3;
         vector<int> args;
         for (int i1 = 0; i1 < sig->arity(); i1++) {
             args.push_back(O(sig->branch(i1)));
@@ -116,7 +115,7 @@ static int infereSigOrder(Tree sig)
     else if (isSigPrefix(sig, s1, s2))
         return 3;
 
-    else if (isSigFixDelay(sig, s1, s2))
+    else if (isSigDelay(sig, s1, s2))
         return 3;
 
     else if (isSigBinOp(sig, &i, s1, s2))
@@ -215,9 +214,6 @@ static int infereSigOrder(Tree sig)
     else if (isSigSelect2(sig, sel, s1, s2))
         return 3;
 
-    else if (isSigSelect3(sig, sel, s1, s2, s3))
-        return 3;
-
     else if (isList(sig)) {
         int r1 = 0;
         while (isList(sig)) {
