@@ -39,20 +39,20 @@
 
 Tree SignalPromotion::transformation(Tree sig)
 {
-    int  i;
-    Tree sel, x, y, z;
+    int  op;
+    Tree sel, x, y;
 
-    if (isSigFixDelay(sig, x, y)) {
-        return sigFixDelay(self(x), smartIntCast(getCertifiedSigType(y), self(y)));
+    if (isSigDelay(sig, x, y)) {
+        return sigDelay(self(x), smartIntCast(getCertifiedSigType(y), self(y)));
     }
 
     // Binary operations
-    // kAdd, kSub, kMul, kDiv, kRem, kLsh, kRsh, kGT, kLT, kGE, kLE, kEQ, kNE, kAND, kOR, kXOR };
-    else if (isSigBinOp(sig, &i, x, y)) {
+    // kAdd, kSub, kMul, kDiv, kRem, kLsh, kARsh, kGT, kLT, kGE, kLE, kEQ, kNE, kAND, kOR, kXOR };
+    else if (isSigBinOp(sig, &op, x, y)) {
         Type tx = getCertifiedSigType(x);
         Type ty = getCertifiedSigType(y);
 
-        switch (i) {
+        switch (op) {
             case kAdd:
             case kSub:
             case kMul:
@@ -64,37 +64,45 @@ Tree SignalPromotion::transformation(Tree sig)
             case kNE:
                 if (tx->nature() == ty->nature()) {
                     // same nature => no promotion needed
-                    return sigBinOp(i, self(x), self(y));
+                    return sigBinOp(op, self(x), self(y));
                 } else {
                     // float promotion needed
-                    return sigBinOp(i, smartFloatCast(tx, self(x)), smartFloatCast(ty, self(y)));
+                    return sigBinOp(op, smartFloatCast(tx, self(x)), smartFloatCast(ty, self(y)));
                 }
 
             case kRem:
                 if (tx->nature() == kInt && ty->nature() == kInt) {
                     // int arguments => no promotion needed
-                    return sigBinOp(i, self(x), self(y));
+                    return sigBinOp(op, self(x), self(y));
                 } else {
                     // float promotion needed, rem (%) replaced by fmod
                     vector<Tree> lsig = {smartFloatCast(tx, self(x)), smartFloatCast(ty, self(y))};
                     return gGlobal->gFmodPrim->computeSigOutput(lsig);
                 }
 
-            case kDiv:
+            case kDiv: {
+                // Done here instead of 'simplify' to be sure the signals are correctly typed.
+                interval i1 = tx->getInterval();
+                interval j1 = ty->getInterval();
+                if (i1.valid & j1.valid && gGlobal->gMathExceptions && j1.haszero()) {
+                    //cerr << "WARNING : potential division by zero (" << i1 << "/" << j1 << ") in " << ppsig(sig) << endl;
+                    cerr << "WARNING : potential division by zero (" << i1 << "/" << j1 << ")" << endl;
+                }
                 // the result of a division is always a float
-                return sigBinOp(i, smartFloatCast(tx, self(x)), smartFloatCast(ty, self(y)));
-
+                return sigBinOp(op, smartFloatCast(tx, self(x)), smartFloatCast(ty, self(y)));
+            }
+                
             case kAND:
             case kOR:
             case kXOR:
             case kLsh:
-            case kRsh:
+            case kARsh:
                 // these operations require integers
-                return sigBinOp(i, smartIntCast(tx, self(x)), smartIntCast(ty, self(y)));
+                return sigBinOp(op, smartIntCast(tx, self(x)), smartIntCast(ty, self(y)));
 
             default:
                 // TODO: no clear rules here
-                return sigBinOp(i, self(x), self(y));
+                return sigBinOp(op, self(x), self(y));
         }
     }
 
@@ -111,20 +119,6 @@ Tree SignalPromotion::transformation(Tree sig)
             // float promotion needed
             return sigSelect2(smartIntCast(ts, self(sel)), smartFloatCast(tx, self(x)), smartFloatCast(ty, self(y)));
         }
-    } else if (isSigSelect3(sig, sel, x, y, z)) {
-        Type ts = getCertifiedSigType(sel);
-        Type tx = getCertifiedSigType(x);
-        Type ty = getCertifiedSigType(y);
-        Type tz = getCertifiedSigType(z);
-
-        if ((tx->nature() == ty->nature()) && (tx->nature() == tz->nature())) {
-            // same nature => no promotion needed
-            return sigSelect3(smartIntCast(ts, self(sel)), self(x), self(y), self(z));
-        } else {
-            // float promotion needed
-            return sigSelect3(smartIntCast(ts, self(sel)), smartFloatCast(tx, self(x)), smartFloatCast(ty, self(y)),
-                              smartFloatCast(tz, self(z)));
-        }
     }
 
     // Int and Float Cast
@@ -143,31 +137,22 @@ Tree SignalPromotion::transformation(Tree sig)
 /*
 ## smartIntCast[S] : adds an intCast(S) only if needed
 
-    smartIntCast[S]     = intCast(S)    when type(S) = float
-    smartIntCast[S]     = S             otherwise
+    smartIntCast[S] = intCast(S) when type(S) = float
+    smartIntCast[S] = S          otherwise
 */
 
 Tree SignalPromotion::smartIntCast(Type t, Tree sig)
 {
-    if (t->nature() == kReal) {
-        return sigIntCast(sig);
-    } else {
-        return sig;
-    }
+    return (t->nature() == kReal) ? sigIntCast(sig) : sig;
 }
 
 /*
 ## smartFloatCast[S] : adds a floatCast(S) only if needed
 
-    smartFloatCast[S]   = floatCast(S)      when type(S) = int
-                        = S                 otherwise
-
+    smartFloatCast[S] = floatCast(S) when type(S) = int
+    smartFloatCast[S] = S            otherwise
 */
 Tree SignalPromotion::smartFloatCast(Type t, Tree sig)
 {
-    if (t->nature() == kInt) {
-        return sigFloatCast(sig);
-    } else {
-        return sig;
-    }
+    return (t->nature() == kInt) ? sigFloatCast(sig) : sig;
 }
