@@ -53,9 +53,17 @@ struct TestMemoryReader : public MemoryReader {
         soundfile->fOffset[part] = offset;
         
         // Audio frames have to be written for each chan
-        for (int sample = 0; sample < SOUND_LENGTH; sample++) {
-            for (int chan = 0; chan < SOUND_CHAN; chan++) {
-                soundfile->fBuffers[chan][offset + sample] = std::sin(part + (2 * M_PI * float(sample)/SOUND_LENGTH));
+       if (soundfile->fIsDouble) {
+            for (int sample = 0; sample < SOUND_LENGTH; sample++) {
+                for (int chan = 0; chan < SOUND_CHAN; chan++) {
+                    static_cast<double**>(soundfile->fBuffers)[chan][offset + sample] = std::sin(part + (2 * M_PI * double(sample)/SOUND_LENGTH));
+                }
+            }
+        } else {
+            for (int sample = 0; sample < SOUND_LENGTH; sample++) {
+                for (int chan = 0; chan < SOUND_CHAN; chan++) {
+                    static_cast<float**>(soundfile->fBuffers)[chan][offset + sample] = std::sin(part + (2 * M_PI * float(sample)/SOUND_LENGTH));
+                }
             }
         }
 
@@ -101,7 +109,7 @@ struct CheckControlUI : public GenericUI {
    
     bool checkDefaults()
     {
-        for (auto& it : fControlZone) {
+        for (const auto& it : fControlZone) {
             if (*it.first != it.second) return false;
         }
         return true;
@@ -109,7 +117,7 @@ struct CheckControlUI : public GenericUI {
     
     void initRandom()
     {
-        for (auto& it : fControlZone) {
+        for (const auto& it : fControlZone) {
             *it.first = 0.123456789;
         }
     }
@@ -162,10 +170,8 @@ static void runPolyDSP(dsp* dsp, int& linenum, int nbsamples, int num_voices = 4
     
     // Soundfile
     TestMemoryReader memory_reader;
-    SoundUI sound_ui("", -1, &memory_reader);
-    DSP->setGroup(false);
+    SoundUI sound_ui("", -1, &memory_reader, (sizeof(FAUSTFLOAT) == sizeof(double)));
     DSP->buildUserInterface(&sound_ui);
-    DSP->setGroup(true);
   
     // Get control and then 'initRandom'
     CheckControlUI controlui;
@@ -253,7 +259,7 @@ static void runDSP(dsp* DSP, const string& file, int& linenum, int nbsamples, bo
     
     // Soundfile
     TestMemoryReader memory_reader;
-    SoundUI sound_ui("", -1, &memory_reader);
+    SoundUI sound_ui("", -1, &memory_reader, (sizeof(FAUSTFLOAT) == sizeof(double)));
     DSP->buildUserInterface(&sound_ui);
     
     // Get control and then 'initRandom'
@@ -327,7 +333,7 @@ static void runDSP(dsp* DSP, const string& file, int& linenum, int nbsamples, bo
     GUI::updateAllGuis();
     
     // print audio frames
-    int i;
+    int i = 0;
     try {
         while (nbsamples > 0) {
             if (run == 0) {
@@ -352,7 +358,7 @@ static void runDSP(dsp* DSP, const string& file, int& linenum, int nbsamples, bo
            
             run++;
             // Print samples
-            for (int i = 0; i < nFrames; i++) {
+            for (i = 0; i < nFrames; i++) {
                 printf("%6d : ", linenum++);
                 for (int c = 0; c < nouts; c++) {
                     FAUSTFLOAT f = normalize(ochan->buffers()[c][i]);
