@@ -23,17 +23,19 @@
  
  ************************************************************************/
 
+#include <iostream>
+
 #include "faust/dsp/dsp-optimizer.h"
 #include "faust/misc.h"
 
 using namespace std;
 
 template <typename REAL>
-static void bench(dsp_optimizer<REAL> optimizer, const string& name, bool is_trace)
+static void bench(dsp_optimizer_real<REAL> optimizer, const string& in_filename, bool is_trace)
 {
     pair<double, vector<string> > res = optimizer.findOptimizedParameters();
-    if (is_trace) cout << "Best value for '" << name << "' is : " << res.first << " with ";
-    for (int i = 0; i < res.second.size(); i++) {
+    if (is_trace) cout << "Best value for '" << in_filename << "' is : " << res.first << " MBytes/sec with ";
+    for (size_t i = 0; i < res.second.size(); i++) {
         cout << res.second[i] << " ";
     }
     cout << endl;
@@ -42,10 +44,10 @@ static void bench(dsp_optimizer<REAL> optimizer, const string& name, bool is_tra
 template <typename REAL>
 static void bench_single(const string& in_filename, dsp* DSP, int buffer_size, int run, bool is_control, bool is_trace)
 {
-    measure_dsp_aux<REAL> mes(DSP, buffer_size, 5., true, is_control);  // Buffer_size and duration in sec of measure
+    measure_dsp_real<REAL> mes(DSP, buffer_size, 5., true, is_control);  // Buffer_size and duration in sec of measure
     for (int i = 0; i < run; i++) {
         mes.measure();
-        if (is_trace) cout << in_filename << " : " << mes.getStats() << " " << "(DSP CPU % : " << (mes.getCPULoad() * 100) << ")" << endl;
+        if (is_trace) cout << in_filename << " : " << mes.getStats() << " MBytes/sec (DSP CPU % : " << (mes.getCPULoad() * 100) << " at 44100 Hz)" << endl;
         FAUSTBENCH_LOG<REAL>(mes.getStats());
     }
 }
@@ -62,7 +64,7 @@ static void splitTarget(const string& target, string& triple, string& cpu)
 int main(int argc, char* argv[])
 {
     if (argc == 1 || isopt(argv, "-h") || isopt(argv, "-help")) {
-        cout << "faustbench-llvm [-notrace] [-control] [-generic] [-single] [-run <num>] [-bs <frames>] [-opt <level (0..4|-1)>] [additional Faust options (-vec -vs 8...)] foo.dsp" << endl;
+        cout << "faustbench-llvm [-notrace] [-control] [-generic] [-single] [-run <num>] [-bs <frames>] [-opt <level (0..4|-1)>] [-us <factor>] [-ds <factor>] [-filter <filter(0..4)>] [additional Faust options (-vec -vs 8...)] foo.dsp" << endl;
         cout << "Use '-notrace' to only generate the best compilation parameters\n";
         cout << "Use '-control' to update all controllers with random values at each cycle\n";
         cout << "Use '-generic' to compile for a generic processor, otherwise the native CPU will be used\n";
@@ -70,6 +72,9 @@ int main(int argc, char* argv[])
         cout << "Use '-run <num>' to execute each test <num> times\n";
         cout << "Use '-bs <frames>' to set the buffer-size in frames\n";
         cout << "Use '-opt <level (0..4|-1)>' to pass an optimisation level to LLVM, between 0 and 4 (-1 means 'maximal level' if range changes in the future)\n";
+        cout << "Use '-us <factor>' to upsample the DSP by a factor\n";
+        cout << "Use '-ds <factor>' to downsample the DSP by a factor\n";
+        cout << "Use '-filter <filter>' for upsampling or downsampling [0..4]\n";
         return 0;
     }
     
@@ -81,8 +86,11 @@ int main(int argc, char* argv[])
     int run = lopt(argv, "-run", 1);
     int buffer_size = lopt(argv, "-bs", 512);
     int opt = lopt(argv, "-opt", -1);
+    int ds = lopt(argv, "-ds", 0);
+    int us = lopt(argv, "-us", 0);
+    int filter = lopt(argv, "-filter", 0);
     
-    if (is_trace) cout << "Libfaust version : " << getCLibFaustVersion () << endl;
+    if (is_trace) cout << "Libfaust version : " << getCLibFaustVersion() << endl;
     
     // Possibly activate 'generic' generation mode
     string target;
@@ -108,7 +116,10 @@ int main(int argc, char* argv[])
             continue;
         } else if (string(argv[i]) == "-run"
                    || string(argv[i]) == "-opt"
-                   || string(argv[i]) == "-bs") {
+                   || string(argv[i]) == "-bs"
+                   || string(argv[i]) == "-ds"
+                   || string(argv[i]) == "-us"
+                   || string(argv[i]) == "-filter") {
             i++;
             continue;
         }
@@ -142,16 +153,32 @@ int main(int argc, char* argv[])
             }
             
             if (is_double) {
-                bench_single<double>(in_filename,DSP, buffer_size, run, is_control, is_trace);
+                bench_single<double>(in_filename, DSP, buffer_size, run, is_control, is_trace);
             } else {
-                bench_single<float>(in_filename,DSP, buffer_size, run, is_control, is_trace);
+                bench_single<float>(in_filename, DSP, buffer_size, run, is_control, is_trace);
             }
             
         } else {
             if (is_double) {
-                bench(dsp_optimizer<double>(in_filename.c_str(), argc1, argv1, target, buffer_size, run, -1, is_trace, is_control), in_filename, is_trace);
+                bench(dsp_optimizer_real<double>(in_filename.c_str(),
+                                                argc1, argv1,
+                                                target, buffer_size,
+                                                run, -1,
+                                                is_trace,
+                                                is_control,
+                                                ds, us, filter),
+                                                in_filename,
+                                                is_trace);
             } else {
-                bench(dsp_optimizer<float>(in_filename.c_str(), argc1, argv1, target, buffer_size, run, -1, is_trace, is_control), in_filename, is_trace);
+                bench(dsp_optimizer_real<float>(in_filename.c_str(),
+                                               argc1, argv1,
+                                               target, buffer_size,
+                                               run, -1,
+                                               is_trace,
+                                               is_control,
+                                               ds, us, filter),
+                                               in_filename,
+                                               is_trace);
             }
         }
     } catch (...) {
