@@ -106,7 +106,6 @@ int main(int argc, char* argv[])
 {
     char name[256];
     char filename[256];
-    char* home = getenv("HOME");
     
     snprintf(name, 255, "%s", basename(argv[0]));
     snprintf(filename, 255, "%s", basename(argv[argc-1]));
@@ -126,13 +125,15 @@ int main(int argc, char* argv[])
         cout << "-trace 1 to collect FP_SUBNORMAL only\n";
         cout << "-trace 2 to collect FP_SUBNORMAL, FP_INFINITE and FP_NAN\n";
         cout << "-trace 3 to collect FP_SUBNORMAL, FP_INFINITE, FP_NAN, INTEGER_OVERFLOW, DIV_BY_ZERO and CAST_INT_OVERFLOW\n";
-        cout << "-trace 4 to collect FP_SUBNORMAL, FP_INFINITE, FP_NAN, INTEGER_OVERFLOW, DIV_BY_ZERO, CAST_INT_OVERFLOW and LOAD/STORE errors, fails at first FP_INFINITE, FP_NAN or LOAD/STORE errors\n";
-        cout << "-trace 5 to collect FP_SUBNORMAL, FP_INFINITE, FP_NAN, INTEGER_OVERFLOW, DIV_BY_ZERO, CAST_INT_OVERFLOW and LOAD/STORE errors, continue after FP_INFINITE, FP_NAN or LOAD/STORE errors\n";
+        cout << "-trace 4 to collect FP_SUBNORMAL, FP_INFINITE, FP_NAN, INTEGER_OVERFLOW, DIV_BY_ZERO, CAST_INT_OVERFLOW and LOAD/STORE errors, fails at first FP_INFINITE, FP_NAN, CAST_INT_OVERFLOW or LOAD/STORE errors\n";
+        cout << "-trace 5 to collect FP_SUBNORMAL, FP_INFINITE, FP_NAN, INTEGER_OVERFLOW, DIV_BY_ZERO, CAST_INT_OVERFLOW and LOAD/STORE errors, continue after FP_INFINITE, FP_NAN, CAST_INT_OVERFLOW or LOAD/STORE errors\n";
         cout << "-trace 6 to only check LOAD/STORE errors and continue\n";
         cout << "-trace 7 to only check LOAD/STORE errors and exit\n";
         exit(EXIT_FAILURE);
     }
-    cout << "Libfaust version : " << getCLibFaustVersion () << endl;
+    
+    cout << "Filename: " << argv[argc-1] << endl;
+    cout << "Libfaust version : " << getCLibFaustVersion() << endl;
     
     int argc1 = 0;
     const char* argv1[64];
@@ -191,23 +192,15 @@ int main(int argc, char* argv[])
         }
         
         cout << "getName " << factory->getName() << endl;
-        dummyaudio_real<float>* dummy_driver_float = nullptr;
-        dummyaudio_real<double>* dummy_driver_double = nullptr;
+        dummyaudio_base* audio = nullptr;
         
         if (isopt(argv, "-double")) {
-            dummy_driver_double = new dummyaudio_real<double>(44100, 16, INT_MAX);
+            audio = new dummyaudio_real<double>(44100, 16, INT_MAX, -1, false, trace_mode == 4);
         } else {
-            dummy_driver_float = new dummyaudio_real<float>(44100, 16, INT_MAX);
+            audio = new dummyaudio_real<float>(44100, 16, INT_MAX, -1, false, trace_mode == 4);
         }
-        
-        if (dummy_driver_float) {
-            if (!dummy_driver_float->init(filename, DSP)) {
-                exit(EXIT_FAILURE);
-            }
-        } else {
-            if (!dummy_driver_double->init(filename, DSP)) {
-                exit(EXIT_FAILURE);
-            }
+        if (!audio->init(filename, DSP)) {
+            exit(EXIT_FAILURE);
         }
         
         if (!is_noui) {
@@ -224,7 +217,7 @@ int main(int argc, char* argv[])
                 
                 cout << "------------------------------" << endl;
                 cout << "Check control min/max for " << ctl.fControlZone.size() << " controls" << endl;
-                for (int index = 0; index < ctl.fControlZone.size(); index++) {
+                for (size_t index = 0; index < ctl.fControlZone.size(); index++) {
                     cout << "------------------------------" << endl;
                     cout << "Control: " << ctl.getParamAddress(ctl.fControlZone[index].first) << endl;
                     FAUSTFLOAT min = ctl.fControlZone[index].second.fMin;
@@ -233,20 +226,12 @@ int main(int argc, char* argv[])
                     // Test min
                     cout << "Min: " << min << endl;
                     *ctl.fControlZone[index].first = min;
-                    if (dummy_driver_float) {
-                        dummy_driver_float->render();
-                    } else {
-                        dummy_driver_double->render();
-                    }
+                    audio->render();
                     *ctl.fControlZone[index].first = init; // reset to init
                     // Test max
                     cout << "Max: " << max << endl;
                     *ctl.fControlZone[index].first = max;
-                    if (dummy_driver_float) {
-                        dummy_driver_float->render();
-                    } else {
-                        dummy_driver_double->render();
-                    }
+                    audio->render();
                     *ctl.fControlZone[index].first = init; // reset to init
                 }
             }
@@ -258,28 +243,19 @@ int main(int argc, char* argv[])
                 
                 cout << "------------------------------" << endl;
                 cout << "Check control min/max successively keeping min for " << ctl.fControlZone.size() << " controls" << endl;
-                for (int index = 0; index < ctl.fControlZone.size(); index++) {
+                for (size_t index = 0; index < ctl.fControlZone.size(); index++) {
                     cout << "------------------------------" << endl;
                     cout << "Control: " << ctl.getParamAddress(ctl.fControlZone[index].first) << endl;
                     FAUSTFLOAT min = ctl.fControlZone[index].second.fMin;
                     FAUSTFLOAT max = ctl.fControlZone[index].second.fMax;
-                    FAUSTFLOAT init = ctl.fControlZone[index].second.fInit;
                     // Test max
                     cout << "Max: " << max << endl;
                     *ctl.fControlZone[index].first = max;
-                    if (dummy_driver_float) {
-                        dummy_driver_float->render();
-                    } else {
-                        dummy_driver_double->render();
-                    }
+                    audio->render();
                     // Test min
                     cout << "Min: " << min << endl;
                     *ctl.fControlZone[index].first = min;
-                    if (dummy_driver_float) {
-                        dummy_driver_float->render();
-                    } else {
-                        dummy_driver_double->render();
-                    }
+                    audio->render();
                 }
             }
             
@@ -290,27 +266,38 @@ int main(int argc, char* argv[])
                 
                 cout << "------------------------------" << endl;
                 cout << "Check control min/max successively, keeping max for " << ctl.fControlZone.size() << " controls" << endl;
-                for (int index = 0; index < ctl.fControlZone.size(); index++) {
+                for (size_t index = 0; index < ctl.fControlZone.size(); index++) {
                     cout << "------------------------------" << endl;
                     cout << "Control: " << ctl.getParamAddress(ctl.fControlZone[index].first) << endl;
                     FAUSTFLOAT min = ctl.fControlZone[index].second.fMin;
                     FAUSTFLOAT max = ctl.fControlZone[index].second.fMax;
-                    FAUSTFLOAT init = ctl.fControlZone[index].second.fInit;
                     // Test min
                     cout << "Min: " << min << endl;
                     *ctl.fControlZone[index].first = min;
-                    if (dummy_driver_float) {
-                        dummy_driver_float->render();
-                    } else {
-                        dummy_driver_double->render();
-                    }
+                    audio->render();
                     // Test max
                     cout << "Max: " << max << endl;
                     *ctl.fControlZone[index].first = max;
-                    if (dummy_driver_float) {
-                        dummy_driver_float->render();
-                    } else {
-                        dummy_driver_double->render();
+                    audio->render();
+                }
+            }
+            
+            // Check by setting each control to zero if contained in the [min..max] range
+            {
+                CheckControlUI ctl;
+                DSP->buildUserInterface(&ctl);
+                
+                cout << "------------------------------" << endl;
+                cout << "Check control to zero if contained in the [min..max] range" << endl;
+                for (size_t index = 0; index < ctl.fControlZone.size(); index++) {
+                    FAUSTFLOAT min = ctl.fControlZone[index].second.fMin;
+                    FAUSTFLOAT max = ctl.fControlZone[index].second.fMax;
+                    if (min < 0 && max > 0) {
+                        cout << "------------------------------" << endl;
+                        cout << "Control: " << ctl.getParamAddress(ctl.fControlZone[index].first) << endl;
+                        // Test zero
+                        *ctl.fControlZone[index].first = 0;
+                        audio->render();
                     }
                 }
             }
@@ -319,24 +306,16 @@ int main(int argc, char* argv[])
             DSP->buildUserInterface(&random);
             cout << "------------------------------" << endl;
             cout << "Use RandomControlUI" << endl;
-            for (int step = 0; step < 1000; step++) {
-                cout << "Set random controllers, step: " << step <<  " until: " << 1000 << endl;
+            for (int step = 0; step < 10; step++) {
+                cout << "Set random controllers, step: " << step <<  " until: " << 10 << endl;
                 random.update();
-                if (dummy_driver_float) {
-                    dummy_driver_float->render();
-                } else {
-                    dummy_driver_double->render();
-                }
+                audio->render();
             }
             
             goto end;
             
         } else {
-            if (dummy_driver_float) {
-                dummy_driver_float->start();
-            } else {
-                dummy_driver_double->start();
-            }
+            audio->start();
         }
         
         if (!is_noui) {
@@ -346,14 +325,8 @@ int main(int argc, char* argv[])
             usleep(time_out * 1e6);
         }
         
-        if (dummy_driver_float) {
-            dummy_driver_float->stop();
-        } else {
-            dummy_driver_double->stop();
-        }
-        
-        delete dummy_driver_float;
-        delete dummy_driver_double;
+        audio->stop();
+        delete audio;
         
     } catch (...) {
         cout << endl;
